/////////////////////////////////////////////////////////////////////////////
// DsDrv.cpp
/////////////////////////////////////////////////////////////////////////////
// Copyright (c) 2002
/////////////////////////////////////////////////////////////////////////////
//
//  This file is subject to the terms of the GNU General Public License as
//  published by the Free Software Foundation.  A copy of this license is
//  included with this software distribution in the file COPYING.  If you
//  do not have a copy, you may obtain a copy by writing to the Free
//  Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
//
//  This software is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details
/////////////////////////////////////////////////////////////////////////////
//
// PCICard.cpp, HardwareDriver.cpp, HardwareMemory.cpp are from Dscaler
// Copyright (c) 2001 John Adcock.  All rights reserved.
/////////////////////////////////////////////////////////////////////////////

// What's this ?
// Helper DLL to access DsDrv v4 drivers from software written for Dsdrv v3 
// It's "written" by me (E-nek) and it is a very ugly !
// Many Thanks to Dscaler team for their drivers.
//
// General Notes :
// - I have made few modifications, search for "E-nek".
// - Not all functions are implemented (like readPort*/WritePort* ...)
//
// Return codes :
// You have similar error codes for (obviously not all error codes)
// pciGetHardwareResources, LoadDriver and DoesThisPCICardExist :
//
// 0:  All is OK
// 1: "Can't load driver"		(Another app is already running with DsDrv ? or files missing ?)
// 2: "Can't find this card"
// 3: "Can't lock this card"	(memorymap failed ?)
//
// Changelog :
//
// 28/5/2002: v0.4
// - Updated HardwareDriver.cpp from latest Dscaler source
//
// 25/5/2002: v0.3
// - Added support for Win95 (Thanks to Tomzo)
//
// 29/3/2002: v0.2
// - Added VERSION resource
// - New functions :
//   LoadDriver, UnloadDriver, DoesThisPCICardExist, InstallNTDriver, UnInstallNTDriver
//   memoryMap & memoryUnMap have been suppressed
// - pciGetHardwareResources & DoesThisPCICardExist have more explicit
//   error codes.

// 24/3/2002: v0.1 (First release)
// - Fix crash when accesssing Pcicard and Pcicard was NULL ;-)
// - pciGetHardwareResources has a new parameter
// - memoryAlloc/memoryFree is using code from DSDrv v3


#include "stdafx.h"
#include <malloc.h>

#include "PCICard.h"
#include "HardwareDriver.h"


// Export functions as C
#define DSDRV_API extern "C" __declspec(dllexport)


CPCICard		*Pcicard = NULL;
CHardwareDriver	*HardwareDriver = NULL;
BOOL			CardOpened = FALSE;		// BT Card has been opened ?



BOOL APIENTRY DllMain( HANDLE hModule, DWORD  ul_reason_for_call, LPVOID lpReserved)
{
    switch (ul_reason_for_call)
	{
		case DLL_PROCESS_ATTACH:
		case DLL_PROCESS_DETACH:
			break;
    }
    return TRUE;
}


DSDRV_API DWORD WINAPI LoadDriver()
{
	HardwareDriver = new CHardwareDriver();
	if( !HardwareDriver->LoadDriver()) 
	{
		delete HardwareDriver;
		HardwareDriver = NULL;
		return 1;
	}
	Pcicard = new CPCICard(HardwareDriver);
	return ERROR_SUCCESS;
}


DSDRV_API void WINAPI UnloadDriver()
{
	if (Pcicard != NULL) {
		delete Pcicard;
		Pcicard = NULL;
	}
	
	if (HardwareDriver != NULL)
	{
		HardwareDriver->UnloadDriver();
		delete HardwareDriver;
		HardwareDriver = NULL;
	}
}


// It seems to be safe to call InstallNTDriver and quit immediately your application.
// The next launch will give you a clean driver.
DSDRV_API DWORD WINAPI InstallNTDriver()
{
BOOL ret;

	HardwareDriver = new CHardwareDriver();
	if (HardwareDriver == NULL)
		return 1;

	ret = HardwareDriver->InstallNTDriver();
	delete HardwareDriver;

	return (ret ? ERROR_SUCCESS: 2);
}


DSDRV_API DWORD WINAPI UnInstallNTDriver()
{
BOOL ret;

	HardwareDriver = new CHardwareDriver();
	if (HardwareDriver == NULL)
		return 1;

	ret = HardwareDriver->UnInstallNTDriver();
	delete HardwareDriver;

	return (ret ? ERROR_SUCCESS: 2);
}


DSDRV_API DWORD WINAPI 
DoesThisPCICardExist(DWORD dwVendorID, DWORD dwDeviceID, DWORD dwCardIndex)
{
BOOL ret;
DWORD dwSubSystemId;

	if ( !HardwareDriver || !Pcicard )
		return 1;

	ret = HardwareDriver->DoesThisPCICardExist( (unsigned short)dwVendorID, (unsigned short)dwDeviceID, dwCardIndex, dwSubSystemId);
	if ( !ret )
		return 2;

	return ERROR_SUCCESS;
}



// Note :
// New parameter added : dwCardIndex
DSDRV_API DWORD WINAPI pciGetHardwareResources(DWORD   dwVendorID,
                                      DWORD   dwDeviceID,
									  DWORD   dwCardIndex,
                                      PDWORD  pdwMemoryAddress,
                                      PDWORD  pdwMemoryLength,
                                      PDWORD  pdwSubSystemId)
{
BOOL ret;
DWORD dwSubSystemId;

	if ( !HardwareDriver || !Pcicard )
		return 1;

	ret = HardwareDriver->DoesThisPCICardExist( (unsigned short)dwVendorID, (unsigned short)dwDeviceID, dwCardIndex, dwSubSystemId);
	if ( !ret )
		return 2;

	ret = Pcicard->OpenPCICard((unsigned short)dwVendorID, (unsigned short)dwDeviceID, dwCardIndex);
	if ( !ret )
		return 3;

	*pdwSubSystemId  = Pcicard->GetSubSystemId();
	*pdwMemoryLength = Pcicard->GetMemoryLength();
	*pdwSubSystemId  = Pcicard->GetMemoryAddress();
	CardOpened = TRUE;

	return ERROR_SUCCESS;
}


DSDRV_API void WINAPI memoryWriteBYTE(DWORD Offset, BYTE Data)  
{ 
	if (Pcicard == NULL || !CardOpened) return;
	Pcicard->WriteByte(Offset, Data); 
}
DSDRV_API void WINAPI memoryWriteWORD(DWORD Offset, WORD Data)  
{ 
	if (Pcicard == NULL || !CardOpened) return;
	Pcicard->WriteWord(Offset, Data); 
}
DSDRV_API void WINAPI memoryWriteDWORD(DWORD Offset, DWORD Data)
{ 
	if (Pcicard == NULL || !CardOpened) return;
	Pcicard->WriteDword(Offset, Data); 
}

DSDRV_API BYTE WINAPI memoryReadBYTE(DWORD Offset)		
{ 
	if (Pcicard == NULL || !CardOpened) return 0;
	return Pcicard->ReadByte(Offset); 
}
DSDRV_API WORD WINAPI memoryReadWORD(DWORD Offset)		
{ 
	if (Pcicard == NULL || !CardOpened) return 0;
	return Pcicard->ReadWord(Offset); 
}
DSDRV_API DWORD WINAPI memoryReadDWORD(DWORD Offset)	
{ 
	if (Pcicard == NULL || !CardOpened) return 0;
	return Pcicard->ReadDword(Offset); 
}

// Functions From DSDrv v3.11
//
// We should use CContigMemory & CUserMemory (newer code) but that's too complicated.
DSDRV_API DWORD WINAPI memoryAlloc(DWORD dwLength, DWORD dwFlags, PMemStruct* ppMemStruct)
{
    TDSDrvParam paramIn;
    DWORD dwReturnedLength;
    DWORD status;
    DWORD nPages = 0;

    if(dwFlags & ALLOC_MEMORY_CONTIG)
    {
        nPages = 1;
    }
    else
    {
        nPages = (dwLength + 4095) / 4096 + 1;
    }
    
    DWORD dwOutParamLength = sizeof(TMemStruct) + nPages * sizeof(TPageStruct);
    *ppMemStruct = (PMemStruct) malloc(dwOutParamLength);

    paramIn.dwValue = dwLength;
    paramIn.dwFlags = dwFlags;
    if(dwFlags & ALLOC_MEMORY_CONTIG)
    {
        paramIn.dwAddress = 0;
    }
    else
    {
        paramIn.dwAddress = (ULONG)malloc(dwLength);
        memset((void*)paramIn.dwAddress, 0, dwLength);
        if(paramIn.dwAddress == NULL)
        {
            free(*ppMemStruct);
            return ERROR_NOT_ENOUGH_MEMORY;
        }
    }
    status = HardwareDriver->SendCommand(ioctlAllocMemory,
                            &paramIn,
                            sizeof(paramIn),
                            *ppMemStruct,
                            dwOutParamLength,
                            &dwReturnedLength);

    if(status != ERROR_SUCCESS || ppMemStruct == NULL || (*ppMemStruct)->dwUser == 0)
    {
        if(!(dwFlags & ALLOC_MEMORY_CONTIG))
        {
            free((void*)paramIn.dwAddress);
        }
        free(*ppMemStruct);
        *ppMemStruct = NULL;
    }

    return status;
}


DSDRV_API DWORD WINAPI memoryFree(PMemStruct pMemStruct)
{
    DWORD status = ERROR_SUCCESS;
    if(pMemStruct != NULL)
    {
        DWORD dwInParamLength = sizeof(TMemStruct) + pMemStruct->dwPages * sizeof(TPageStruct);
        status = HardwareDriver->SendCommand(ioctlFreeMemory, pMemStruct, dwInParamLength);
        if(!(pMemStruct->dwFlags & ALLOC_MEMORY_CONTIG))
        {
            free(pMemStruct->dwUser);
        }
        free(pMemStruct);
    }
    return status;
}

/*
//
// These functions are useless so they are not implemented ...
//

int WINAPI isDriverOpened (void);
BYTE WINAPI readPort(WORD address);
WORD WINAPI readPortW(WORD address);
DWORD WINAPI readPortL(WORD address);
void WINAPI writePort(WORD address, BYTE bValue);
void WINAPI writePortW(WORD address, WORD uValue);
void WINAPI writePortL(WORD address, DWORD dwValue);
DWORD WINAPI memoryMap  (DWORD dwAddress, DWORD dwLength) 
void WINAPI memoryUnmap(DWORD dwAddress, DWORD dwLength) 
*/
