/////////////////////////////////////////////////////////////////////////////
// btaudio.c
/////////////////////////////////////////////////////////////////////////////
// Copyright (c) 2000-2002 Quenotte  All rights reserved.
/////////////////////////////////////////////////////////////////////////////
//
//	This file is subject to the terms of the GNU General Public License as
//	published by the Free Software Foundation.  A copy of this license is
//	included with this software distribution in the file COPYING.  If you
//	do not have a copy, you may obtain a copy by writing to the Free
//	Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
//
//	This software is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details
/////////////////////////////////////////////////////////////////////////////
//
// This software was based on MultiDec 5.9 Those portions are
// Copyright (C) 1999/2000 Espresso (echter_espresso@hotmail.com)
//
/////////////////////////////////////////////////////////////////////////////
#include "stdafx.h"
#include "resource.h"
#include "..\Plugins\ResCommun.h"

#include "bt848.h"
#include "btaudio.h"
#include "K!TV-XP.h"
#include "bt_io.h"

// MSP34x0 definitions
#define MSP_CONTROL 0x00 // Software reset
#define MSP_TEST	0x01 // Internal use
#define MSP_WR_DEM	0x10 // Write demodulator
#define MSP_RD_DEM	0x11 // Read demodulator
#define MSP_WR_DSP	0x12 // Write DSP
#define MSP_RD_DSP	0x13 // Read DSP

#define MSP_MODE_AM_DETECT   0
#define MSP_MODE_FM_RADIO    2
#define MSP_MODE_FM_TERRA    3
#define MSP_MODE_FM_SAT      4
#define MSP_MODE_FM_NICAM1   5
#define MSP_MODE_FM_NICAM2   6
#define MSP_MODE_AM_NICAM    7
#define VIDEO_SOUND_MONO	1
#define VIDEO_SOUND_STEREO	2
#define VIDEO_SOUND_LANG1	3
#define VIDEO_SOUND_LANG2	4

#define WriteDem(wAddr,wData) Audio_MSP_Write(MSP_WR_DEM,wAddr,wData) // I2C_MSP3400C_DEM
#define WriteDSP(wAddr,wData) Audio_MSP_Write(MSP_WR_DSP,wAddr,wData) // I2C_MSP3400C_DEM
#define ReadDem(wAddr) Audio_MSP_Read(MSP_RD_DEM,wAddr)  // I2C_MSP3400C_DFP
#define ReadDSP(wAddr) Audio_MSP_Read(MSP_RD_DSP,wAddr)  // I2C_MSP3400C_DFP


// This macro is allowed for *constants* only, gcc must calculate it
// at compile time.  Remember -- no floats in kernel mode
#define MSP_CARRIER(freq) ((int)((float)(freq/18.432)*(1<<24)))


BOOL MSPNicam;
char MSPVersion[16];
int MSPMode=7;
int MSPStereo;
int MSPMajorMode=3;
int MSPMinorMode=2;
char MSPStatus[30];

static BYTE AudioDeviceWrite, AudioDeviceRead;
static BOOL MSPToneControl=FALSE;

BOOL AutoStereoSelect=TRUE;
BOOL InitialSuperBass=FALSE;
char InitialEqualizer1=0x00;
char InitialEqualizer2=0x00;
char InitialEqualizer3=0x00;
char InitialEqualizer4=0x00;
char InitialEqualizer5=0x00;

char InitialBalance=0, InitialSpatial=0, InitialLoudness=0;
char InitialBass=0,    InitialTreble=0,  InitialNicam=0x70, InitialAM=0x10;



static int carrier_detect_main[4] =
{
	// main carrier
	 MSP_CARRIER(4.5),   // 4.5   NTSC                 
	 MSP_CARRIER(5.5),   // 5.5   PAL B/G              
	 MSP_CARRIER(6.0),   // 6.0   PAL I                
	 MSP_CARRIER(6.5),   // 6.5   PAL D/K + SAT + SECAM
};

static int carrier_detect[8] =
{
	// PAL B/G
	 MSP_CARRIER(5.7421875), // 5.742 PAL B/G FM-stereo
	 MSP_CARRIER(5.85),      // 5.85  PAL B/G NICAM
	// PAL SAT / SECAM
	 MSP_CARRIER(5.85),      // 5.85  PAL D/K / Secam NICAM
	 MSP_CARRIER(6.2578125), //6.25  PAL D/K1 FM-stereo
	 MSP_CARRIER(6.7421875), //6.74  PAL D/K2 FM-stereo
	 MSP_CARRIER(7.02),      //7.02  PAL SAT FM-stereo s/b
	 MSP_CARRIER(7.20),      //7.20  PAL SAT FM-stereo s
	 MSP_CARRIER(7.38),      //7.38  PAL SAT FM-stereo b
};

struct MSP_INIT_DATA_DEM
{
	int fir1[6];
	int fir2[6];
	int cdo1;
	int cdo2;
	int ad_cv;
	int mode_reg;
	int dfp_src;
	int dfp_matrix;
};

static struct MSP_INIT_DATA_DEM msp_init_data[] =
{
	// AM (for carrier detect / msp3400)
	{ { 75, 19, 36, 35, 39, 40 }, { 75, 19, 36, 35, 39, 40 },
	  MSP_CARRIER(5.5), MSP_CARRIER(5.5),
	  0x00d0, 0x0500,   0x0020, 0x3000},

	// AM (for carrier detect / msp3410)
	{ { -1, -1, -8, 2, 59, 126 }, { -1, -1, -8, 2, 59, 126 },
	  MSP_CARRIER(5.5), MSP_CARRIER(5.5),
	  0x00d0, 0x0100,   0x0020, 0x3000},

	// FM Radio */
	{ { -8, -8, 4, 6, 78, 107 }, { -8, -8, 4, 6, 78, 107 },
	  MSP_CARRIER(10.7), MSP_CARRIER(10.7),
	  0x00d0, 0x0480, 0x0020, 0x3000 },

	// Terrestial FM-mono + FM-stereo
	{ {  3, 18, 27, 48, 66, 72 }, {  3, 18, 27, 48, 66, 72 },
	  MSP_CARRIER(5.5), MSP_CARRIER(5.5),
	  0x00d0, 0x0480,   0x0030, 0x3000},

	// Sat FM-mono
	{ {  1,  9, 14, 24, 33, 37 }, {  3, 18, 27, 48, 66, 72 },
	  MSP_CARRIER(6.5), MSP_CARRIER(6.5),
	  0x00c6, 0x0480,   0x0000, 0x3000},

	// NICAM B/G, D/K
	{ { -2, -8, -10, 10, 50, 86 }, {  3, 18, 27, 48, 66, 72 },
	  MSP_CARRIER(5.5), MSP_CARRIER(5.5),
	  0x00d0, 0x0040,   0x0120, 0x3000},

	// NICAM I
	{ {  2, 4, -6, -4, 40, 94 }, {  3, 18, 27, 48, 66, 72 },
	  MSP_CARRIER(6.0), MSP_CARRIER(6.0),
	  0x00d0, 0x0040,   0x0120, 0x3000},

	// NICAM/AM mono Secam L
	 { {  -2, -8, -10, 10, 50, 86 }, {  -4, -12, -9, 23, 79, 126 },
	  MSP_CARRIER(6.5), MSP_CARRIER(6.5),
	  0x00c6, 0xA170, 0x0200, 0x7000},
};


static BOOL Audio_MSP_Write(BYTE bSubAddr, int wAddr, int wData)
{
	I2CBus_Lock();
    I2CBus_Start();
    I2CBus_SendByte(AudioDeviceWrite,0);
    I2CBus_SendByte(bSubAddr,0);
    if(bSubAddr!=MSP_CONTROL && bSubAddr!=MSP_TEST)
    {
         I2CBus_SendByte((BYTE)(wAddr>>8),0);
         I2CBus_SendByte((BYTE)(wAddr&0xFF),0);
    }
    I2CBus_SendByte((BYTE)(wData>>8),0);
    I2CBus_SendByte((BYTE)(wData&0xFF),0);
    I2CBus_Stop();
	I2CBus_Unlock();
    return TRUE;
}


static WORD Audio_MSP_Read(BYTE bSubAddr, WORD wAddr)
{
     WORD wResult;
     BYTE B0,B1;
     B0=(BYTE)(wAddr>>8);
     B1=(BYTE)(wAddr&0xFF);
	 I2CBus_Lock();
     I2CBus_Start();
     I2CBus_SendByte(AudioDeviceWrite,2);
     I2CBus_SendByte(bSubAddr,0);
     I2CBus_SendByte(B0,0);
     I2CBus_SendByte(B1,0);
     I2CBus_Start();
     if (I2CBus_SendByte(AudioDeviceRead,2))
	 {
		B0=I2CBus_ReadByte(0);
		B1=I2CBus_ReadByte(1);
		wResult=B0 << 8 | B1 ;
	 }
	 else
	 { 
		 wResult =(WORD)(-1);
	 }
     I2CBus_Stop();
	 I2CBus_Unlock();
     return wResult;
}


static BOOL Audio_MSP_Version()
{
	int	rev1,rev2;
    
	MSPNicam=FALSE;

	rev1 = ReadDSP( 0x1e);
	rev2 = ReadDSP( 0x1f);
	if (0 == rev1 && 0 == rev2)
	{
		return (FALSE);
	}

	sprintf(MSPVersion,Langue_GetString(ID_MSPINFO),(rev2>>8)&0xff, (rev1&0xff)+'@', ((rev1>>8)&0xff)+'@', rev2&0x1f);
	MSPNicam = (((rev2>>8)&0xff) != 00) ? 1 : 0;
    return(TRUE);
}


BOOL Audio_Init(BYTE DWrite,BYTE DRead)
{
	AudioDeviceWrite=DWrite;
	AudioDeviceRead=DRead;

	Has_MSP=TRUE;

	if(!I2CBus_AddDevice(DRead))
	{
	    Has_MSP=FALSE;
		return(FALSE);
	}

	if(!I2CBus_AddDevice(DWrite))
	{
	    Has_MSP=FALSE;
		return(FALSE);
	}

	sprintf(MSPStatus, Langue_GetString(ID_MSPVERSION));

	Audio_MSP_Reset();
	Sleep(4);
	Audio_MSP_Version();
	Sleep(4);
	Audio_MSP_SetMode( MSPMode );
	Sleep(4);
	Audio_MSP_Set_MajorMinor_Mode(MSPMajorMode,MSPMinorMode );
	Sleep(4);
	Audio_MSP_SetStereo(MSPMajorMode, MSPMinorMode,MSPStereo);
	Audio_MSP_Watch_Mode();
	MSPToneControl=TRUE;
	Audio_MSP_SetVolume(InitialVolume);
	Audio_MSP_SetVolumeNicam(InitialNicam);
	Audio_MSP_SetVolumeAM(InitialAM);
	Audio_MSP_SetBalance(InitialBalance);
	Audio_MSP_SetSuperBass(InitialSuperBass);
	Audio_MSP_SetBass(InitialBass);
	Audio_MSP_SetTreble(InitialTreble);
	Audio_MSP_SetLoudness(InitialLoudness);
	Audio_MSP_SetSpatial(InitialSpatial);
	Audio_MSP_SetEqualizer1(InitialEqualizer1);
	Audio_MSP_SetEqualizer2(InitialEqualizer2);
	Audio_MSP_SetEqualizer3(InitialEqualizer3);
	Audio_MSP_SetEqualizer4(InitialEqualizer4);
	Audio_MSP_SetEqualizer5(InitialEqualizer5);
	return(TRUE);
}


static void Audio_MSP_SetToneControl(BOOL  nMode)
{
	if ( nMode == MSPToneControl )	return;
	MSPToneControl=nMode;
	if (MSPToneControl)
	{
		WriteDSP(2,InitialBass<<8); // Bass
		WriteDSP(0x31,InitialBass<<8);
		WriteDSP(3,InitialTreble<<8); // Treble
		WriteDSP(0x32,InitialTreble<<8);
		WriteDSP(0x21,0); // Eq
		WriteDSP(0x22,0);
		WriteDSP(0x23,0);
		WriteDSP(0x24,0);
		WriteDSP(0x25,0);
		WriteDSP(0x20,0); // Mode control here (need eq=0)
	} 
	else
	{
		WriteDSP(0x20,0xFF<<8); // Mode control here (need eq=0)
		WriteDSP(2,0); // Bass
		WriteDSP(0x31,0);
		WriteDSP(3,0); // Treble
		WriteDSP(0x32,0);
		WriteDSP(0x21,InitialEqualizer1); // Eq
		WriteDSP(0x22,InitialEqualizer2);
		WriteDSP(0x23,InitialEqualizer3);
		WriteDSP(0x24,InitialEqualizer4);
		WriteDSP(0x25,InitialEqualizer5);
	}
}


BOOL Audio_MSP_SetVolume(int nVolume)
{
	Audio_MSP_SetToneControl(TRUE);
	if(nVolume<0 || nVolume>1000)
		return FALSE;
	InitialVolume=nVolume;
	if(nVolume>0)
		nVolume=MulDiv(nVolume+400,0x7f0,1400);
	WriteDSP(0,nVolume<<4);
	WriteDSP(6,nVolume<<4);
	return TRUE;
}


BOOL Audio_MSP_SetVolumeNicam(char nNicam)
{
	Audio_MSP_SetToneControl(TRUE);
	InitialNicam=nNicam;
	WriteDSP(0x10,nNicam<<8);
	return TRUE;
}


BOOL Audio_MSP_SetVolumeAM(char nAM)
{
	Audio_MSP_SetToneControl(TRUE);
	InitialAM=nAM;
	WriteDSP(0x000d,nAM<<8);
	return TRUE;
}


BOOL Audio_MSP_SetBalance(char nBalance)
{
	Audio_MSP_SetToneControl(TRUE);
	InitialBalance=nBalance;
	WriteDSP(1,nBalance<<8);
	WriteDSP(0x30,nBalance<<8);
	return TRUE;
}


BOOL Audio_MSP_SetBass(char nBass)
{
	Audio_MSP_SetToneControl(TRUE);
	if(nBass<-96) return FALSE;
	InitialBass=nBass;
	WriteDSP(2,nBass<<8);
	WriteDSP(0x31,nBass<<8);
	return TRUE;
}


BOOL Audio_MSP_SetTreble(char nTreble)
{
	Audio_MSP_SetToneControl(TRUE);
	if(nTreble<-96) return FALSE;
	InitialTreble=nTreble;
	WriteDSP(3,nTreble<<8);
	WriteDSP(0x32,nTreble<<8);
	return TRUE;
}


BOOL Audio_MSP_SetLoudness(BYTE nLoudness)
{
	Audio_MSP_SetToneControl(TRUE);
	if(nLoudness>68) return FALSE;
	InitialLoudness=nLoudness;
	WriteDSP(4,(nLoudness<<8)+(InitialSuperBass?0x4:0));
	WriteDSP(0x33,(nLoudness<<8)+(InitialSuperBass?0x4:0));
	return TRUE;
}


BOOL Audio_MSP_SetSuperBass(BOOL bSuperBass)
{
	Audio_MSP_SetToneControl(TRUE);
	InitialSuperBass=bSuperBass;
	WriteDSP(4,(InitialLoudness<<8)+(bSuperBass?0x4:0));
	WriteDSP(0x33,(InitialLoudness<<8)+(bSuperBass?0x4:0));
	return TRUE;
}


BOOL Audio_MSP_SetSpatial(char nSpatial)
{
	Audio_MSP_SetToneControl(TRUE);
	InitialSpatial=nSpatial;
	WriteDSP(0x5,(nSpatial<<8)+0x8); // Mode A, Automatic high pass gain
	return TRUE;
}


BOOL Audio_MSP_SetEqualizer1(char nLevel)
{
	Audio_MSP_SetToneControl(FALSE);
	if(nLevel<-96 || nLevel>96) return FALSE;
	InitialEqualizer1=nLevel;
	WriteDSP(0x21,nLevel<<8);
	return TRUE;
}


BOOL Audio_MSP_SetEqualizer2(char nLevel)
{
	Audio_MSP_SetToneControl(FALSE);
	if(nLevel<-96 || nLevel>96) return FALSE;
	InitialEqualizer2=nLevel;
	WriteDSP(0x22,nLevel<<8);
	return TRUE;
}


BOOL Audio_MSP_SetEqualizer3(char nLevel)
{
	Audio_MSP_SetToneControl(FALSE);
	if(nLevel<-96 || nLevel>96) return FALSE;
	InitialEqualizer3=nLevel;
	WriteDSP(0x23,nLevel<<8);
	return TRUE;
}


BOOL Audio_MSP_SetEqualizer4(char nLevel)
{
	Audio_MSP_SetToneControl(FALSE);
	if(nLevel<-96 || nLevel>96) return FALSE;
	InitialEqualizer4=nLevel;
	WriteDSP(0x24,nLevel<<8);
	return TRUE;
}


BOOL Audio_MSP_SetEqualizer5(char nLevel)
{
	Audio_MSP_SetToneControl(FALSE);
	if(nLevel<-96 || nLevel>96) return FALSE;
	InitialEqualizer5=nLevel;
	WriteDSP(0x25,nLevel<<8);
	return TRUE;
}


BOOL Audio_MSP_Reset()
{
	BOOL ret = TRUE;
	I2CBus_Lock();
    I2CBus_Start();
    I2CBus_SendByte(AudioDeviceWrite,5);
	I2CBus_SendByte(0x00,0);
	I2CBus_SendByte(0x80,0);
	I2CBus_SendByte(0x00,0);
    I2CBus_Stop();
    I2CBus_Start();
    if ( (!I2CBus_SendByte(AudioDeviceWrite,5)) ||
		 (!I2CBus_SendByte(0x00,0)) ||
		 (!I2CBus_SendByte(0x00,0)) ||
		 (!I2CBus_SendByte(0x00,0)) )
		 ret=FALSE;

    I2CBus_Stop();
	I2CBus_Unlock();
	return ret;
}


static void Audio_MSP_SetCarrier(int cdo1, int cdo2)
{
	WriteDem(0x93, cdo1 & 0xfff);
	WriteDem(0x9b, cdo1 >> 12);
	WriteDem(0xa3, cdo2 & 0xfff);
	WriteDem(0xab, cdo2 >> 12);
}


void Audio_MSP_SetMode(int type)
{
	int i;

	MSPStereo=VIDEO_SOUND_MONO;
	MSPMode=type;
	if (!Has_MSP)
	{ 
		// Pas de msp !
		return;
	}

	WriteDem(0xbb,msp_init_data[type].ad_cv);
    
	for (i = 5; i >= 0; i--) WriteDem( 0x01,msp_init_data[type].fir1[i]);
    
	WriteDem( 0x05, 0x0004); // fir 2 
	WriteDem( 0x05, 0x0040);
	WriteDem( 0x05, 0x0000);

	for (i = 5; i >= 0; i--) WriteDem( 0x05,msp_init_data[type].fir2[i]);
    
	WriteDem( 0x83,msp_init_data[type].mode_reg);
    
	Audio_MSP_SetCarrier( msp_init_data[type].cdo1,msp_init_data[type].cdo2);
    
	WriteDem( 0x60, 0); // LOAD_REG_1/2 
	WriteDSP( 0x08,msp_init_data[type].dfp_src);
	WriteDSP( 0x09,msp_init_data[type].dfp_src);
	WriteDSP( 0x0a,msp_init_data[type].dfp_src);
	WriteDSP( 0x0e,msp_init_data[type].dfp_matrix);
	if(MSPNicam)	WriteDSP(0x10, InitialNicam<<8);
}


void Audio_MSP_SetStereo(int MajorMode, int MinorMode, int mode)
{
     int nicam=0; 
	 int src=0;

	 MSPStereo=mode;

	// switch demodulator 
	switch (MSPMode)
	{
		case MSP_MODE_FM_TERRA:
 			Audio_MSP_SetCarrier(carrier_detect[MinorMode], carrier_detect_main[MajorMode]);
			switch (MSPStereo)
			{
				case VIDEO_SOUND_STEREO:
					WriteDSP(0x0e, 0x3001);
					break;

				case VIDEO_SOUND_MONO:
				case VIDEO_SOUND_LANG1:
				case VIDEO_SOUND_LANG2:
					WriteDSP( 0x0e, 0x3000);
				break;
			}
			break;

		case MSP_MODE_FM_SAT:
			switch (MSPStereo)
			{
				case VIDEO_SOUND_MONO:
					Audio_MSP_SetCarrier( MSP_CARRIER(6.5), MSP_CARRIER(6.5));
					break;

				case VIDEO_SOUND_STEREO:
					Audio_MSP_SetCarrier(  MSP_CARRIER(7.2), MSP_CARRIER(7.02));
					break;

				case VIDEO_SOUND_LANG1:
					Audio_MSP_SetCarrier( MSP_CARRIER(7.38), MSP_CARRIER(7.02));
					break;

				case VIDEO_SOUND_LANG2:
					Audio_MSP_SetCarrier(  MSP_CARRIER(7.38), MSP_CARRIER(7.02));
					break;
			}
			break;

		case MSP_MODE_FM_NICAM1:
		case MSP_MODE_FM_NICAM2:
		case MSP_MODE_AM_NICAM:
	 		Audio_MSP_SetCarrier(carrier_detect[MinorMode], carrier_detect_main[MajorMode]);
			nicam=0x0100;
			break;

		default:
			// can't do stereo - abort here 
			return;
	}

	// switch audio 
	switch (MSPStereo)
	{
		case VIDEO_SOUND_STEREO:
			src=0x0020|nicam;
			WriteDSP( 0x05,0x4000);
			break;

		case VIDEO_SOUND_MONO:
			if (MSPMode == MSP_MODE_AM_NICAM)
			{
				// AM mono decoding is handled by tuner, not MSP chip
				// so let's redirect sound from tuner via SCART
				// volume prescale for SCART
				WriteDSP(0x000d,InitialAM<<8);
				// SCART switching control register
				WriteDSP(0x0013,0xe900);
				src=0x0200;
				break;
			} 

		case VIDEO_SOUND_LANG1:
			src=0x0000|nicam;
			break;

		case VIDEO_SOUND_LANG2:
			src=0x0010|nicam;
			break;
	}
	WriteDSP(0x08,src);
	WriteDSP(0x09,src);
	WriteDSP(0x0a,src);
}


void Audio_MSP_Set_MajorMinor_Mode(int MajorMode,int MinorMode)
{
	 MSPMajorMode=MajorMode;
	 MSPMinorMode=MinorMode;

		switch (MajorMode)
		{
			case 1: // 5.5 
				if (MinorMode == 0)
				{
					// B/G FM-stereo 
//					Audio_MSP_SetMode(MSP_MODE_FM_TERRA);
					Audio_MSP_SetStereo(MajorMode,MinorMode,VIDEO_SOUND_MONO);
				}
				else if (MinorMode == 1 && MSPNicam)
				{
					// B/G NICAM 
//					Audio_MSP_SetMode(MSP_MODE_FM_NICAM1);
					Audio_MSP_SetCarrier(carrier_detect[MinorMode], carrier_detect_main[MajorMode]);
				}
				else
				{
//					Audio_MSP_SetMode(MSP_MODE_FM_TERRA);
					Audio_MSP_SetCarrier(carrier_detect[MinorMode], carrier_detect_main[MajorMode]);
				}
				break;

			case 2: // 6.0 
				// PAL I NICAM 
//				Audio_MSP_SetMode(MSP_MODE_FM_NICAM2);
				Audio_MSP_SetCarrier(MSP_CARRIER(6.552), carrier_detect_main[MajorMode]);
				break;

			case 3: // 6.5 
				if (MinorMode == 1 || MinorMode == 2)
				{
					// D/K FM-stereo 
//					Audio_MSP_SetMode( MSP_MODE_FM_TERRA);
					Audio_MSP_SetStereo(MajorMode,MinorMode,VIDEO_SOUND_MONO);
				}
				else if (MinorMode == 0 && MSPNicam)
				{
					// D/K NICAM 
//					Audio_MSP_SetMode(MSP_MODE_FM_NICAM1);
					Audio_MSP_SetCarrier(carrier_detect[MinorMode], carrier_detect_main[MajorMode]);
				}
				else
				{
//					Audio_MSP_SetMode(MSP_MODE_FM_TERRA);
					Audio_MSP_SetCarrier(carrier_detect[MinorMode], carrier_detect_main[MajorMode]);
				}
				break;

			case 0: // 4.5 
			default:
//				Audio_MSP_SetMode(MSP_MODE_FM_TERRA);
				Audio_MSP_SetCarrier(carrier_detect[MinorMode], carrier_detect_main[MajorMode]);
				break;
		}
}


static void Audio_MSP_Print_Mode()
{
    if(!Has_MSP) strcpy(Text, Langue_GetString(ID_NOMSP));
	else
	{
		switch(MSPMode)
		{
			case 0 : strcpy(Text,"AM (msp3400)+"); break;
			case 1 : strcpy(Text,"AM (msp3410)+"); break;
			case 2 : strcpy(Text,"FM Radio+"); break;
			case 3 : strcpy(Text,"TV Terrestial+"); break;
			case 4 : strcpy(Text,"TV Sat+"); break;
			case 5 : strcpy(Text,"NICAM B/G+"); break;
			case 6 : strcpy(Text,"NICAM I+"); break;
			case 7 :
				if(MSPStereo==VIDEO_SOUND_MONO) strcpy(Text,"AM ");
				else strcpy(Text,"NICAM ");
				break;
		}
		if(MSPMode<7)
		{
		    switch(MSPMajorMode)
			{
				case 0 : strcat(Text,"NTSC+"); break;
				case 1 : strcat(Text,"PAL B/G+"); break;
				case 2 : strcat(Text,"PAL I+"); break;
				case 3 : strcat(Text,"PAL D/K (Sat+Secam)+"); break;
			}		   
			switch(MSPMinorMode)
			{
				case 0 : strcat(Text,"FM-stereo "); break;
				case 1 : strcat(Text,"NICAM "); break;
				case 2 : strcat(Text,"NICAM "); break;
				case 3 : strcat(Text,"D/K1 FM-Stereo "); break;
				case 4 : strcat(Text,"D/K2 FM-stereo "); break;
				case 5 : strcat(Text,"SAT FM-stereo s/b "); break;
				case 6 : strcat(Text,"SAT FM-stereo s "); break;
				case 7 : strcat(Text,"SAT FM-stereo b "); break;
			}
		}
		switch(MSPStereo)
		{
			case 1 : strcat(Text,"Mono"); break;
			case 2 : strcat(Text,"Stro"); break;
			case 3 : strcat(Text,"Canal 1"); break;
			case 4 : strcat(Text,"Canal 2"); break;
		}		   
	}
	StatusBar_Update(1, Text);
}


void Audio_MSP_Watch_Mode( )
{
	int val;
    int newstereo=MSPStereo;

	if (!Has_MSP) return;

 	switch (MSPMode)
	{
		case MSP_MODE_FM_TERRA:
			val = ReadDSP( 0x18 );
			if (val > 4096)
			{
				newstereo = VIDEO_SOUND_STEREO;
			}
			else if (val < -4096)
			{
				newstereo = VIDEO_SOUND_LANG1;
			}
			else
			{
				newstereo = VIDEO_SOUND_MONO;
			}
			break;

		case MSP_MODE_AM_NICAM://Quenotte
			newstereo = VIDEO_SOUND_MONO;
			//Sleep(750); //On n'attend plus...remplace par un timer => commutation vraiment auto a present donc...
			if( ReadDem(0x57)<700 )	newstereo = VIDEO_SOUND_STEREO;
			break;

		case MSP_MODE_FM_NICAM1:
		case MSP_MODE_FM_NICAM2:
			val = ReadDSP(0x23);
			switch ((val & 0x1e) >> 1)
			{
				case 0:
				case 8:
					newstereo = VIDEO_SOUND_STEREO;
					break;

				default:
					newstereo = VIDEO_SOUND_MONO;
					break;
			}
			break;
	} 
	
	if ( MSPStereo != newstereo )
	{
		if ( AutoStereoSelect ) 
		{
			Audio_MSP_SetStereo(MSPMajorMode,MSPMinorMode,newstereo);
			Audio_MSP_SetSpatial(InitialSpatial);
		}
	}
	Audio_MSP_Print_Mode();
}

void Audio_MSP_Hauppauge_Boot()
{
    // reset/enable the MSP on some Hauppauge cards 
    // Thanks to Kysti Mlkki (kmalkki@cc.hut.fi)! 
    BT8X8_AndOrDataDword(BT848_GPIO_DATA, 32, ~32);
    BT8X8_AndOrDataDword(BT848_GPIO_DATA, 0, ~32);
    Sleep(10);
    BT8X8_AndOrDataDword(BT848_GPIO_DATA, 32, ~32);
}

