/////////////////////////////////////////////////////////////////////////////
// NextView.cpp
/////////////////////////////////////////////////////////////////////////////
// Copyright (c) 2000-2002 Quenotte  All rights reserved.
/////////////////////////////////////////////////////////////////////////////
//
//	This file is subject to the terms of the GNU General Public License as
//	published by the Free Software Foundation.  A copy of this license is
//	included with this software distribution in the file COPYING.  If you
//	do not have a copy, you may obtain a copy by writing to the Free
//	Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
//
//	This software is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details
/////////////////////////////////////////////////////////////////////////////
//
// Communication avec le plugin NextView.
// Une grande part du code provient de tvsim inclut dans NextView.
//
// NextViewEPG :  Copyright (c) Thorsten Zoerner
/////////////////////////////////////////////////////////////////////////////
#include "stdafx.h"
#include <time.h>
#include <shlobj.h>     // for SHBroweForFolder 
#include "resource.h"

#include "K!TV-XP.h"
#include "nextview_defines.h"
#include "nextviewplugin.h"
#include "debuglog.h"
#include "osd.h"
#include "settings.h"
#include "misc.h"
#include "versionno.h"
#include "nextview.h"
#include "VBI_VideoText.h"

#define TVAPP_FEAT (TVAPP_FEAT_TTX_FWD|TVAPP_FEAT_VPS_FWD|TVAPP_FEAT_REQ_CNAME|TVAPP_FEAT_CMD_TUNE)

static void NexTView_ChargerPlugin();
static void NexTView_OSDInfo(const char *title, time_t start, time_t stop, char *pdcinfo);
void __cdecl NexTView_EpgEvent();
static void NexTView_HandleEpgCmd();

typedef TVApp * (WINAPI *tfuncInit)();

char NextViewPath[MAX_PATH], *pdc_series = "series";
TVApp *CurrentTVApp;
BOOL PluginNextViewPresent = FALSE, ChannelChanged = FALSE;
HINSTANCE hDllInstance;

PROCESS_INFORMATION ProcNXTV;
BOOL bNXTVactive = FALSE,	// NextView.exe lanc ou termin par le menu
bNextViewAuto = TRUE;		// Control automatique de NextView gr par K!TV ?
HWND NXThWnd=NULL;



BOOL CALLBACK NexTView_Chercher(HWND hwnd, LPARAM lParam)
{
	char Text[255];
	GetWindowText(hwnd,(LPSTR)&Text,13);
	if(strncmp(Text,"Nextview EPG",12)==0)
	{
		NXThWnd=hwnd;
		return FALSE;
	}
	return TRUE;
}


BOOL NexTView_Demarrer(BOOL bInit)
{
	STARTUPINFO StartNXTV;

	if (!bNextViewAuto)	return FALSE;

	NXThWnd=NULL;

	// On recherche si NexTView est deja lance...
	EnumWindows(NexTView_Chercher,0);
	if (NXThWnd!=NULL) 
	// Si NexTView est deja lance, on le minimise...
		ShowWindow(NXThWnd,SW_MINIMIZE);
	else 
	{
		 //Sinon on le lance...
		char pszBuffer[MAX_PATH];

		strcpy(pszBuffer, NextViewPath);
//		strcat(pszBuffer, "\\nxtvepg.exe");
		strcat(pszBuffer, "\\nxtvepg.exe -iconic");
		ZeroMemory(&StartNXTV, sizeof(STARTUPINFO));
		StartNXTV.cb = sizeof(STARTUPINFO);

		if(!CreateProcess(NULL, pszBuffer, (LPSECURITY_ATTRIBUTES)NULL,(LPSECURITY_ATTRIBUTES)NULL,FALSE,(DWORD)0,NULL,NextViewPath,&StartNXTV,&ProcNXTV))		
		{
			if(bInit)
			{
				bNXTVactive=FALSE;
				strcpy(NextViewPath, "");
			}
			else
			{
				CloseHandle(ProcNXTV.hThread);
				CloseHandle(ProcNXTV.hProcess);
				NexTView_Chemin();
				strcpy(pszBuffer, NextViewPath);
				strcat(pszBuffer, "\\nxtvepg.exe -iconic");
				if(!CreateProcess(NULL, pszBuffer, (LPSECURITY_ATTRIBUTES)NULL,(LPSECURITY_ATTRIBUTES)NULL,FALSE,(DWORD)0,NULL,NextViewPath,&StartNXTV,&ProcNXTV))
				{
					bNXTVactive=FALSE;
					strcpy(NextViewPath, "");
				}
			}
		}
		CloseHandle(ProcNXTV.hThread);
		CloseHandle(ProcNXTV.hProcess);
		if(bNXTVactive)	SetTimer(hWnd, TIMER_FOREGROUND, 1, NULL);
	}
	return TRUE;
}


BOOL NexTView_Fermer()
{
	if (!bNextViewAuto)
		return FALSE;

	if(ProcNXTV.dwProcessId != 0)
		EnumWindows(NexTView_Chercher,0);
	SendMessage(NXThWnd, WM_CLOSE, 0, 0);
	NXThWnd=NULL;
	return TRUE;
}


static void NexTView_DisplayError()
{
	const uchar *pShmErrMsg;

	pShmErrMsg = CurrentTVApp->GetErrorMsg();
	if (pShmErrMsg) 
	{
		//	RealErrorBox(pShmErrMsg);
		LOG(1, "error: %s", pShmErrMsg);

// TODO : c'est un pointeur valide mais free() provoque un crash ...
//		free ((void *)pShmErrMsg);
	}
}


void NexTView_Chemin()
{
	BROWSEINFO browseinfo;
    LPMALLOC pMalloc;
	char pszBuffer[MAX_PATH];
	LPITEMIDLIST pidl;
	
    if (SHGetMalloc(&pMalloc) != NOERROR)
		return;
	
	ZeroMemory(&browseinfo, sizeof(BROWSEINFO));
	browseinfo.hwndOwner = hWnd;
	browseinfo.lpszTitle = Langue_GetString(ID_NXTVIEW_SELPATH);
	browseinfo.ulFlags = BIF_RETURNFSANCESTORS | BIF_RETURNONLYFSDIRS;
	
	if ((pidl = SHBrowseForFolder(&browseinfo)) != NULL)
	{
		if (SHGetPathFromIDList(pidl, pszBuffer))
			strcpy(NextViewPath, pszBuffer);
		else
			strcpy(NextViewPath, "");
		pMalloc->Free(pidl);
	} 
	pMalloc->Release();
}


void NexTView_Init()
{
	BOOL ret;
	int error;
	static WINSHMCLNT_TVAPP_INFO KTV_info = {"K!TV " STRPRODUCTVER, "", TVAPP_KTV, TVAPP_FEAT, (void (*)())NexTView_EpgEvent};

	NexTView_ChargerPlugin();
	if (!PluginNextViewPresent) return;

	ret = CurrentTVApp->Init(&KTV_info, &error);
	if (!ret)	NexTView_DisplayError();
}


// si NexTView_Epg_DbAcqNewVbiFrame retourne FALSE il y'a eu changement de chaine
// il ne faut pas transmettre de vbi .
void NexTView_AcqNewVbiFrame(unsigned int frameSeqNo)
{
	if (!PluginNextViewPresent) return;
	ChannelChanged = FALSE;
	ChannelChanged = CurrentTVApp->AcqNewVbiFrame(frameSeqNo);
	ChannelChanged = !ChannelChanged;
}


void NexTView_AddPacket(const unsigned char * data)
{	
	if (!PluginNextViewPresent) return;
	if (ChannelChanged) 		return;
	CurrentTVApp->AddPacket(data + 3);
}


void NexTView_AddVpsData( const unsigned char * data )
{
	if (!PluginNextViewPresent) return;
	if (ChannelChanged) return;
	CurrentTVApp->AddVpsData(data);
}


// TODO : 
// Send stored CNI instead of channel name ? It should be faster.
void NexTView_ChannelChange()
{
	if (!PluginNextViewPresent) return;
	CurrentTVApp->SetStation(Program[CurrentProgram].Name, 0, 0, MulDiv(Program[CurrentProgram].freq*1000,16,1000000) );
}


void NexTView_HandleEpgEvent(void)
{

	int curEvent;
	uint  loopCount;
	bool  shouldExit;
	
	if (!PluginNextViewPresent) return;

	shouldExit = FALSE;
	loopCount = 3;
	do
	{
		curEvent = CurrentTVApp->GetEpgEvent();
		switch (curEvent)
		{
		case SHM_EVENT_ATTACH:
            break;
			
		case SHM_EVENT_DETACH:
            shouldExit = TRUE;
            break;
			
		case SHM_EVENT_ATTACH_ERROR:
			NexTView_DisplayError();
            shouldExit = TRUE;
            break;
			
		case SHM_EVENT_PROG_INFO:
			NexTView_UpdateProgInfo();
            break;
			
		case SHM_EVENT_CMD_ARGV:
			NexTView_HandleEpgCmd();
            break;
			
		case SHM_EVENT_INP_FREQ:
// TODO:
// If K!tv granted tuner to NextView, we should handle this event
//            TvSimuMsg_ReqTuner();
            break;
			
		case SHM_EVENT_NONE:
            shouldExit = TRUE;
            break;
			
		default:
            LOG(1, "TvSimu-IdleHandler: unknown EPG event %d", curEvent);
            break;
		}
		loopCount -= 1;
	}
	while ((shouldExit == FALSE) && (loopCount > 0));

	// refaire un passage (cas limite ?)
//	if (shouldExit == FALSE )
//		NexTView_EpgEvent();
}



void NexTView_UpdateProgInfo()
{
uchar pTitle[EPG_TITLE_MAX_LEN];
const char *title;
time_t start, stop;
unsigned int themeCount;
uchar pThemes[7];
uchar ret;

	if (!PluginNextViewPresent) return;
	ret = CurrentTVApp->GetProgInfo(&start, &stop, pThemes, &themeCount, pTitle, sizeof(pTitle));

	if ( ret && (pTitle[0] != 0) && (start != stop) )
	{
		unsigned int themeIdx;
		char * pThemeStr;

		// Process PDC theme array: convert theme code to text
		// Note: up to 7 theme codes can be supplied (e.g. "movie general" plus "comedy")
		//       you have to decide yourself how to display them; you might use icons for the
		//       main themes (e.g. movie, news, talk show) and add text for sub-categories.
		// Use loop to search for the first valid theme code (simplest possible handling)
		pThemeStr = "";
		for (themeIdx = 0; themeIdx < themeCount; themeIdx++)
		{
			if (pThemes[themeIdx] >= 128)
			{  // PDC codes 0x80..0xff identify series (most EPG providers just use 0x80 for
				// all series, but they could use different codes for every series of a network)
				pThemeStr = pdc_series;
				break;
			}
			else if (pdc_themes[pThemes[themeIdx]] != NULL)
			{  // this is a known PDC series code
				pThemeStr = pdc_themes[pThemes[themeIdx]];
				break;
			}
			// else: unknown series code -> keep searching
		}
		title = (const char *)pTitle;
		NexTView_OSDInfo(title, start, stop, pThemeStr);
	}
}


static void NexTView_HandleEpgCmd()
{
	uint argc;
	char pArgStr[EPG_CMD_MAX_LEN];
	char * pArg2;
	uchar ret;

	if (!PluginNextViewPresent) return;
	
	ret = CurrentTVApp->GetCmdArgv(&argc, pArgStr, sizeof(pArgStr));
	
	if (argc == 2 && ret)
	{
		pArg2 = pArgStr + strlen(pArgStr) + 1;
		
		if (strcmp(pArgStr, "setstation") == 0)
		{
			for (int i = 0; i < MAXPROGS; i++) 
			{		
				if (strstr(Program[i].Name, pArg2) != NULL)
				{
					ChangementCanal(i);
					return;
				}
			}
		}
		else if (strcmp(pArgStr, "capture") == 0)
		{
			// when capturing is switched off, grant tuner to EPG
		}
		else if (strcmp(pArgStr, "volume") == 0)
		{
			// command ignored in simulation because audio is not supported
		}
	}
}

// Demande le traitement du message par le thread principal de K!TV.
// Le thread EPG est libr.
void __cdecl NexTView_EpgEvent()
{
	PostMessage(hWnd, WM_COMMAND, IDM_NEXTVIEW_EVENT, 0);
}

// Quelques definitions from OSD.cpp
#define OSD_COLOR_SECTION   RGB(150,150,255)
#define OSD_COLOR_TITLE     RGB(255,150,150)
extern double OSD_GetLineYpos(int nLine, double dfMargin, double Size);

static void NexTView_OSDInfo(const char *title, time_t start, time_t stop, char *pdcinfo)
{
	char tempChar[255];
	double  dfMargin=0.02;
	int	Size = 6;
	int Line=12;
	int i=0;

	OSD_Clear(hWnd,0);

	GetLocalTime(&LocalTime);
	sprintf (tempChar, "%d:%02d", LocalTime.wHour, LocalTime.wMinute);
    OSD_AddText(tempChar, Size+1, OSD_COLOR_TITLE, -1, OSDBACK_LASTONE, OSD_XPOS_RIGHT, 1-dfMargin, OSD_GetLineYpos(Line++, dfMargin, Size),0);

	OSD_AddText(Langue_GetString(ID_NXTVIEW_PROGRAM_TITLE), Size, OSD_COLOR_SECTION, -1, OSDBACK_LASTONE, OSD_XPOS_LEFT, dfMargin, OSD_GetLineYpos (Line, dfMargin, Size),0);
	OSD_AddText(title, Size, -1, -1, OSDBACK_LASTONE, OSD_XPOS_RIGHT, 1-dfMargin, OSD_GetLineYpos(Line, dfMargin, Size),0);

	Line ++;
	OSD_AddText(Langue_GetString(ID_NXTVIEW_PDC_THEME), Size, OSD_COLOR_SECTION, -1, OSDBACK_LASTONE, OSD_XPOS_LEFT, dfMargin, OSD_GetLineYpos (Line, dfMargin, Size),0);
	OSD_AddText(pdcinfo, Size, -1, -1, OSDBACK_LASTONE, OSD_XPOS_RIGHT, 1-dfMargin, OSD_GetLineYpos(Line, dfMargin, Size),0);

	Line ++;
//	strftime(tempChar, sizeof(tempChar), "%#x %A %d %B %H:%M", localtime(&start));
	strftime(tempChar, sizeof(tempChar), "%#x %H:%M", localtime(&start));
	OSD_AddText(Langue_GetString(ID_NXTVIEW_PRGM_START), Size, OSD_COLOR_SECTION, -1, OSDBACK_LASTONE, OSD_XPOS_LEFT, dfMargin, OSD_GetLineYpos (Line, dfMargin, Size),0);
	OSD_AddText(tempChar, Size, -1, -1, OSDBACK_LASTONE, OSD_XPOS_RIGHT, 1-dfMargin, OSD_GetLineYpos(Line, dfMargin, Size),0);

	Line ++;
	strftime(tempChar, sizeof(tempChar), "%H:%M", localtime(&stop));
	OSD_AddText(Langue_GetString(ID_NXTVIEW_PRGM_STOP), Size, OSD_COLOR_SECTION, -1, OSDBACK_LASTONE, OSD_XPOS_LEFT, dfMargin, OSD_GetLineYpos (Line, dfMargin, Size),0);
	OSD_AddText(tempChar, Size, -1, -1, OSDBACK_LASTONE, OSD_XPOS_RIGHT, 1-dfMargin, OSD_GetLineYpos(Line, dfMargin, Size),0);

	OSD_Show(hWnd, OSD_AUTOHIDE, 0,OSD_TIMER_ID);
	KillTimer(hWnd, OSD_TIMER_ID);
	SetTimer(hWnd, OSD_TIMER_ID, OSD_TIMER_DELAY2, NULL);
}

static void NexTView_ChargerPlugin() 
{
	char mod[256], *TypePlugin = "NextView";
	HMODULE hPlugInMod;
	tfuncInit funcInit;

	__try
	{
		LOG (1, Langue_GetString(ID_PLG_INFO), TypePlugin);
		PluginNextViewPresent=FALSE;
		hPlugInMod = LoadLibrary("Plugins/NextViewPlugin.dll");
		if (hPlugInMod == NULL)
		{
			LOG (1, Langue_GetString(ID_PLG_ERROR_LOAD), TypePlugin);
			return;
		}
	    funcInit = (tfuncInit) GetProcAddress(hPlugInMod, "InitThisPlugin");
		if ( funcInit == NULL)
		{
			PluginNextViewPresent=FALSE;
			FreeLibrary((HMODULE)hPlugInMod);
			sprintf(mod, Langue_GetString(ID_PLG_ERROR_INIT), TypePlugin);
			RealErrorBox(mod);
			return;
		}
		LOG (1, Langue_GetString(ID_PLG_SUCCES), TypePlugin);
		PluginNextViewPresent=TRUE;
		hDllInstance = hPlugInMod;
		CurrentTVApp = funcInit();
		// TODO : check version
	}

    __except (EXCEPTION_EXECUTE_HANDLER  ) 
	{  
		PluginNextViewPresent=FALSE;
		FreeLibrary((HMODULE)hPlugInMod);
		sprintf(mod, Langue_GetString(ID_PLG_ERROR_VER), TypePlugin, "");
		RealErrorBox(mod);
		return;
	}
}


void NexTView_Exit()
{
	__try
	{
		if (PluginNextViewPresent)
		{
			CurrentTVApp->Exit();
			Sleep(20);
			FreeLibrary((HMODULE)hDllInstance);
			PluginNextViewPresent=FALSE;
		}
	}
    __except (EXCEPTION_EXECUTE_HANDLER  ) 
	{  
		return;
	}
}
