/////////////////////////////////////////////////////////////////////////////
// OSD.c
/////////////////////////////////////////////////////////////////////////////
// Copyright (c) 2000-2002 Quenotte  All rights reserved.
/////////////////////////////////////////////////////////////////////////////
//
//	This file is subject to the terms of the GNU General Public License as
//	published by the Free Software Foundation.  A copy of this license is
//	included with this software distribution in the file COPYING.  If you
//	do not have a copy, you may obtain a copy by writing to the Free
//	Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
//
//	This software is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details
/////////////////////////////////////////////////////////////////////////////
//
// Based on DScaler 
// Copyright (c) 2002 John Adcock.  All rights reserved.
//
/////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "resource.h"

#include "OSD.h"
#include "K!TV-XP.h"
#include "vbi-vt.h"
#include "VBI_VideoText.H"
#include "ddraw.h"
#include "Dscaler/DS_Control.h"
#include "settings.h"
#include "versionno.h"
#include "plugin.h"
#include "bt848.h"
#include "btaudio.h"
#include "misc.h"
#include "thread.h"


#define OSD_COLOR_TITLE     RGB(255,150,150)
#define OSD_COLOR_SECTION   RGB(150,150,255)
#define OSD_COLOR_CURRENT   RGB(200,150,0)

char szFontName[128] = "Arial";
long OutlineColor = RGB(0,0,0);
long TextColor = RGB(0,255,0);
long DefaultSizePerc = 10;
long DefaultSmallSizePerc = 4;
BOOL bAntiAlias = TRUE;
BOOL bOutline = TRUE;
eOSDBackground Background;
BOOL bAutoHide = TRUE;

//---------------------------------------------------------------------------
// Global OSD Information structure
TOsdInfo    grOSD[OSD_MAX_TEXT+3];
static int  NbText = 0;

static int  IdxCurrentScreen = -1;  // index of the current displayed OSD screen
static BOOL bRestoreScreen = FALSE; // Restore Info screen when clear OSD

BOOL        bOverride[4] = {FALSE,FALSE,FALSE,FALSE};
BOOL		Enable_OSD = TRUE;

//---------------------------------------------------------------------------
// Clean the list of texts for OSD
void OSD_ClearAllTexts()
{
    NbText = 0;
}

//---------------------------------------------------------------------------
// Add a new single line of text to the list of texts for OSD
void OSD_AddTextSingleLine(LPCTSTR szText, double Size, long NewTextColor, long BackgroundColor, 
                           eOSDBackground BackgroundMode, eOSDTextXPos TextXPos, double XPos, 
                           double YPos,int NbTextForce)
{
    int NbTextSave=0;

	if ( (strlen(szText) == 0) || (NbText > OSD_MAX_TEXT+2) )
    {
        return;
    }

	if(NbTextForce!=0)
	{
		NbTextSave=NbText;
		NbText=NbTextForce;
	}
    
	if (Size == 0)
    {
        grOSD[NbText].Size = DefaultSizePerc;
    }
    else
    {
        grOSD[NbText].Size = Size;
    }
    if (NewTextColor == -1)
    {
        grOSD[NbText].TextColor = TextColor;
    }
    else
    {
        grOSD[NbText].TextColor = NewTextColor;
    }
    if (BackgroundColor == -1)
    {
        grOSD[NbText].BackgroundColor = OutlineColor;
    }
    else
    {
        grOSD[NbText].BackgroundColor = BackgroundColor;
    }
    if (BackgroundMode == OSDBACK_LASTONE)
    {
        grOSD[NbText].BackgroundMode = Background;
    }
    else
    {
        grOSD[NbText].BackgroundMode = BackgroundMode;
    }
    grOSD[NbText].TextXPos = TextXPos;
    grOSD[NbText].XPos = XPos;
    grOSD[NbText].YPos = YPos;
    strncpy(grOSD[NbText].szText, szText, sizeof(grOSD[NbText].szText));

	if(NbTextForce==0)	NbText++;
	else NbText=NbTextSave;
}

//---------------------------------------------------------------------------
// Add a new line of text to the list of texts for OSD. '\n' character is supported.
void OSD_AddText(LPCTSTR szText, double Size, long NewTextColor, long BackgroundColor, 
                 eOSDBackground BackgroundMode, eOSDTextXPos TextXPos, double XPos, double YPos, int NbTextForce)
{
    char      SingleLine[512];
    int       SingleLineIndex = 0;

    for(unsigned int i = 0; i < strlen(szText); i++)
    {
        if(SingleLineIndex == 512)
        {
            SingleLineIndex--;
            break;
        }
        if( szText[i] != '\n')
        {
            SingleLine[SingleLineIndex++] = szText[i];
        }
        else
        {
            SingleLine[SingleLineIndex] = 0x00;
            OSD_AddTextSingleLine(SingleLine, Size, NewTextColor, BackgroundColor, BackgroundMode,
                                  TextXPos, XPos, YPos, NbTextForce);
            SingleLineIndex = 0;
            if(Size == 0)
            {
                YPos += DefaultSizePerc/100;
            }
            else
            {
                YPos += Size/100;
            }
        }
    }
    SingleLine[SingleLineIndex] = 0x00;
    OSD_AddTextSingleLine(SingleLine,Size,NewTextColor,BackgroundColor,BackgroundMode,TextXPos,
                          XPos,YPos,NbTextForce);
}

//---------------------------------------------------------------------------
// Display defined OSD texts
void OSD_Show(HWND hWnd, int ShowType, int refresh_delay, int iID)
{
    RECT        winRect;
    HDC         hDC;

    if (bOverride[iID-42])
    {
        return;
    }
    if (ShowType == OSD_PERSISTENT)
    {
        KillTimer(hWnd, iID);
    }
//    KillTimer(hWnd, OSD_TIMER_REFRESH_ID);
    hDC = GetDC(hWnd);
    GetClientRect(hWnd,&winRect);

	if ( bShowMenu && bDisplayStatusBar && !bIsFullScreen)
    {
        winRect.bottom -= StatusBar_Height();
    }

	if ( VTState!=VT_OFF || Sous_Titres.bRun)
		InvalidateRect(hWnd, &winRect, FALSE);
	else Affichage_Overlay_PaintColorkey(hWnd, TRUE, hDC, &winRect);

    OSD_Redraw(hWnd, hDC);

	ReleaseDC(hWnd, hDC);
    if (ShowType == OSD_AUTOHIDE)
    {
        SetTimer(hWnd, iID, OSD_TIMER_DELAY, NULL);
    }
/*    if (refresh_delay > 0)
    {
        SetTimer(hWnd, OSD_TIMER_REFRESH_ID, refresh_delay, NULL);
    }*/
}


//---------------------------------------------------------------------------
// Calculate vertical position of line in OSD screen
//    Return Value between 0 (top) and 1 (bottom)
//    Use line number > 0 if reference is top
//    Use line number < 0 if reference is bottom
//    dfMargin is a percent of screen height/width and Value must be between 0 and 1
//    Size is a percent of screen height and Value must be between 0 and 100
double OSD_GetLineYpos (int nLine, double dfMargin, double Size)
{
    double  dfY;
    double  dfH = Size / 100;

    // Line number 0 has no sense
    if (nLine == 0) return (0);

    if (nLine > 0)
    {
        dfY = dfMargin + ((double)nLine - 1) * dfH;
    }
    else
    {
        dfY = 1 - dfMargin + (double)nLine * dfH;
    }

    // Line outside screen
    if ( (dfY < 0) || ((dfY + dfH) > 1) )
    {
        dfY = 0;
    }

    return (dfY);
}


//---------------------------------------------------------------------------
// Display specified OSD text with autohide
void OSD_ShowText(HWND hWnd, LPCTSTR szText, double Size)
{
    if (!Enable_OSD) return;
    if (bOverride[0])
    {
        return;
    }

    if (strlen(szText))
    {
        OSD_AddText(szText, Size, -1, -1, OSDBACK_LASTONE, OSD_XPOS_RIGHT, 0.9, 0.1,0);
        OSD_Show(hWnd, OSD_AUTOHIDE, 0,OSD_TIMER_ID);
        if (bAutoHide)
        {
            IdxCurrentScreen = -1;
        }
        bRestoreScreen = (IdxCurrentScreen != -1);
    }
    else
    {
        // If OSD message is blank, kill previous OSD message
        OSD_Clear(hWnd,0);
    }
}

//---------------------------------------------------------------------------
// Displayed specified OSD text without autohide timer.
// Stays on screen until a new OSD message replaces current OSD message.
void OSD_ShowTextPersistent(HWND hWnd, LPCTSTR szText, double Size)
{
    if (!Enable_OSD) return;
    if (bOverride[0])
    {
        return;
    }

    if (strlen(szText))
    {
        OSD_AddText(szText, Size, -1, -1, OSDBACK_LASTONE, OSD_XPOS_RIGHT, 0.9, 0.1,0);
        OSD_Show(hWnd, OSD_PERSISTENT, 0, OSD_TIMER_ID);
        if (bAutoHide)
        {
            IdxCurrentScreen = -1;
        }
        bRestoreScreen = (IdxCurrentScreen != -1);
    }
    else
    {
        // If OSD message is blank, kill previous OSD message
        OSD_Clear(hWnd,0);
    }
}

//---------------------------------------------------------------------------
// Override all previous OSD text, and force this current OSD text
// to override all other OSD text showings (done by the above functions).
// This is useful for external programs to override DScaler's own OSD text
// for its own controls.
void OSD_ShowTextOverride(HWND hWnd, LPCTSTR szText, double Size)
{
    bOverride[0] = FALSE;
    OSD_ShowText(hWnd, szText, Size);
    if (strlen(szText))
    {
        bOverride[0] = TRUE;
    }
}


void OSD_GeneralInfo()
{
	char tempChar[255];
	double  dfMargin = 0.02;    // 2% of screen height/width
	int	Size = 5;
	int Line=0;
	int i=0;

	OSD_Clear(hWnd,0);
	sprintf (tempChar, "K!TV XP v.%s Copyright (C) 2000-2002 Quenotte", STRPRODUCTVER);
    OSD_AddText(tempChar, 6, OSD_COLOR_TITLE, -1, OSDBACK_LASTONE, OSD_XPOS_CENTER, 0.5, OSD_GetLineYpos(Line, dfMargin, Size),0);
    
	// Carte 
 	OSD_AddText(Langue_GetString(ID_OSD_CARD), Size, OSD_COLOR_SECTION, -1, OSDBACK_LASTONE, OSD_XPOS_LEFT, dfMargin, OSD_GetLineYpos (Line+=3, dfMargin, Size),0);
 	OSD_AddText(ParametreCarte[CardType].Name, Size, -1, -1, OSDBACK_LASTONE, OSD_XPOS_LEFT, dfMargin, OSD_GetLineYpos (++Line, dfMargin, Size),0);
	// Source
 	OSD_AddText(Langue_GetString(ID_OSD_SRC), Size, OSD_COLOR_SECTION, -1, OSDBACK_LASTONE, OSD_XPOS_LEFT, dfMargin, OSD_GetLineYpos (Line+=2, dfMargin, Size),0);
	OSD_AddText(ChaineVideoSource[VideoSource], Size, -1, -1, OSDBACK_LASTONE, OSD_XPOS_LEFT, dfMargin, OSD_GetLineYpos(++Line, dfMargin, Size),0);
	OSD_AddText(TVSettings[TVTYPE].Name, Size, -1, -1, OSDBACK_LASTONE, OSD_XPOS_LEFT, dfMargin, OSD_GetLineYpos(++Line, dfMargin, Size),0);
	if(VideoSource==0)
	    OSD_AddText(Program[CurrentProgram].Name, Size, -1, -1, OSDBACK_LASTONE, OSD_XPOS_LEFT, dfMargin, OSD_GetLineYpos(++Line, dfMargin, Size),0);
	

	//Plugin Video
	OSD_AddText(Langue_GetString(ID_OSD_VIDEO_PLUGIN), Size, OSD_COLOR_SECTION, -1, OSDBACK_LASTONE, OSD_XPOS_LEFT, dfMargin, OSD_GetLineYpos (Line=14, dfMargin, Size),0);
	if(PluginVideoPresent)	strcpy(tempChar,PluginVideo->description);
	else	strcpy(tempChar,Langue_GetString(ID_NOTHING));
	OSD_AddText(tempChar, Size, -1, -1, OSDBACK_LASTONE, OSD_XPOS_LEFT, dfMargin, OSD_GetLineYpos(++Line, dfMargin, Size),0);

	//Plugin Audio	
	OSD_AddText(Langue_GetString(ID_OSD_AUDIO_PLUGIN), Size, OSD_COLOR_SECTION, -1, OSDBACK_LASTONE, OSD_XPOS_LEFT, dfMargin, OSD_GetLineYpos (Line+=2, dfMargin, Size),0);
	if(PluginAudioPresent)	strcpy(tempChar,PluginAudio->description);
	else	strcpy(tempChar,Langue_GetString(ID_NOTHING));
	OSD_AddText(tempChar, Size, -1, -1, OSDBACK_LASTONE, OSD_XPOS_LEFT, dfMargin, OSD_GetLineYpos(++Line, dfMargin, Size),0);

	//Affichage	
	OSD_AddText(Langue_GetString(ID_OSD_DISPLAY), Size, OSD_COLOR_SECTION, -1, OSDBACK_LASTONE, OSD_XPOS_RIGHT, 1 - dfMargin, OSD_GetLineYpos (Line=3, dfMargin, Size),0);
    if(ModeOverlay) strcpy(tempChar,"Overlay");
	else strcpy(tempChar,"DIB");
	OSD_AddText(tempChar, Size, -1, -1, OSDBACK_LASTONE, OSD_XPOS_RIGHT, 1 - dfMargin, OSD_GetLineYpos (++Line, dfMargin, Size),0);

	//Dsentrelacement	
	if(ModeOverlay)
	{
		OSD_AddText(Langue_GetString(ID_OSD_DI), Size, OSD_COLOR_SECTION, -1, OSDBACK_LASTONE, OSD_XPOS_RIGHT, 1 - dfMargin, OSD_GetLineYpos (Line+=2, dfMargin, Size),0);
		if(DeInterlace==0) strcpy(tempChar,"Pas de dsentrelacement");
		else if(DeInterlace==1) strcpy(tempChar,"Dsentrelacement Hardware 1");
		else if(DeInterlace==2) strcpy(tempChar,"Dsentrelacement Hardware 2");
		else strcpy(tempChar,DScalerMethod->szName);
		OSD_AddText(tempChar, Size, -1, -1, OSDBACK_LASTONE, OSD_XPOS_RIGHT, 1 - dfMargin, OSD_GetLineYpos(++Line, dfMargin, Size),0);
	}
	
	//MSP	
	if(Has_MSP)
	{
		OSD_AddText(Langue_GetString(ID_OSD_MSP), Size, OSD_COLOR_SECTION, -1, OSDBACK_LASTONE, OSD_XPOS_RIGHT, 1 - dfMargin, OSD_GetLineYpos (Line+=2, dfMargin, Size),0);
		OSD_AddText(MSPVersion, Size, -1, -1, OSDBACK_LASTONE, OSD_XPOS_RIGHT, 1 - dfMargin, OSD_GetLineYpos (++Line, dfMargin, Size),0);
	}
	Line=14;
	GetLocalTime(&LocalTime);
	sprintf (tempChar, "%d:%02d", LocalTime.wHour, LocalTime.wMinute);
    OSD_AddText(tempChar, Size+4, OSD_COLOR_TITLE, -1, OSDBACK_LASTONE, OSD_XPOS_RIGHT, 1-dfMargin, OSD_GetLineYpos(Line++, dfMargin, Size),0);

	// Info thread d'affichage :
	OSD_AddText("Statistics :", Size, OSD_COLOR_SECTION, -1, OSDBACK_LASTONE, OSD_XPOS_RIGHT, 1-dfMargin, OSD_GetLineYpos (++Line, dfMargin, Size),0);
	sprintf(tempChar, "Plugin: %d ms", counter1.GetElapsedTime());
	OSD_AddText(tempChar, Size, -1, -1, OSDBACK_LASTONE, OSD_XPOS_RIGHT, 1-dfMargin, OSD_GetLineYpos (++Line, dfMargin, Size),0);
	sprintf(tempChar, "Copy  : %d ms", counter2.GetElapsedTime());
	OSD_AddText(tempChar, Size, -1, -1, OSDBACK_LASTONE, OSD_XPOS_RIGHT, 1-dfMargin, OSD_GetLineYpos (++Line, dfMargin, Size),0);
	sprintf(tempChar, "Display : %d ms", counter3.GetElapsedTime());
	OSD_AddText(tempChar, Size, -1, -1, OSDBACK_LASTONE, OSD_XPOS_RIGHT, 1-dfMargin, OSD_GetLineYpos (++Line, dfMargin, Size),0);


	OSD_Show(hWnd, OSD_AUTOHIDE, 0,OSD_TIMER_ID);
	KillTimer(hWnd, OSD_TIMER_ID);
	SetTimer(hWnd, OSD_TIMER_ID, OSD_TIMER_DELAY2, NULL);
	if (bAutoHide)
    {
		IdxCurrentScreen = -1;
    }
    bRestoreScreen = (IdxCurrentScreen != -1);
}


void OSD_ShowText_Top(HWND hWnd, LPCTSTR szText)
{
    if (!Enable_OSD) return;
    if (bOverride[1])
    {
        return;
    }

    if (strlen(szText))
    {
        OSD_AddText(szText, 0, -1, -1, OSDBACK_LASTONE, OSD_XPOS_RIGHT, 0.93, 0.08,OSD_MAX_TEXT);
        OSD_Show(hWnd, OSD_AUTOHIDE, 0,OSD_TIMER_ID_TOP);
        if (bAutoHide)
        {
            IdxCurrentScreen = -1;
        }
        bRestoreScreen = (IdxCurrentScreen != -1);
    }
    else
    {
        // If OSD message is blank, kill previous OSD message
        OSD_Clear(hWnd,1);
    }
}


void OSD_ShowText_TopMini(HWND hWnd, LPCTSTR szText, double dSize, BOOL bPersistent)
{
    if (!Enable_OSD) return;
    
	if (bOverride[2])
    {
        return;
    }
	
    if (strlen(szText))
    {
		OSD_AddText(szText, (dSize==0)?6:dSize, -1, -1, OSDBACK_LASTONE, OSD_XPOS_RIGHT, 0.93, 0.19,OSD_MAX_TEXT+1);
		OSD_Show(hWnd, bPersistent?OSD_PERSISTENT:OSD_AUTOHIDE, 0,OSD_TIMER_ID_TOP_MINI);

		
		if (bAutoHide)
        {
            IdxCurrentScreen = -1;
        }
        bRestoreScreen = (IdxCurrentScreen != -1);
		if(bPersistent)	bOverride[2]=TRUE;
    }
    else
    {
        // If OSD message is blank, kill previous OSD message
        OSD_Clear(hWnd,2);
    }
}


void OSD_ShowText_Center(HWND hWnd, LPCTSTR szText)
{
    if (!Enable_OSD) return;

    if (bOverride[3])
    {
        return;
    }

    if (strlen(szText))
    {
        OSD_AddText(szText, 0, -1, -1, OSDBACK_LASTONE, OSD_XPOS_CENTER, 0.5, 0.87,OSD_MAX_TEXT+2);
        OSD_Show(hWnd, OSD_AUTOHIDE, 0, OSD_TIMER_ID_CENTER);
        if (bAutoHide)
        {
            IdxCurrentScreen = -1;
        }
        bRestoreScreen = (IdxCurrentScreen != -1);
    }
    else
    {
        // If OSD message is blank, kill previous OSD message
        OSD_Clear(hWnd,3);
    }
}


//---------------------------------------------------------------------------
// Clear currently displayed OSD
void OSD_Clear(HWND hWnd, int Type)
{
    int i;

    KillTimer(hWnd, OSD_TIMER_ID+Type);
    KillTimer(hWnd, OSD_TIMER_REFRESH_ID);
    bOverride[Type] = FALSE;
	if(Type==0)
	{
		if(NbText==0) NbText=64;
		for (i = 0 ; i < NbText ; i++)
		{
			strcpy(grOSD[i].szText,"");
			InvalidateRect(hWnd, &(grOSD[i].CurrentRect), FALSE);
        }
        OSD_ClearAllTexts();
	}
	else if(Type==1)
	{
		strcpy(grOSD[OSD_MAX_TEXT].szText,"");
		InvalidateRect(hWnd, &(grOSD[OSD_MAX_TEXT].CurrentRect), FALSE);
	}
	else if(Type==2)
	{
		strcpy(grOSD[OSD_MAX_TEXT+1].szText,"");
        InvalidateRect(hWnd, &(grOSD[OSD_MAX_TEXT+1].CurrentRect), FALSE);
	}
	else
	{
		strcpy(grOSD[OSD_MAX_TEXT+2].szText,"");
        InvalidateRect(hWnd, &(grOSD[OSD_MAX_TEXT+2].CurrentRect), FALSE);
	}
    IdxCurrentScreen = -1;
}

//---------------------------------------------------------------------------
// OSD Redrawing code.  Can be called from a paint event.
void OSD_Redraw(HWND hWnd, HDC hDC)
{
    CHAR        szCurrentFont[64];
    HFONT       hTmp, hOSDfont, hOSDfontOutline;
    int         nLen, nFontsize;
    int         nXpos, nYpos;
    int         nXWinSize, nYWinSize;
    TEXTMETRIC  tmOSDFont;
    SIZE        sizeText;
    RECT        winRect;
    DWORD       dwQuality = NONANTIALIASED_QUALITY;
    int         i;

    nLen = strlen(grOSD[0].szText)+strlen(grOSD[OSD_MAX_TEXT].szText)+strlen(grOSD[OSD_MAX_TEXT+1].szText)+strlen(grOSD[OSD_MAX_TEXT+2].szText);
    if (nLen && hDC != NULL)
    {
        GetClientRect(hWnd,&winRect);
        nXWinSize = winRect.right  - winRect.left;
        nYWinSize = winRect.bottom - winRect.top;

		if ( bShowMenu && bDisplayStatusBar && !bIsFullScreen)
        {
            nYWinSize -= StatusBar_Height();
        }

        for (i = 0 ; i < OSD_MAX_TEXT+3 ; i++)
        {
            if(strlen(grOSD[i].szText)==0)	continue;
			
            nFontsize = (int)((double)nYWinSize * (grOSD[i].Size / 100.00));

            // Set specified font
            if(bAntiAlias)
            {
                dwQuality = ANTIALIASED_QUALITY;
            }

            strcpy(szCurrentFont, szFontName);
            hOSDfont = CreateFont(nFontsize, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, dwQuality, DEFAULT_PITCH | FF_DONTCARE, szFontName);
            if (!hOSDfont)
            {
                // Fallback to Arial
                strcpy(szCurrentFont, "Arial");
                hOSDfont = CreateFont(nFontsize, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, dwQuality, VARIABLE_PITCH | FF_SWISS, szCurrentFont);
                if (!hOSDfont)
                {
                    // Otherwise, fallback to any available font
                    strcpy(szCurrentFont, "");
                    hOSDfont = CreateFont(nFontsize, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, dwQuality, VARIABLE_PITCH | FF_SWISS, szCurrentFont);
                }
            }
            if (!hOSDfont)
            {
                RealErrorBox(Langue_GetString(ID_ERROR_FONT));
            }
            hOSDfontOutline = CreateFont(nFontsize, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, NONANTIALIASED_QUALITY, DEFAULT_PITCH | FF_DONTCARE, szCurrentFont);
            if (!hOSDfontOutline)
            {
                hOSDfontOutline = hOSDfont;
            }

            hTmp = (HFONT)SelectObject(hDC, hOSDfontOutline);
            if (hTmp)
            {
                GetTextMetrics(hDC, &tmOSDFont);
                GetTextExtentPoint32(hDC, grOSD[i].szText, strlen(grOSD[i].szText), &sizeText);

                switch (grOSD[i].TextXPos)
                {
                case OSD_XPOS_RIGHT:
                    nXpos = (int)((double)nXWinSize * grOSD[i].XPos) - sizeText.cx;
                    break;
                case OSD_XPOS_CENTER:
                    nXpos = (int)((double)nXWinSize * grOSD[i].XPos - (double)sizeText.cx / 2.0);
                    break;
                case OSD_XPOS_LEFT:
                default:
                    nXpos = (int)((double)nXWinSize * grOSD[i].XPos);
                    break;
                }

                nYpos = (int)((double)nYWinSize * grOSD[i].YPos);

                // Draw the requested background for the text
                switch(grOSD[i].BackgroundMode)
                {
                case OSDB_TRANSPARENT:
                    SetBkMode(hDC, TRANSPARENT);
                    SetBkColor(hDC, grOSD[i].BackgroundColor);
                    break;
            
                case OSDB_BLOCK:
                    SetBkMode(hDC, OPAQUE);
                    SetBkColor(hDC, grOSD[i].BackgroundColor);
                    break;
            
                case OSDB_SHADED:
                    {
                        HBRUSH hBrush;
                        HBRUSH hBrushOld;
                        HBITMAP hBM;
                        WORD bBrushBits[8] = {0xAAAA, 0x5555, 0xAAAA, 0x5555, 0xAAAA, 0x5555, 0xAAAA, 0x5555, };
                        SetBkMode(hDC, TRANSPARENT);
                        SetTextColor(hDC, grOSD[i].BackgroundColor);
						SetBkColor(hDC, OverlayColor);
                        hBM = CreateBitmap(8, 8, 1, 1, (LPBYTE)bBrushBits); 
                        hBrush = CreatePatternBrush(hBM); 
                        hBrushOld = (HBRUSH)SelectObject(hDC, hBrush);
                        if(bOutline)
                        {
                            PatBlt(hDC, nXpos - 2, nYpos - 2, sizeText.cx + 4, sizeText.cy + 4, PATCOPY);
                        }
                        else
                        {
                            PatBlt(hDC, nXpos, nYpos, sizeText.cx, sizeText.cy, PATCOPY);
                        }
                        SelectObject(hDC, hBrushOld);
                        DeleteObject(hBrush);
                        DeleteObject(hBM);
                    }
                    break;
                default:
                    break;
                }

                if(bOutline)
                {
                    // Draw OSD outline if required
                    SetTextColor(hDC, OutlineColor);
                    TextOut(hDC, nXpos - 2, nYpos, grOSD[i].szText, strlen(grOSD[i].szText));
                    TextOut(hDC, nXpos + 2, nYpos, grOSD[i].szText, strlen(grOSD[i].szText));
                    TextOut(hDC, nXpos, nYpos - 2, grOSD[i].szText, strlen(grOSD[i].szText));
                    TextOut(hDC, nXpos, nYpos + 2, grOSD[i].szText, strlen(grOSD[i].szText));
                    TextOut(hDC, nXpos - 1, nYpos - 1, grOSD[i].szText, strlen(grOSD[i].szText));
                    TextOut(hDC, nXpos + 1, nYpos - 1, grOSD[i].szText, strlen(grOSD[i].szText));
                    TextOut(hDC, nXpos - 1, nYpos + 1, grOSD[i].szText, strlen(grOSD[i].szText));
                    TextOut(hDC, nXpos + 1, nYpos + 1, grOSD[i].szText, strlen(grOSD[i].szText));
                }

                // Draw OSD text
                if (SelectObject(hDC, hOSDfont))
                {
                    SetTextColor(hDC, grOSD[i].TextColor);
                    SetBkColor(hDC, grOSD[i].BackgroundColor);
                    TextOut(hDC, nXpos, nYpos, grOSD[i].szText, strlen(grOSD[i].szText));

                    // Calculate rectangle for the entire OSD 
                    // so we do not invalidate the entire window to remove it.
                    SIZE sz;
                    GetTextExtentExPoint(hDC, grOSD[i].szText, strlen(grOSD[i].szText), 
                                            32000, NULL, NULL, &sz);
                    grOSD[i].CurrentRect.left = nXpos-4; if (grOSD[i].CurrentRect.left < 0) grOSD[i].CurrentRect.left = 0;
                    grOSD[i].CurrentRect.right = nXpos + sz.cx + 4;
                    grOSD[i].CurrentRect.top = nYpos-4; if (grOSD[i].CurrentRect.top < 0) grOSD[i].CurrentRect.top = 0;
                    grOSD[i].CurrentRect.bottom = nYpos + sz.cy + 4;
                }

                SelectObject(hDC, hTmp);
                DeleteObject(hOSDfont);
                DeleteObject(hOSDfontOutline);
            }           
        }
    }
}
