/////////////////////////////////////////////////////////////////////////////
// Thread.c
/////////////////////////////////////////////////////////////////////////////
// Copyright (c) 2000-2002 Quenotte  All rights reserved.
/////////////////////////////////////////////////////////////////////////////
//
//	This file is subject to the terms of the GNU General Public License as
//	published by the Free Software Foundation.  A copy of this license is
//	included with this software distribution in the file COPYING.  If you
//	do not have a copy, you may obtain a copy by writing to the Free
//	Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
//
//	This software is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details
/////////////////////////////////////////////////////////////////////////////
//
// This software was based on MultiDec 5.9 Those portions are
// Copyright (C) 1999/2000 Espresso (echter_espresso@hotmail.com)
//
/////////////////////////////////////////////////////////////////////////////
#include "stdafx.h"
#include "resource.h"

#include "DScaler/DS_ApiCommon.h"
#include "K!TV-XP.h"
#include "cpu.h"
#include "bt848.h"
#include "ddraw.h"
#include "thread.h"
#include "coderisc.h"
#include "hardwaremisc.h"
#include "plugin.h"
#include "vbi-vt.h"
#include "misc_asm.h"


#define MAXLINES 288

static short *OutEvenLines[MAXLINES], *OutOddLines[MAXLINES];
static short *EvenLines[5][MAXLINES], *OddLines[5][MAXLINES];

int ThreadClassId=1;
static int nFrame;
static DWORD   dwLastCount = 0;
static int BytesOut, MovieBoostLocal;
static HDC  DIB_Hdc;
static HANDLE OutThread;
volatile BOOL StopDisplayThread = FALSE;
HANDLE Thread_Event;
CRITICAL_SECTION ThreadCrit;
Counter counter1, counter2, counter3;

extern BITMAPINFO *pDIB;
extern unsigned short  *pDIBMemory;


static void *(__cdecl *AfficherLigne) (void *, void *, size_t);


////////////////////////////////////////////////////////////////////////////////////
// Phase d'initialisation.
////////////////////////////////////////////////////////////////////////////////////
void InitThread()
{
	int i, j;
	int TabPrioriteThread[5] = {
		THREAD_PRIORITY_BELOW_NORMAL,	THREAD_PRIORITY_NORMAL,
		THREAD_PRIORITY_ABOVE_NORMAL,	THREAD_PRIORITY_HIGHEST,
		THREAD_PRIORITY_TIME_CRITICAL 
	};

	i=SetThreadAffinityMask (GetCurrentThread(), 1<<(DisplayProcessor));         
	SetThreadPriority(GetCurrentThread(), TabPrioriteThread[ThreadClassId]);

	memset(OutEvenLines, 0, sizeof(short *) * MAXLINES);
	memset(OutOddLines , 0, sizeof(short *) * MAXLINES);
	dwLastCount = GetTickCount();
	nFrame = CurrentFrame = 0;
	BytesOut = 2 * CurrentX;
	
	//
	// Initialisation des pointeurs vers les buffers sources
	//
	for ( j= 0 ; j < 5 ; j++ )
	{
		for ( i = 0; i < CurrentY ; i+=2 )
		{
			EvenLines[j][(i/2)] = (short *)pDisplay[j] + (i * 1024);
			OddLines [j][(i/2)] = (short *)pDisplay[j] + (i+1) * 1024;
		}
	}

	// Initialisation du mode d'affichage
	if (( ModeOverlay ) && ( Can_Overlay ) && ( lpDDOverlay != NULL ))
	{
		Affichage_Overlay_Clean();
		// Pas de conversion (copie directe)
		AfficherLigne = (void *(__cdecl *)(void *, void *, size_t))memcpy; //memcpyMMX;	
	}
	else 
	{
		// Affichage DIB si chec du mode Overlay ou si l'on souhaite travailler en DIB
		OutDib = TRUE; ModeOverlay = FALSE;
		memset(pDIBMemory, 0, CurrentX * CurrentY * 2);
		j=CurrentY;

		for ( i=0; i< MAXLINES; i++ )
		{
			j--;
			if (j < 0) break;
			OutEvenLines[i]=(short*)&pDIBMemory[j*(BytesOut/2)];
			j--;
			if (j < 0) break;
			OutOddLines[i]=(short*)&pDIBMemory[j*(BytesOut/2)];
		}
		AfficherLigne = (void *(__cdecl *)(void *, void *, size_t))Affichage_RGB15;
	}

	if(DeInterlace==2)
	{
		// Movieboost desacive pour DI HW2 (Scaler Bob)
		MovieBoostLocal=0;
	}
	else
	{
		if (IsPluginVideoRunning)
			MovieBoostLocal = MovieBoost;
		else
			// Ncessaire pour le mode DIB qui ne capture par toujours 
			// 768 lignes en hauteur
			MovieBoostLocal = ((CurrentY/2) * MovieBoost/288);
	}

	BT8X8_Restart_RISC_Code();
	BT8X8_SetDMA(TRUE);

	// Initialisation du plugin DScaler
	memset(&DIinfo, 0, sizeof(DIinfo));
    DIinfo.CpuFeatureFlags = CpuFeatureFlags;
	DIinfo.Overlay		= (BYTE*)lpOverlay + 2* MovieBoostLocal * OverlayPitch;
    DIinfo.OverlayPitch = 0;
	DIinfo.pMemcpy		= memcpyMMX;


	// Cette initialisation est ncessaire meme si le plugin ne tourne pas
	// Il peut avoir besoin de ces informations.
	if (PluginVideoPresent)
	{
		PluginVideo->hwndParent = hWnd;
		PluginVideo->CurrentX = CurrentX;
		PluginVideo->FullField = FullField;
		PluginVideo->MovieBoost = MovieBoostLocal;
		PluginVideo->DeInterlace = PluginDScalerPresent;
		PluginVideo->bIsBT878 = bIsBT878;

		memcpyMMX(PluginVideo->EvenLines, EvenLines, sizeof(EvenLines));
		memcpyMMX(PluginVideo->OddLines , OddLines, sizeof(OddLines));

		memcpyMMX(PluginVideo->OutEvenLines, EvenLines, sizeof(EvenLines[0]));
		memcpyMMX(PluginVideo->OutOddLines , OddLines, sizeof(OddLines[0]));

		// Initialisation si le plugin est actif
		if (IsPluginVideoRunning)
			PluginVideo->InitStuff();
	}
}

// Lors du Lock (Affichage_Overlay_Lock) lpOverlay peut changer
// Dans ce cas mettre a jour les pointeurs vidos OutEvenLines/OutOddLines
static void UpdateDisplayPointers()
{
	int i;
	
	if (DeInterlace<2 || (IsPluginVideoRunning && !FullField)) 
	{
		memset(OutEvenLines, 0, sizeof(short *) * MAXLINES);
		memset(OutOddLines , 0, sizeof(short *) * MAXLINES);

		if (IsPluginVideoRunning && ! FullField)
		{
			for ( i=0; i < CurrentY/2; i++ )
				OutEvenLines[i] = (short*)lpOverlay + i * (OverlayPitch/2);
		}
		else 
		{
			for ( i=0; i < CurrentY/2; i++ )
			{
				OutEvenLines[i] = (short*)lpOverlay + (2*i)   * (OverlayPitch/2);
				OutOddLines [i] = (short*)lpOverlay + (2*i+1) * (OverlayPitch/2);
			}
		}
	}
	// maj des infos overlays (ils changent lors du Lock)
	DIinfo.Overlay		= (BYTE*)lpOverlay + 2* MovieBoostLocal * OverlayPitch;
	DIinfo.OverlayPitch = OverlayPitch;
}


// Fin de boucle
// Decodage du VBI, affichage des fps...
static void EstimateFPS()
{
	if ( Capture_VBI ) VBI_Decode();
	ReconnaissanceSignal();

	if (dwLastCount+1000 <= GetTickCount())
	{
		char szBuffer[20];

		if (IsPluginVideoRunning) PluginVideo->CheckStuff();
		
		if(nFrame>50)	nFrame=50;
		sprintf(szBuffer, "%d ips", nFrame);
		StatusBar_Update(4, szBuffer); 
		nFrame=0;
		dwLastCount = GetTickCount();
	}
}


// Copie des trames vers le buffer vido correspondant.
static void UpdateBuffer(short *pSource[MAXLINES], BOOL bIsOdd)
{
	int i;
	short **pDest;
	
	if (bIsOdd)	pDest = OutOddLines;
	else		pDest = OutEvenLines;

	for ( i=1+MovieBoostLocal; i < CurrentY/2-MovieBoostLocal; i++ )
	{
		if (pDest[i] == 0) continue;
		AfficherLigne(pDest[i], pSource[i], BytesOut);
	}
}


// Affichage avec plugins de desentrelacement de DScaler
static BOOL Affichage_Deinterlace(DWORD dwFlipFlag,BOOL bIsOdd)
{
	DIinfo.LineLength = CurrentX * 2;
	DIinfo.FrameWidth = CurrentX;
	DIinfo.FrameHeight = CurrentY;
	DIinfo.FieldHeight = CurrentY / 2 - 2*MovieBoostLocal;
	DIinfo.CombFactor = -1;
	DIinfo.FieldDiff = -1;
	DIinfo.IsOdd = bIsOdd;
	Affichage_GetDestRect(&DIinfo.DestRect);
//	DIinfo.bDoAccurateFlips

	if (bIsOdd) 
	{
		memmove(&DIinfo.OddLines[1], &DIinfo.OddLines[0], sizeof(DIinfo.OddLines) - sizeof(DIinfo.OddLines[0]));
		if (IsPluginVideoRunning)
			DIinfo.OddLines[0] = &PluginVideo->OutOddLines[PluginVideo->CurrentFrame][MovieBoostLocal];
		else
			DIinfo.OddLines[0] = &OddLines[CurrentFrame][MovieBoostLocal];
	}
	else
	{
		memmove(&DIinfo.EvenLines[1], &DIinfo.EvenLines[0], sizeof(DIinfo.EvenLines) - sizeof(DIinfo.EvenLines[0]));
		if (IsPluginVideoRunning)
			DIinfo.EvenLines[0] = &PluginVideo->OutEvenLines[PluginVideo->CurrentFrame][MovieBoostLocal];
		else
			DIinfo.EvenLines[0] = &EvenLines[CurrentFrame][MovieBoostLocal];
	}
	DScalerMethod->pfnAlgorithm(&DIinfo);
	return TRUE;
}


// Affichage normal
static BOOL Affichage_Normal(DWORD *dwFlipFlag,BOOL bIsOdd, BOOL *bDoFlip, BOOL ViewBuffer)
{
	if (IsPluginVideoRunning) // Affichage si plugin video actif
	{
		// Travail sur buffer Even en mode HF
		if (!FullField)
			bIsOdd = FALSE;

		if (bIsOdd)
			UpdateBuffer(PluginVideo->OutOddLines[PluginVideo->CurrentFrame], TRUE);
		else
			UpdateBuffer(PluginVideo->OutEvenLines[PluginVideo->CurrentFrame], FALSE);

		// Le plugin dcide du Flip Overlay
		*bDoFlip = ViewBuffer;

		// Dsentrelacement Hardware 1 : 
		// Flip de l'overlay entre l'affichage des 2 trames
		if (FullField && DeInterlace==1)
		{
			*dwFlipFlag |= bIsOdd? DDFLIP_ODD: DDFLIP_EVEN;
			*bDoFlip = TRUE;
		}
	}
	else //Affichage si plugin video non actif
	{
		if (bIsOdd)
			UpdateBuffer(OddLines[CurrentFrame], bIsOdd);
		else
			UpdateBuffer(EvenLines[CurrentFrame], bIsOdd);

		// Dsentrelacement Hardware 1
		if (DeInterlace==1) 
		{
			*dwFlipFlag |= bIsOdd? DDFLIP_ODD : DDFLIP_EVEN;
			*bDoFlip = TRUE;
		}
		else 
			*bDoFlip = bIsOdd;
	}
	return TRUE;
}


// Fonction appele en cas de crash du Thread d'affichage
static void ReStartThread(const char *MessagePourLog)
{
	BT8X8_SetDMA(FALSE);
	LOG(1, MessagePourLog);
	PostMessage(hWnd, WM_COMMAND, IDM_AFFICHAGE_STOP, 0);
	PostMessage(hWnd, WM_COMMAND, IDM_AFFICHAGE_START, 0);
	ExitThread(1);
//	return 1;	
}


static DWORD WINAPI Affichage_Thread() 
{
	int height, width;
	BOOL IsOdd=FALSE, DoFlip=FALSE, bOverlayLocked;
	RECT Centre;
	DWORD FlipFlag;

	BOOL bWaitForFlip=TRUE;		// 
	BOOL ViewBuffer=FALSE;		// Afficher le buffer vido
	BOOL UpdateBuffer=TRUE;		// Mettre  jour le buffer vido


	// Calcul de diverses variables en dehors du thread
	GetClientRect(hWnd, &Centre);
	if ( bDisplayStatusBar ) Centre.bottom -= StatusBar_Height();
	height = abs(CurrentY - (Centre.bottom-Centre.top))/2;
	width  = (Centre.right-Centre.left)/2 - CurrentX/2;

	// Gestion exception boucle principale...
	__try
	{
		while (!StopDisplayThread)
		{
			// On ne fait rien lors des modifications de parametres...
			WaitForSingleObject(Thread_Event,INFINITE);

			// attente de la capture des trames
			IsOdd = WaitForNextField();
	
			// Le plugin peut avoir besoin de CurrentFrame meme s'il 
			// n'est pas encore lanc (notamment pour DetecteSignal).
			if (PluginVideoPresent)
				PluginVideo->CurrentFrame = CurrentFrame;

			nFrame++;

			// Si le plugin video est actif alors traitement
			counter1.Start();
			if (IsPluginVideoRunning) 
				PluginVideo->DoStuff(IsOdd, &UpdateBuffer, &ViewBuffer);
			counter1.Stop();

			// Si le buffer n'est pas modifi, pas d'affichage (c'est inutile !)
			// la variable ViewBuffer n'est donc pas pris en compte.
			if (IsPluginVideoRunning && !UpdateBuffer )
			{
				nFrame--;
				continue;
			}

			// Gestion exception partie affichage...
			__try
			{
				// Overlay : verrouillage de la surface et initialisation
				if ( ModeOverlay ) 
				{
					if(!Affichage_Overlay_Lock())
					{
						ReStartThread("Probleme dans Overlay_Lock");
                        return 1;
					}
                    bOverlayLocked = TRUE;
					UpdateDisplayPointers();
					FlipFlag = (bWaitForFlip)?DDFLIP_WAIT:DDFLIP_DONOTWAIT;
				}

				// Affichage avec les plugins de desentrelacement de DScaler
				counter2.Start();
				if ( (ModeOverlay && PluginDScalerPresent) && (!IsPluginVideoRunning || 
					(IsPluginVideoRunning && FullField ) ) )
				{
					
					DoFlip=TRUE;
					if(!Affichage_Deinterlace(FlipFlag,IsOdd))
					{
						ReStartThread("Probleme dans Affichage_Deinterlace");
						return 0;
					}
				}
				// Affichage normal
				else 
				{
					if(!Affichage_Normal(&FlipFlag,IsOdd,&DoFlip, ViewBuffer ))
					{
						ReStartThread("Probleme dans Affichage_Normal");
						ExitThread(1);
						return 0;
					}
				}
				counter2.Stop();

				// Overlay : Dverrouillage de la surface et flip si necessaire
				counter3.Start();
				if ( ModeOverlay ) 
				{
					if(!Affichage_Overlay_UnLock())
					{
						ReStartThread("Probleme dans Overlay_Unlock");
						ExitThread(1);
						return 0;
					}
					bOverlayLocked = FALSE;
				
					if (DoFlip)
					{
						if(!Affichage_Overlay_Flip(FlipFlag))
						{
							ReStartThread("Probleme dans Overlay_Flip");
							ExitThread(1);
							return 0;
						}
					}
				}
				else 
				// DIB : on effectue le rendu
				{
					SetDIBitsToDevice( DIB_Hdc, width, height, pDIB->bmiHeader.biWidth,CurrentY,0,0,0,CurrentY,pDIBMemory,pDIB,DIB_RGB_COLORS);
				}
				counter3.Stop();

			}
			// Gestion exception partie affichage...
            __except (EXCEPTION_EXECUTE_HANDLER) 
			{ 
				if (bOverlayLocked)
					Affichage_Overlay_UnLock();
				LOG(1, "Crash in output code");
			}

			// Decodage du VBI,affichage des ips...
			EstimateFPS();
		}
	}	
	// Gestion exception boucle principale...
    __except (EXCEPTION_EXECUTE_HANDLER) 
	{  
        LOG(1, "Crash dans boucle principale du thread d'affichage");
        ExitThread(1);
        return 0;
	}

    // On essaye d'arreter proprement la capture...
	__try
    {
        BT8X8_SetDMA(FALSE);
    }
    __except (EXCEPTION_EXECUTE_HANDLER) 
    { 
        LOG(1, "Crash dans SetDMA");
        ExitThread(1);
        return 0;
    }
    ExitThread(0);
	return 0;
}


// Demarre le thread d'affichage
void Start_Thread()
{
	DWORD LinkThreadID;
	EnterCriticalSection(&ThreadCrit);
	
	CurrentFrame = 0;
	DIB_Hdc=GetDC(hWnd);
	
		// Au cas ou le thread ne serait pas stopp...
	if ( OutThread != NULL )
	{
		LeaveCriticalSection(&ThreadCrit);
		Stop_Thread();
		EnterCriticalSection(&ThreadCrit);
	}
	StopDisplayThread = FALSE;
	InitThread(); 	  

	Thread_Event=CreateEvent(NULL,TRUE,FALSE,"THREAD_EVENT");

	OutThread=CreateThread ((LPSECURITY_ATTRIBUTES)NULL, (DWORD)0,
		(LPTHREAD_START_ROUTINE)Affichage_Thread, NULL, (DWORD)0, (LPDWORD)&LinkThreadID);
	SetEvent(Thread_Event);
	LeaveCriticalSection(&ThreadCrit);
	return;
}


// Arrete le thread d'affichage
void Stop_Thread()
{
	EnterCriticalSection(&ThreadCrit);
	if ( OutThread != NULL )
	{
		SetThreadPriority(OutThread,THREAD_PRIORITY_NORMAL);
		StopDisplayThread = TRUE;
		SetEvent(Thread_Event);//on reveille le Thread
		if(WaitForSingleObject(OutThread,2000)==WAIT_TIMEOUT) //Attend la fin du Thread pendant 2s
		{
			TerminateThread(OutThread,0);//on le kill si pb
		}
		CloseHandle(OutThread);
		CloseHandle(Thread_Event);
		OutThread=NULL;
		ReleaseDC(hWnd,DIB_Hdc);
		DIB_Hdc=NULL;
	}
	LeaveCriticalSection(&ThreadCrit);
}
