/////////////////////////////////////////////////////////////////////////////
// coderisc.c
/////////////////////////////////////////////////////////////////////////////
// Copyright (c) 2000-2002 Quenotte  All rights reserved.
/////////////////////////////////////////////////////////////////////////////
//
//	This file is subject to the terms of the GNU General Public License as
//	published by the Free Software Foundation.  A copy of this license is
//	included with this software distribution in the file COPYING.  If you
//	do not have a copy, you may obtain a copy by writing to the Free
//	Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
//
//	This software is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details
/////////////////////////////////////////////////////////////////////////////
//
// This software was based on MultiDec 5.9 Those portions are
// Copyright (C) 1999/2000 Espresso (echter_espresso@hotmail.com)
//
// Code RISC bas sur DScaler
// Copyright (c) 2000 John Adcock.  All rights reserved.
/////////////////////////////////////////////////////////////////////////////
#include "stdafx.h"
#include "resource.h"
#include "..\Plugins\ResCommun.h"

#include "hardwaremisc.h"
#include "coderisc.h"
#include "bt848.h"
#include "vbi-vt.h"
#include "K!TV-XP.h"
#include "bt_io.h"
#include "externe/dsdrv.h"


static long BytesPerRISCField = 1;
long Sleep_Interval = 1;   // How long to wait for BT chip (default=0 but CPU=99%)



// Indique la frame courante capture en se basant sur la position du code RISC
//
// Valeur de retour:
// 0  9: Numro de la frame en cours de capture
//
static int BT8X8_GetRISCPosAsInt()
{
	int CurrentPos = 10;

	while(CurrentPos > 9)
	{
		DWORD CurrentRiscPos = BT8X8_ReadDword(BT848_RISC_COUNT);
		CurrentPos = (CurrentRiscPos - RiscBasePhysical) / BytesPerRISCField;
	}
	return CurrentPos;
}


// Patiente pendant la capture de la frame 
// Adapt de DScaler.
BOOL WaitForNextField()
{
	BOOL bIsOddField;
	// Position Field sur l'image prcdente (pour comparer avec l'image actuelle)
	static int OldPos = 0;	

	while(OldPos == BT8X8_GetRISCPosAsInt())
		Sleep(Sleep_Interval);

	OldPos = BT8X8_GetRISCPosAsInt();
	switch(OldPos)
	{
		case 0: bIsOddField = TRUE;  CurrentFrame = 4; break;
		case 1: bIsOddField = FALSE; CurrentFrame = 0; break;
		case 2: bIsOddField = TRUE;  CurrentFrame = 0; break;
		case 3: bIsOddField = FALSE; CurrentFrame = 1; break;
		case 4: bIsOddField = TRUE;  CurrentFrame = 1; break;
		case 5: bIsOddField = FALSE; CurrentFrame = 2; break;
		case 6: bIsOddField = TRUE;  CurrentFrame = 2; break;
		case 7: bIsOddField = FALSE; CurrentFrame = 3; break;
		case 8: bIsOddField = TRUE;  CurrentFrame = 3; break;
		case 9: bIsOddField = FALSE; CurrentFrame = 4; break;
	} 
	return bIsOddField;
}

//
// Code commun de capture des donnes VBI 
static void BT8X8_CreateRiscCode4VBI(DWORD **pRiscCode, int nField)
{
	int nLine;
	LPBYTE pUser;
	PHYS pPhysical;
	DWORD GotBytesPerLine;
	int CurrentVBILines;

	if ( Capture_VBI ) 
	{
		*((*pRiscCode)++) = (DWORD) (BT848_RISC_SYNC | BT848_FIFO_STATUS_FM1);
		*((*pRiscCode)++) = 0;
		CurrentVBILines = TVSettings[TVTYPE].VBILines;	// Nb lignes pour VBI (selon norme PAL/SECAM ...)
		
		pUser = pVBILines[nField / 2];
		
		// Pour le secam, nous capturons les 16 premires lignes de la frame paire
		// suivit des lignes suivantes provenant de la frame impaire.
		// Par le dcodage on aura : for (line=0; line < 2 * TVSettings[TVTYPE].VBILines; line++)
		if(nField & 1)
			pUser += CurrentVBILines * 2048;
		
		for (nLine = 0; nLine < CurrentVBILines; nLine++)
		{
			pPhysical = GetPhysicalAddress(Vbi_dma[nField / 2], pUser, VBI_SPL, &GotBytesPerLine);
			if(pPhysical == 0 || VBI_SPL > GotBytesPerLine)
			{
				return;
			}
			*((*pRiscCode)++) = BT848_RISC_WRITE | BT848_RISC_SOL | BT848_RISC_EOL | VBI_SPL;
			*((*pRiscCode)++) = pPhysical;
			pUser += 2048;
		}
	}

}


// Creates the RISC code
// First syncs to field
// Then waits for data
// Then tells the bt848 where to put each line of data
void BT8X8_CreateRiscCode(int SizeX, int SizeY)
{
	DWORD *pRiscCode;
	int nField;
	int nLine;
	LPBYTE pUser;
	PHYS pPhysical;
	DWORD GotBytesPerLine;
	DWORD BytesPerLine = 0;
	long Offset;

	pRiscCode = (DWORD *)Risc_dma->dwUser;

	// we create the RISC code for 10 fields
	// the first one (0) is even
	// last one (9) is odd
	for(nField = 0; nField < 10; nField++)
	{
		// First we sync onto either the odd or even field
		if(nField & 1)
		{
			*(pRiscCode++) = (DWORD) (BT848_RISC_SYNC | BT848_RISC_RESYNC | BT848_FIFO_STATUS_VRO);
		}
		else
		{
			*(pRiscCode++) = (DWORD) (BT848_RISC_SYNC | BT848_RISC_RESYNC | BT848_FIFO_STATUS_VRE  | ((0xF1 + nField / 2) << 16));
		}
		*(pRiscCode++) = 0;

		BT8X8_CreateRiscCode4VBI(&pRiscCode, nField);

		*(pRiscCode++) = (DWORD) (BT848_RISC_SYNC | BT848_FIFO_STATUS_FM1);
		*(pRiscCode++) = 0;

		// work out the position of the first line, first line is line zero an even line
		pUser = pDisplay[nField / 2];
		Offset=2048;		// E-nek, ne pas dplacer

		if(nField & 1)
			pUser += Offset;
		BytesPerLine = SizeX * 2;
		for (nLine = 0; nLine < SizeY /2; nLine++)
		{

			pPhysical = GetPhysicalAddress(Display_dma[nField / 2], pUser, BytesPerLine, &GotBytesPerLine);
			if(pPhysical == 0 || BytesPerLine > GotBytesPerLine)
			{
				return;
			}
			*(pRiscCode++) = BT848_RISC_WRITE | BT848_RISC_SOL | BT848_RISC_EOL | BytesPerLine;
			*(pRiscCode++) = pPhysical;
			// since we are doing all the lines of the same
			// polarity at the same time we skip two lines
			pUser += 2*Offset;
		}
	}

	BytesPerRISCField = ((long)pRiscCode - (long)Risc_dma->dwUser) / 10;
	*(pRiscCode++) = BT848_RISC_JUMP;
	*(pRiscCode++) = RiscBasePhysical;

	BT8X8_WriteDword(BT848_RISC_STRT_ADD, RiscBasePhysical);
}
