/////////////////////////////////////////////////////////////////////////////
// ddraw.c
/////////////////////////////////////////////////////////////////////////////
// Copyright (c) 2000-2002 Quenotte  All rights reserved.
/////////////////////////////////////////////////////////////////////////////
//
//	This file is subject to the terms of the GNU General Public License as
//	published by the Free Software Foundation.  A copy of this license is
//	included with this software distribution in the file COPYING.  If you
//	do not have a copy, you may obtain a copy by writing to the Free
//	Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
//
//	This software is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details
/////////////////////////////////////////////////////////////////////////////
//
// This software was based on MultiDec 5.9 Those portions are
// Copyright (C) 1999/2000 Espresso (echter_espresso@hotmail.com)
//
// Une grande partie des fonctions sont tires de Dscaler 3.1.0
// Copyright (c) 2000 John Adcock.  All rights reserved.
/////////////////////////////////////////////////////////////////////////////

#include <stdio.h>
#include <ddraw.h>
#include "plugin.h"

#include "resource.h"
#include "..\Plugins\ResCommun.h"
#include "K!TV-XP.h"
#include "ddraw.h"
#include "hardwaremisc.h"
#include "misc.h"

LPVOID lpOverlay=NULL;
static LPDIRECTDRAW	lpDD;
static LPDIRECTDRAWSURFACE lpDDSurface;
static LPDIRECTDRAWSURFACE lpDDOverlayBack = NULL;
LPDIRECTDRAWSURFACE lpDDOverlay;
CRITICAL_SECTION hDDCritSect;

BOOL Can_Overlay = FALSE, Can_ColorKey = FALSE, Can_Bob = FALSE;
int OverlayPitch;
static HRESULT FlipResult = 0;             // Need to try again for flip?

// For DDraw Clipping
COLORREF OverlayColor = RGB(32, 16, 16);
static COLORREF PhysicalOverlayColor = RGB(32, 16, 16);
static RECT destinationRectangle = {0,0,0,0};



//----------------------------------------------------------------------------
// Repaints the overlay colorkey, optionally with black borders around it
// during aspect ratio control
//-----------------------------------------------------------------------------
void Affichage_Overlay_PaintColorkey(HWND hWnd, BOOL bEnable, HDC hDC, RECT* PaintRect, BOOL bNoMiddlePainting)
{
    // MRS 9-9-00
    HBRUSH black = CreateSolidBrush(RGB(0,0,0));
    HBRUSH overlay;
    RECT r;
    RECT r2, winRect;

	if (bEnable && ((ModeOverlay && lpDDOverlay != NULL) ||	((!OutDib) &&  (!ModeOverlay) && lpDDSurface != NULL)))
    {
        overlay = CreateSolidBrush(GetNearestColor(hDC, OverlayColor));
    }
    else
    {
        overlay = CreateSolidBrush(RGB(0,0,0));
    }

    // MRS 2-22-01 - Reworked to fixup the rectangle rather than not draw piecemeal to help out defered overlay setting
    if (destinationRectangle.right < destinationRectangle.left) {
        int t = destinationRectangle.right;
        destinationRectangle.right = destinationRectangle.left;
        destinationRectangle.left = t;
    }
    if (destinationRectangle.bottom < destinationRectangle.top) {
        int t = destinationRectangle.top;
        destinationRectangle.top = destinationRectangle.bottom;
        destinationRectangle.bottom = t;
    }
    
    // Draw black in the 4 borders
    GetClientRect(hWnd,&winRect);
    
    //TJ 010508 make sure we dont paint over the statusbar
    //if we do, it will cause flashing
    if (bDisplayStatusBar)
        winRect.bottom -= StatusBar_Height();

    // Top
    r2.left = 0;
    r2.top = 0;
    r2.right = winRect.right;
    r2.bottom = destinationRectangle.top;
    IntersectRect(&r, &r2, PaintRect);
    FillRect(hDC, &r, black);

    // Bottom
    r2.left = 0;
    r2.top = destinationRectangle.bottom;
    r2.right = winRect.right;
    r2.bottom = winRect.bottom;
    IntersectRect(&r, &r2, PaintRect);
    FillRect(hDC, &r, black);

    // Left
    r2.left = 0;
    r2.top = 0;
    r2.right = destinationRectangle.left;
    r2.bottom = winRect.bottom;
    IntersectRect(&r, &r2, PaintRect);
    FillRect(hDC, &r, black);

    // Right
    r2.left = destinationRectangle.right;
    r2.top = 0;
    r2.right = winRect.right;
    r2.bottom = winRect.bottom;
    IntersectRect(&r, &r2, PaintRect);
    FillRect(hDC, &r, black);

/*    if (AspectSettings.OverlayNeedsSetting)
    { 
        // MRS 2-22-01
        // Intended to prevent purple flashing by setting overlay
        // after drawing black but before drawing purple.
        Overlay_Update(&AspectSettings.SourceRect, &AspectSettings.DestinationRectWindow, DDOVER_SHOW);
        AspectSettings.OverlayNeedsSetting = FALSE;
        // Wait till current frame is done before drawing purple...
        // Overlay changes do not seem to take place (at least on a GeForce)
        // until the VBI...so need to wait to avoid purple flashing

        // JA 29/6/2001 made wait for blank optional and swicthed off
        // the flashing has been much reduced by using dark grey as
        // overlay colour.  Also this may cause the pausing effect
        // on Teletext and CC
        if (lpDD != NULL && AspectSettings.bWaitForVerticalBlank == TRUE)
        {
            lpDD->WaitForVerticalBlank(DDWAITVB_BLOCKBEGIN, NULL);
        }
    }
*/
    // Draw overlay color in the middle.
    if(!bNoMiddlePainting)
    {
        IntersectRect(&r, &destinationRectangle, PaintRect);
        FillRect(hDC, &r, overlay);
    }

    DeleteObject(black);
    DeleteObject(overlay);
}



//-----------------------------------------------------------------------------
// Name: DDColorMatch()
// Desc: Convert a RGB color to a pysical color.
//       We do this by leting GDI SetPixel() do the color matching
//       then we lock the memory and see what it got mapped to.
//-----------------------------------------------------------------------------
static DWORD Affichage_Overlay_ColorMatch(LPDIRECTDRAWSURFACE pdds, COLORREF rgb)
{
    COLORREF rgbT;
    HDC hdc;
    DWORD dw = CLR_INVALID;
    DDSURFACEDESC ddsd;
    HRESULT hres;

    //
    //  Use GDI SetPixel to color match for us
    //
	hres = pdds->GetDC(&hdc);
    if (SUCCEEDED(hres))
    {
        rgbT = GetPixel(hdc, 0, 0);     // Save current pixel value
        SetPixel(hdc, 0, 0, rgb);       // Set our value
		pdds->ReleaseDC(hdc);
    }
    //
    // Now lock the surface so we can read back the converted color
    //
    ddsd.dwSize = sizeof(ddsd);
    hres = pdds->Lock(NULL, &ddsd, DDLOCK_WAIT, NULL);
    if (SUCCEEDED(hres))
    {
        dw = *(DWORD *) ddsd.lpSurface;                 // Get DWORD
        if (ddsd.ddpfPixelFormat.dwRGBBitCount < 32)
		{
            dw &= (1 << ddsd.ddpfPixelFormat.dwRGBBitCount) - 1;  // Mask it to bpp
		}
        pdds->Unlock(NULL);
    }
    //
    //  Now put the color that was there back.
    //
	hres = pdds->GetDC(&hdc);
    if (SUCCEEDED(hres))
    {
        SetPixel(hdc, 0, 0, rgbT);
        pdds->ReleaseDC(hdc);
    }
    return dw;
}


void Affichage_Overlay_Clean()
{
    unsigned int nPixel;
    unsigned int nLine;
    HRESULT ddrval;
    DDSURFACEDESC SurfaceDesc;
    unsigned short* pPixel;

    EnterCriticalSection(&hDDCritSect);
    if (lpDDOverlay != NULL)
    {
        memset(&SurfaceDesc, 0x00, sizeof(SurfaceDesc));
        SurfaceDesc.dwSize = sizeof(SurfaceDesc);

        ddrval = lpDDOverlay->Lock(NULL, &SurfaceDesc, DDLOCK_WAIT, NULL);

        for (nLine = 0; nLine < SurfaceDesc.dwHeight; nLine++)
        {
            pPixel = (unsigned short*)((BYTE*)SurfaceDesc.lpSurface + nLine * SurfaceDesc.lPitch);
            for (nPixel = 0; nPixel < SurfaceDesc.dwWidth; ++nPixel)
            {
                *pPixel = 0x8000;
                pPixel++;
            }
        }
        ddrval = lpDDOverlay->Unlock(SurfaceDesc.lpSurface);
    }
    if (lpDDOverlayBack != NULL)
    {
        memset(&SurfaceDesc, 0x00, sizeof(SurfaceDesc));
        SurfaceDesc.dwSize = sizeof(SurfaceDesc);

        ddrval = lpDDOverlayBack->Lock(NULL, &SurfaceDesc, DDLOCK_WAIT, NULL);

        for (nLine = 0; nLine < SurfaceDesc.dwHeight; nLine++)
        {
            pPixel = (unsigned short*)((BYTE*)SurfaceDesc.lpSurface + nLine * SurfaceDesc.lPitch);
            for (nPixel = 0; nPixel < SurfaceDesc.dwWidth; ++nPixel)
            {
                *pPixel = 0x8000;
                pPixel++;
            }
        }
        ddrval = lpDDOverlayBack->Unlock(SurfaceDesc.lpSurface);
    }
    LeaveCriticalSection(&hDDCritSect);
}


BOOL Affichage_Overlay_Update(int x,int y,LPRECT pDestRect,DWORD  dwFlags,BOOL ColorKey)
{
	RECT    sSourceRect;
	HRESULT ddrval;
	DDOVERLAYFX DDOverlayFX;
	int i=0;

	if ( (lpDD == NULL ) || (lpDDSurface == NULL) || (lpDDOverlay == NULL )) return(FALSE);
    
	EnterCriticalSection(&hDDCritSect);
	memset(&DDOverlayFX,0x00, sizeof(DDOverlayFX));
	DDOverlayFX.dwSize = sizeof(DDOverlayFX);
	
	// Zoom pour DirectX (sSourceRect est la source  afficher  l'cran)
	// En changeant la source, on change la partie de l'image visible.
	sSourceRect.left   = 0 + DebutLigne;
	sSourceRect.top    = 0 + HauteurBandeNoire;
	sSourceRect.right  = x - DebutLigne;
	sSourceRect.bottom = y - HauteurBandeNoire;
	
	if (( x == 0 ) && ( y == 0 ))
	{
		ddrval = lpDDOverlay->UpdateOverlay(NULL, lpDDSurface, NULL, dwFlags, &DDOverlayFX);
		LeaveCriticalSection(&hDDCritSect);
		return(TRUE);
	}
	
    i=dwFlags;
	if ( (ColorKey ) && (Can_ColorKey))
	{
		dwFlags |= DDOVER_KEYDESTOVERRIDE;	

		PhysicalOverlayColor = Affichage_Overlay_ColorMatch(lpDDSurface, OverlayColor);
        if (PhysicalOverlayColor == 0 && OverlayColor != 0)      // sometimes we glitch and can't get the Value
		{
			LOG(3, "Physical overlay color is zero!  Retrying.");
            PhysicalOverlayColor = Affichage_Overlay_ColorMatch(lpDDSurface, OverlayColor);
		}
        LOG(3, "Physical overlay color is %x", PhysicalOverlayColor);
	
		DDOverlayFX.dckDestColorkey.dwColorSpaceHighValue = PhysicalOverlayColor;
		DDOverlayFX.dckDestColorkey.dwColorSpaceLowValue = PhysicalOverlayColor;
	}

    if(Can_Bob && ( (DeInterlace==1) || (DeInterlace==2) ) )
	{
		dwFlags |= DDOVER_BOB;
	}


	ddrval = lpDDOverlay->UpdateOverlay(&sSourceRect, lpDDSurface, pDestRect, dwFlags , &DDOverlayFX);
	if ( ddrval != DD_OK )
	{
		dwFlags=i;
		memset(&DDOverlayFX,0x00, sizeof(DDOverlayFX));
		DDOverlayFX.dwSize = sizeof(DDOverlayFX);
		
		ddrval = lpDDOverlay->UpdateOverlay(&sSourceRect, lpDDSurface, pDestRect, dwFlags, &DDOverlayFX);
		if ( ddrval != DD_OK )
		{
			Affichage_Overlay_Destroy();
			Affichage_Overlay_Create(x,y);
			ddrval = lpDDOverlay->UpdateOverlay(&sSourceRect, lpDDSurface, pDestRect, dwFlags, &DDOverlayFX);
			
			if ( ddrval != DD_OK )
			{
				LeaveCriticalSection(&hDDCritSect);
				return(FALSE);
			}
		}
	}
	// Adapt de DScaler
	destinationRectangle.left   = pDestRect->left;
	destinationRectangle.right  = pDestRect->right;
	destinationRectangle.top    = pDestRect->top;
	destinationRectangle.bottom = pDestRect->bottom;

	ScreenToClient(hWnd,((PPOINT)&destinationRectangle));
	ScreenToClient(hWnd,((PPOINT)&destinationRectangle)+1);
	InvalidateRect(hWnd,NULL,FALSE);

    LeaveCriticalSection(&hDDCritSect);
	return TRUE;
}


BOOL Affichage_Overlay_Create(int x,int y)
{
    DDSURFACEDESC	SurfaceDesc;
    DDPIXELFORMAT	PixelFormat;
	DDSCAPS caps;
    HRESULT	ddrval;
    int minBuffers, maxBuffers, numBuffers;
    char msg[500];
	long BackBuffers = -1;
	
    EnterCriticalSection(&hDDCritSect);
	
	memset(&PixelFormat,0x00, sizeof(PixelFormat));
	PixelFormat.dwSize = sizeof(DDPIXELFORMAT);
	PixelFormat.dwFlags = DDPF_FOURCC;
	PixelFormat.dwFourCC = MAKEFOURCC('Y','U','Y','2');
	PixelFormat.dwYUVBitCount = 16;

	memset(&SurfaceDesc,0x00, sizeof(SurfaceDesc));
	SurfaceDesc.dwSize         = sizeof( SurfaceDesc );
    SurfaceDesc.dwFlags        = DDSD_CAPS | DDSD_HEIGHT | DDSD_WIDTH | DDSD_PIXELFORMAT | DDSD_BACKBUFFERCOUNT;
    SurfaceDesc.ddsCaps.dwCaps = DDSCAPS_OVERLAY | DDSCAPS_VIDEOMEMORY | DDSCAPS_FLIP | DDSCAPS_COMPLEX;

	lpOverlay=NULL;
	SurfaceDesc.dwWidth  = x;
	SurfaceDesc.dwHeight = y;
	SurfaceDesc.ddpfPixelFormat = PixelFormat;
	
 /*   // If the user specified a particular back buffer Count, use it.  Otherwise
    // try triple buffering and drop down to double buffering, then single
    // buffering, if the card doesn't have enough memory.
    minBuffers = BackBuffers >= 0 ? BackBuffers : 0;
    maxBuffers = BackBuffers >= 0 ? BackBuffers : 2;
	*/
	// Force le double buffering (sinon le movieboost bug)
	minBuffers = 0; maxBuffers = 1;

	if(Overlay_NBB) maxBuffers=0; // Pas de backbuffer pour les anciennes cartes video...
	
	for (numBuffers = maxBuffers; numBuffers >= minBuffers; numBuffers--)
    {
        SurfaceDesc.dwBackBufferCount = numBuffers;
        // if we get down to zero back buffers then the flags are different
        if(numBuffers == 0)
        {
            SurfaceDesc.dwFlags = DDSD_CAPS | DDSD_HEIGHT | DDSD_WIDTH | DDSD_PIXELFORMAT;
            SurfaceDesc.ddsCaps.dwCaps = DDSCAPS_OVERLAY | DDSCAPS_VIDEOMEMORY;
        }
        ddrval = lpDD->CreateSurface( &SurfaceDesc, &lpDDOverlay, NULL);

        if (SUCCEEDED(ddrval) || ddrval != DDERR_OUTOFVIDEOMEMORY)
            break;
    }

    if (FAILED(ddrval))
    {
        lpDDOverlay = NULL;
    }

    if (numBuffers < minBuffers)
    {
        // Not enough video memory.  Display different messages depending
        // on what we tried to allocate, since it may mean different things.

        if (minBuffers == 0)
        {
            // We tried single-buffering and it didn't work.
            sprintf(msg, Langue_GetString(ID_ERROR_DD_NOOVERLAYMEM), BackBuffers);
            RealErrorBox(msg);
//           RealErrorBox("Your video card doesn't have enough overlay\n"
//                     "memory for a TV picture.  Try lowering your\n"
//                     "color depth or screen resolution.  Rebooting\n"
//                     "may help in some cases if memory is being\n"
//                     "used by something else on your system.");
            LeaveCriticalSection(&hDDCritSect);
            return (FALSE);
        }
        else
        {
            // We didn't get down to single-buffering, meaning the user
            // specified a back buffer Count.
            sprintf(msg, Langue_GetString(ID_ERROR_DD_NOOVERLAYMEM), BackBuffers);
            RealErrorBox(msg);
            LeaveCriticalSection(&hDDCritSect);
            return (FALSE);
        }
    }

    if (FAILED(ddrval))
    {
        switch (ddrval) 
        {
        case DDERR_NOOVERLAYHW:
//            RealErrorBox("Your video card doesn't appear to support\n"
//                     "overlays, which DScaler requires.");
			RealErrorBox (Langue_GetString(ID_ERROR_DD_NOOVERLAY));
            LeaveCriticalSection(&hDDCritSect);
			return (FALSE);

            // Any other interesting error codes?
        }
        
        sprintf(msg, "Can't create overlay surface: %x", ddrval);
        RealErrorBox(msg);
		LeaveCriticalSection(&hDDCritSect);
        return FALSE;
    }

    sprintf(msg, "%d Back Buffers", numBuffers);

    ddrval = lpDDOverlay->Lock(NULL, &SurfaceDesc, DDLOCK_WAIT, NULL);
    // sometimes in win98 we get weird error messages here
    // so we need to loop until it's OK or we get a surface lost message
    while(ddrval == DDERR_NOOVERLAYHW || ddrval == DDERR_SURFACEBUSY)
    {
        Sleep(100);
        ddrval = lpDDOverlay->Lock(NULL, &SurfaceDesc, DDLOCK_WAIT, NULL);
    }
    if(ddrval == DDERR_SURFACELOST)
    {
        ddrval = lpDD->CreateSurface(&SurfaceDesc, &lpDDOverlay, NULL);
        if (FAILED(ddrval))
        {
            sprintf(msg, Langue_GetString(ID_ERROR_DD_LOSTSURFACE), ddrval);
            RealErrorBox(msg);
            lpDDOverlay = NULL;
            LeaveCriticalSection(&hDDCritSect);
            return FALSE;
        }
        ddrval = lpDDOverlay->Lock(NULL, &SurfaceDesc, DDLOCK_WAIT, NULL);
    }
    if (FAILED(ddrval))
    {
        char szErrorMsg[200];
        sprintf(szErrorMsg, "Error %x in Lock Surface", ddrval);
        RealErrorBox(szErrorMsg);
        LeaveCriticalSection(&hDDCritSect);
        return (FALSE);
    }

    ddrval = lpDDOverlay->Unlock(SurfaceDesc.lpSurface);
    if (FAILED(ddrval))
    {
        RealErrorBox("Can't Unlock Surface");
        LeaveCriticalSection(&hDDCritSect);
        return (FALSE);
    }

    if(numBuffers != 0)
    {
        memset(&caps, 0, sizeof(caps));
        caps.dwCaps = DDSCAPS_BACKBUFFER;
        ddrval = lpDDOverlay->GetAttachedSurface(&caps, &lpDDOverlayBack);
        if (FAILED(ddrval))
        {
            RealErrorBox(Langue_GetString(ID_ERROR_DD_BACKSURFACE));
            lpDDOverlayBack = NULL;
            LeaveCriticalSection(&hDDCritSect);
            return (FALSE);
        }
        else
        {
            ddrval = lpDDOverlayBack->Lock(NULL, &SurfaceDesc, DDLOCK_WAIT, NULL);
            if (FAILED(ddrval))
            {
                RealErrorBox("Can't Lock Back Surface");
                LeaveCriticalSection(&hDDCritSect);
                return (FALSE);
            }
            ddrval = DDERR_WASSTILLDRAWING;

            ddrval = lpDDOverlayBack->Unlock(SurfaceDesc.lpSurface);
            if (FAILED(ddrval))
            {
                RealErrorBox("Can't Unlock Back Surface");
                LeaveCriticalSection(&hDDCritSect);
                return (FALSE);
            }
        }
    }
    else
    {
        lpDDOverlayBack = lpDDOverlay;
        lpDDOverlayBack->AddRef();
    }

    LeaveCriticalSection(&hDDCritSect);
    Affichage_Overlay_Clean();
	return TRUE;
}


void Affichage_Overlay_Destroy()
{
    EnterCriticalSection(&hDDCritSect);

	if ( lpDDOverlayBack != NULL )
		lpDDOverlayBack->Release();
	lpDDOverlayBack=NULL;

	if ( lpDDOverlay != NULL )
	{
		Affichage_Overlay_Update(0,0,NULL, DDOVER_HIDE,FALSE);
		lpDDOverlay->Release();
	}
	lpDDOverlay=NULL;
    LeaveCriticalSection(&hDDCritSect);
}


void Affichage_CloseDD()
{
	if( lpDD != NULL )
	{		
		Affichage_Overlay_Destroy();

		if ( lpDDSurface != NULL ) 
			lpDDSurface->Release();
		lpDDSurface=NULL;

		lpDD->Release();
		lpDD = NULL;
	}
}


BOOL Affichage_InitDD( HWND hwnd , BOOL Fullscreen, int x , int y )
{
    HRESULT		ddrval;
    LPDIRECTDRAW	lpdd;
    DDSURFACEDESC	ddsd;
    DDCAPS DriverCaps;
	

	Affichage_CloseDD();	
	if (lpDD == NULL)
	{
		if ( DirectDrawCreate(NULL,&lpdd,NULL) != DD_OK )
		{
			RealErrorBox(Langue_GetString(ID_ERROR_DD_DDRAWCREATE));
			return(FALSE);
		}
		LOG (1, "DirectDrawCreate OK");
		lpDD=lpdd;
		
		memset(&DriverCaps,0x00,sizeof(DriverCaps));
		DriverCaps.dwSize = sizeof(DriverCaps);
		ddrval = IDirectDraw_GetCaps(lpDD,&DriverCaps,NULL);		
		if (ddrval == DD_OK)
		{
			if (DriverCaps.dwCaps & DDCAPS_OVERLAY &&
				DriverCaps.dwCaps & DDCAPS_OVERLAYSTRETCH)
			{ 
				LOG (1, "Overlay et OverlayStrech dtect");
				Can_Overlay = TRUE;
				Can_ColorKey = ((DriverCaps.dwCKeyCaps & DDCKEYCAPS_DESTOVERLAY) !=0 );
				Can_Bob = ((DriverCaps.dwCaps2 & DDCAPS2_CANFLIPODDEVEN) != 0);
			}
			else
			{
				RealErrorBox(Langue_GetString(ID_ERROR_DD_NOOVERLAY));
 				return(FALSE);			
			}
		}
	}
	else
		lpdd=lpDD;	
	
	if (Fullscreen)
	{
		ddrval = lpdd->SetCooperativeLevel(hwnd, DDSCL_ALLOWMODEX | DDSCL_EXCLUSIVE | DDSCL_FULLSCREEN );
		if( ddrval != DD_OK )
		{
			RealErrorBox(Langue_GetString(ID_ERROR_DD_SETCOOPLEVEL));
			return(FALSE);
		}
		ddrval = lpdd->SetDisplayMode(x, y, 16);
		if( ddrval != DD_OK )
		{
			RealErrorBox(Langue_GetString(ID_ERROR_DD_FULLSCREEN));
			return(FALSE);
		}
	} 
	else
	{	
		ddrval = lpdd->SetCooperativeLevel(hwnd,DDSCL_NORMAL );
		if( ddrval != DD_OK )
		{
			RealErrorBox(Langue_GetString(ID_ERROR_DD_SETCOOPLEVEL));
			return(FALSE);
		}
	}
	memset(&ddsd,0x00, sizeof(ddsd));
	ddsd.dwSize         = sizeof( ddsd );
	ddsd.dwFlags        = DDSD_CAPS;
	ddsd.ddsCaps.dwCaps = DDSCAPS_PRIMARYSURFACE;
	
	if ( lpdd->CreateSurface(&ddsd, &lpDDSurface, NULL ) !=DD_OK)
	{
		RealErrorBox("Erreur sur DDraw:CreateSurface");
		return(FALSE);
	}
	
	ddrval = lpDDSurface->Lock(NULL, &ddsd, 0, NULL);
	ddrval = lpDDSurface->Unlock(ddsd.lpSurface);
		
	memset(&ddsd,0x00, sizeof(ddsd));
	ddsd.dwSize = sizeof( ddsd );
	ddrval = lpdd->GetDisplayMode(&ddsd);
	  
	return TRUE;
}


// Returns the current source rectangle.
void Affichage_GetDestRect(RECT* rect)
{
    memcpy(rect, &destinationRectangle, sizeof(RECT));
}


BOOL Affichage_Overlay_Lock()
{
    HRESULT         ddrval;
    static DWORD    dwFlags = DDLOCK_WAIT | DDLOCK_NOSYSLOCK;
    DDSURFACEDESC   SurfaceDesc;

    if(lpDDOverlay == NULL || lpDDOverlayBack == NULL)
    {
        LOG(1, "Overlay has been deleted");
        return FALSE;
    }

    EnterCriticalSection(&hDDCritSect);

    if (FlipResult == DDERR_WASSTILLDRAWING)             // prev flip was busy?
    {
        ddrval = lpDDOverlay->Flip(NULL, DDFLIP_DONOTWAIT);  
        if(ddrval == DDERR_SURFACELOST)
        {
            LOG(1, "Flip before lock failed");
            LeaveCriticalSection(&hDDCritSect);
            return FALSE;
        }
        FlipResult = 0;                 // but no time to try any more
    }

    memset(&SurfaceDesc, 0x00, sizeof(SurfaceDesc));
    SurfaceDesc.dwSize = sizeof(SurfaceDesc);
    ddrval = lpDDOverlayBack->Lock( NULL, &SurfaceDesc, dwFlags, NULL);

    // fix suggested by christoph for NT 4.0 sp6
    if(ddrval == E_INVALIDARG && (dwFlags & DDLOCK_NOSYSLOCK))
    {
        //remove flag
        ddrval = lpDDOverlay->Lock(NULL, &SurfaceDesc, DDLOCK_WAIT, NULL);
        if( SUCCEEDED(ddrval) )
        {
            //remember for next time
            dwFlags = DDLOCK_WAIT;
            LOG(1, "Switched to not using NOSYSLOCK");
        }
    }

    if(FAILED(ddrval))
    {
        LOG(1, "Lock failed %8x", ddrval);
        LeaveCriticalSection(&hDDCritSect);
        return FALSE;
    }
	// Set new pitch, may change
 	OverlayPitch = SurfaceDesc.lPitch;
	lpOverlay    = SurfaceDesc.lpSurface;
    // stay in critical section
	return TRUE;
}


BOOL Affichage_Overlay_UnLock()
{
    HRESULT ddrval;
    BOOL RetVal = TRUE;

    if(lpDDOverlayBack == NULL)
    {
        LOG(1, "Overlay has been deleted");
        LeaveCriticalSection(&hDDCritSect);
        return FALSE;
    }

    // we are already in critical section
    ddrval = lpDDOverlayBack->Unlock(NULL);
    if(FAILED(ddrval))
    {
        if(ddrval != DDERR_SURFACELOST)
        {
            LOG(1, "Unexpected failure in Unlock %8x", ddrval);
        }
        RetVal = FALSE;
    }
	LeaveCriticalSection(&hDDCritSect);
    return RetVal;
}


BOOL Affichage_Overlay_Flip(DWORD FlipFlag)
{
    BOOL RetVal = TRUE;

    if(lpDDOverlay == NULL)
    {
        LOG(1, "Overlay has been deleted");
        return FALSE;
    }

    EnterCriticalSection(&hDDCritSect);
    FlipResult = lpDDOverlay->Flip(NULL, FlipFlag);
    if(FAILED(FlipResult))
    {
        if(FlipResult != DDERR_WASSTILLDRAWING && 
            FlipResult != DDERR_SURFACELOST &&
            FlipResult != DDERR_NOTFLIPPABLE)
        {
            LOG(1, "Surface Flip failed %8x", FlipResult);
        }
        // return OK if we get DDERR_WASSTILLDRAWING
        // we'll do the flip next time
        // also if we get DDERR_NOTFLIPPABLE
        // as this probably means the overlay has been hiden
        RetVal = (FlipResult == DDERR_WASSTILLDRAWING) || (FlipResult == DDERR_NOTFLIPPABLE);
    }
	LeaveCriticalSection(&hDDCritSect);
    return RetVal;
}


void Affichage_Stop(HWND hWnd)
{
    RECT winRect;
    HDC hDC;

    GetClientRect(hWnd, &winRect);
    hDC = GetDC(hWnd);
    Affichage_Overlay_PaintColorkey(hWnd, FALSE, hDC, &winRect);
    ReleaseDC(hWnd,hDC);

    Stop_Capture(TRUE);
	Affichage_CloseDD();
    InvalidateRect(hWnd, NULL, FALSE);
}


void Affichage_Start(HWND hWnd)
{
    InvalidateRect(hWnd, NULL, FALSE);
	Affichage_InitDD(hWnd,FALSE, 0, 0);
    Reset_Capture();
}
