/////////////////////////////////////////////////////////////////////////////
// hardwaremisc.c
/////////////////////////////////////////////////////////////////////////////
// Copyright (c) 2000-2002 Quenotte  All rights reserved.
/////////////////////////////////////////////////////////////////////////////
//
//	This file is subject to the terms of the GNU General Public License as
//	published by the Free Software Foundation.  A copy of this license is
//	included with this software distribution in the file COPYING.  If you
//	do not have a copy, you may obtain a copy by writing to the Free
//	Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
//
//	This software is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details
/////////////////////////////////////////////////////////////////////////////
//
// This software was based on MultiDec 5.9 Those portions are
// Copyright (C) 1999/2000 Espresso (echter_espresso@hotmail.com)
//
// La grande majorit du code est tir et adapt du projet DSCALER.
// Copyright (c) 2000 John Adcock.  All rights reserved.
/////////////////////////////////////////////////////////////////////////////
#include "stdafx.h"
#include "resource.h"
#include "..\Plugins\ResCommun.h"

#include "bt848.h"
#include "hardwaremisc.h"
#include "thread.h"
#include "ddraw.h"
#include "K!TV-XP.h"
#include "bt_io.h"
#include "coderisc.h"
#include "plugin.h"
#include "externe/dsdrv.h"
#include "misc.h"


PHYS RiscBasePhysical; 
PMemStruct Risc_dma;
PMemStruct Vbi_dma[5], Display_dma[5];
BYTE  *pVBILines[5] = { NULL,NULL,NULL,NULL,NULL };
BYTE  *pDisplay[5] =  { NULL,NULL,NULL,NULL,NULL };

BOOL CapturePaused=TRUE;		// Capture en pause momentane. (juste aprs stopthread )
BOOL IsWinNT=FALSE;
char BTVendorID[10] = "", BTDeviceID[10] = "", BTChipType[10] = "";
char BTMemInfo [50];
BOOL bIsBT878=FALSE;

CRITICAL_SECTION CaptureCrit;

struct CartesBT {
	DWORD BTVendorID;
	DWORD BTDeviceID;
	char BTChipType[10];
};



// Initialisation de tout le chargement du driver 
BOOL InitialiserDriver()
{
int ret;

	InitializeCriticalSection(&ThreadCrit);
	InitializeCriticalSection(&hDDCritSect);	// DeleteCriticalSection dans misc.cpp
	InitializeCriticalSection(&m_cCrit);
	InitializeCriticalSection(&CaptureCrit);

	ret = LoadDriver();
	if ( ret != 0)
	{
		char buffer[255];

		sprintf (buffer, Langue_GetString(ID_ERROR_DRIVER), ret);
		ret = MessageBox(hWnd, buffer, "K!TV XP", MB_ICONSTOP|MB_YESNO);
		if (ret == IDYES) {
			// Essayons de rparer en supprimant/rinstallant les drivers
			UnInstallNTDriver();
			Sleep(500);
			InstallNTDriver();
		}
		goto erreur;
	}

	if ( !BT8X8_FindTVCard() )
	{
		RealErrorBox(Langue_GetString(ID_ERROR_NOCARD));
		goto erreur;
	}

	BT8X8_MemoryInit();
	BT8X8_Reset_Hardware();
	return TRUE;

erreur :
	Sleep(500);
	SendMessage(hWnd,WM_DESTROY,0,0);
	return FALSE;
}


void FermerDriver()
{
	BT8X8_Close();
	DeleteCriticalSection(&CaptureCrit);
	DeleteCriticalSection(&m_cCrit);
	DeleteCriticalSection(&ThreadCrit);
	UnloadDriver();
}


void DeterminerOS(void)
{
	OSVERSIONINFOEX osvi;
	char VersionString[255];
	
	ZeroMemory(&osvi, sizeof(OSVERSIONINFOEX));
	osvi.dwOSVersionInfoSize=sizeof(OSVERSIONINFOEX);
	if (GetVersionEx((OSVERSIONINFO *) &osvi))
	{
		if ( osvi.dwPlatformId == VER_PLATFORM_WIN32_WINDOWS )
		{ 
			if  ( osvi.dwMinorVersion > 0 ) strcpy(VersionString,"Windows 98");
			else strcpy(VersionString,"Windows 95");
		}
		if ( osvi.dwPlatformId == VER_PLATFORM_WIN32_NT )
		{ 
			IsWinNT=TRUE;
			if  ( osvi.dwMajorVersion == 4 ) strcpy(VersionString,"Windows NT 4.0");
			else if  ( osvi.dwMajorVersion == 5 )
			{
				if  ( osvi.dwMinorVersion == 0 )	strcpy(VersionString,"Windows 2000");
				else if  ( osvi.dwMinorVersion == 1 )
				{
					if( osvi.wSuiteMask == VER_SUITE_PERSONAL )	strcpy(VersionString,"Windows XP HE");
					else	strcpy(VersionString,"Windows XP Pro");
				}

			}
		}
	}
	SplashBox(NULL,VersionString,NULL,NULL, NULL);
		
	LOG(1,"PlatformId=%d MajorVersion=%d MinorVersion=%d", osvi.dwPlatformId, osvi.dwMajorVersion, osvi.dwMinorVersion);
	if ( osvi.szCSDVersion != NULL ) LOG (1, osvi.szCSDVersion);
}


void BT8X8_Close()
{
	int i;

	BT8X8_WriteByte(BT848_SRESET, 0);

	memoryFree(Risc_dma); 
	for (i =0; i<5; i++)
	{
		memoryFree(Vbi_dma[i]);
		memoryFree(Display_dma[i]);
		Display_dma[i] = NULL;
	}
}


BOOL BT8X8_FindTVCard()
{
 	int ret, i;
  	char temp[100];
  	BOOL CarteDetectee = FALSE;
 	struct CartesBT ListeCartesBT[4] =
  	{
  		{0x109e, 0x350, "BT848"},
  		{0x109e, 0x351, "BT849"},
  		{0x109e, 0x36e, "BT878"},
  		{0x109e, 0x36F, "BT879"}
  	};
 	DWORD dwPhysicalAddress = 0, dwMemoryLength = 0, dwSubSystemID = 0;
  
 	for (i =0; i < sizeof(ListeCartesBT) / sizeof(ListeCartesBT[0]); i++) 
 	{
 		ret = pciGetHardwareResources(
 					ListeCartesBT[i].BTVendorID, ListeCartesBT[i].BTDeviceID, 0,
 					&dwPhysicalAddress, &dwMemoryLength, &dwSubSystemID);
 		if (ret == 0) 
 		{
  			sprintf(BTVendorID, "0x%04x", ListeCartesBT[i].BTVendorID);
 			sprintf(BTDeviceID, "0x%04x", ListeCartesBT[i].BTDeviceID);
 			sprintf(BTMemInfo,  "0x%08x-0x%08x", dwPhysicalAddress, dwMemoryLength+dwPhysicalAddress); 
			if(i>1)	bIsBT878=TRUE;
  			strcpy(BTChipType, ListeCartesBT[i].BTChipType);
  			CarteDetectee = TRUE;
  			break;
  		}
  	}
  
 	if (!CarteDetectee) return FALSE;
  
  	sprintf(temp, Langue_GetString(ID_FOUND), BTChipType);
  	SplashBox(NULL,NULL,temp, NULL, NULL);  
 	return TRUE;
}


static void* GetFirstFullPage(PMemStruct pMem)
{
	PPageStruct pPages = (PPageStruct)(pMem + 1);
	DWORD pRetVal;

	pRetVal = (DWORD)pMem->dwUser;

	if(pPages[0].dwSize != 4096)
	{
		pRetVal += pPages[0].dwSize;
	}
	return (void*)pRetVal;	
}


BOOL BT8X8_MemoryInit(void)
{
	int i;

	if (memoryAlloc(83968, ALLOC_MEMORY_CONTIG, &Risc_dma))
	{
		RealErrorBox(Langue_GetString(ID_ERROR_RISCMEM));
		return (FALSE);
	}
	LOG (1, "Risc Memory Allocation OK");

	RiscBasePhysical = GetPhysicalAddress(Risc_dma, (BYTE *)Risc_dma->dwUser, 83968, NULL);
	
	for (i = 0; i < 5; i++)
	{
		// JA 02/01/2001
		// Allocate some extra memory so that we can skip
		// start of buffer that is not page aligned
		if (memoryAlloc(2048 * 19 * 2 + 4095, 0, &Vbi_dma[i]))
		{
			RealErrorBox(Langue_GetString(ID_ERROR_VBICMEM));
			return (FALSE);
		}
		pVBILines[i] = (BYTE *)GetFirstFullPage(Vbi_dma[i]);

		// JA 29/12/2000
		// Allocate some extra memory so that we can skip
		// start of buffer that is not page aligned
		if (memoryAlloc(1024 * 576 * 2 + 4095, 0, &Display_dma[i]))
		{
			RealErrorBox(Langue_GetString(ID_ERROR_DISPMEM));
			return (FALSE);
		}
		pDisplay[i] = (BYTE *)GetFirstFullPage(Display_dma[i]);

	}
	LOG (1, "VBI memory + Display memory OK");
	return (TRUE);
}


PHYS GetPhysicalAddress(PMemStruct pMem, LPBYTE pLinear, DWORD dwSizeWanted, DWORD * pdwSizeAvailable)
{
	PPageStruct pPages = (PPageStruct)(pMem + 1);
	DWORD Offset;
    DWORD i; 
    DWORD sum;
	DWORD pRetVal = 0;

	Offset = (DWORD)pLinear - (DWORD)pMem->dwUser;
    sum = 0; 
	i = 0;
	while (i < pMem->dwPages)
	{
		if (sum + pPages[i].dwSize > (unsigned)Offset)
		{
            Offset -= sum;
		    pRetVal = pPages[i].dwPhysical + Offset;	
            if ( pdwSizeAvailable != NULL )
			{
				*pdwSizeAvailable = pPages[i].dwSize - Offset;
			}
			break;
		}
		sum += pPages[i].dwSize; 
		i++;
	}
	if(pRetVal == 0)
	{
		sum++;
	}
    if ( pdwSizeAvailable != NULL )
	{
		if (*pdwSizeAvailable < dwSizeWanted)
		{
			sum++;
		}
	}

	return pRetVal;	
}


// Arrt ou suspension du thread d'affichage
//
// BOOL bForce :  
//
// - TRUE  : forcer l'arret du thread (le thread est tu)
//
// - FALSE : Mettre le thread en "attente" au lieu de l'arrter
//   Evidemment le dmarrage est plus rapide dans ce cas-ci.
void Stop_Capture(BOOL bForce)
{
	EnterCriticalSection(&CaptureCrit);
	if(!bForce)
	{
		ResetEvent(Thread_Event);
		Sleep(50);
	}
	else
	{
		//  Stop The Output Thread
		Stop_Thread();
		CapturePaused = TRUE;
	}
	// stop capture
	BT8X8_MaskDataByte(BT848_CAP_CTL, 0, 0x0f);
	LeaveCriticalSection(&CaptureCrit);
	UpdateMenuBar();
}


// Remarque :
// bForce doit etre identique au parametre utilis pour Stop_Capture
void Start_Capture(BOOL bForce)
{
	int nFlags = 0;
	EnterCriticalSection(&CaptureCrit);
	
	if (Capture_Video)
		nFlags |= BT848_CAP_CTL_CAPTURE_EVEN | BT848_CAP_CTL_CAPTURE_ODD;
	if (Capture_VBI)
		nFlags |= BT848_CAP_CTL_CAPTURE_VBI_EVEN | BT848_CAP_CTL_CAPTURE_VBI_ODD;

	BT8X8_MaskDataByte(BT848_CAP_CTL, 0, 0x0f);
	BT8X8_CreateRiscCode(CurrentX, CurrentY);	
	BT8X8_MaskDataByte(BT848_CAP_CTL, (BYTE) nFlags, (BYTE) 0x0f);
	Sleep(50);

	if(bForce)
	{
		Start_Thread();
		CapturePaused = FALSE;
	}
	else
	{
		InitThread();
		Sleep(50);
		SetEvent(Thread_Event);
	}
	LeaveCriticalSection(&CaptureCrit);
	UpdateMenuBar();
}


void Reset_Capture()
{
    Stop_Capture(TRUE);
	BT8X8_Reset_Hardware();
	Affichage_Init_Screen_Struct();
	BT8X8_SetAudioSource(AudioSource);
    Start_Capture(TRUE);
}


void BT8X8_Restart_RISC_Code()
{
	BYTE CapCtl = BT8X8_ReadByte(BT848_CAP_CTL);
	BT8X8_MaskDataByte(BT848_CAP_CTL, 0, (BYTE) 0x0f);
	BT8X8_WriteDword(BT848_INT_STAT, (DWORD) 0x0fffffff);
	BT8X8_WriteDword(BT848_RISC_STRT_ADD, RiscBasePhysical);
	BT8X8_WriteByte(BT848_CAP_CTL, CapCtl);
}

