/////////////////////////////////////////////////////////////////////////////
// lang.cpp
/////////////////////////////////////////////////////////////////////////////
// Copyright (c) 2000-2002 Quenotte  All rights reserved.
/////////////////////////////////////////////////////////////////////////////
//
//	This file is subject to the terms of the GNU General Public License as
//	published by the Free Software Foundation.  A copy of this license is
//	included with this software distribution in the file COPYING.  If you
//	do not have a copy, you may obtain a copy by writing to the Free
//	Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
//
//	This software is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details
/////////////////////////////////////////////////////////////////////////////
//
// Fonctions pour afficher les ressources (menu, bdd etc.) dans la langue courante.
//
/////////////////////////////////////////////////////////////////////////////
#include "stdafx.h"
#include "resource.h"
#include <locale.h>
#include <windows.h>

#include "K!TV-XP.h"
#include "lang.h"
#include "plugin.h"


char LangName[50];					// Dernier plugin lang utilis
int TotalLangages = 0;				// Nombre total de langage valides
int AppLang = 0;					// Langage (index dans Langages)
struct listlangages Langages[MAX_LANGAGE];

static BOOL Langue_ChargerPlugin(char *LangName);
static char *Langue_GetStringByInst(int resourceID, HINSTANCE hinst);
static char *Langue_ConvertString(int resourceID, WCHAR *pStr);



// Cherche la liste des langages disponibles dans le rpertoire courant
void Langue_ChargerTousPlugins()
{
    WIN32_FIND_DATA FindFileData;
    HANDLE hFindFile;

	AppLang = 0;
	TotalLangages = 0;
	for (int i=0; i<MAX_LANGAGE; i++)
	{
		Langages[i].hInst   = 0;
		Langages[i].LangID  = 0;
		Langages[i].Info[0] = NULL;
	}

	// Par dfaut le francais est inclus 
	Langages[0].hInst = hInst;
	strcpy (Langages[0].Lang, "Franais");
	strcpy (Langages[0].Info, "Par dfaut");
	Langages[0].LangID = MAKELANGID (LANG_FRENCH, SUBLANG_DEFAULT);
	TotalLangages++;

	// Cherchons les autres langages ...
    hFindFile = FindFirstFile("Langage/K!TVRS*.dll", &FindFileData);

    if (hFindFile != INVALID_HANDLE_VALUE)
    {
        BOOL RetVal = TRUE;
        while (RetVal != 0)
        {
			if (Langue_ChargerPlugin(FindFileData.cFileName)) 
			{
				// C'est le plugin  utiliser par dfaut
				if (strcmp(Langages[TotalLangages].Lang, LangName) == 0)
					AppLang = TotalLangages;

				TotalLangages++;
			}
            RetVal = FindNextFile(hFindFile, &FindFileData);
        }
    }
	FindClose(hFindFile);
	return;
}


static BOOL Langue_ChargerPlugin(char *LangName)
{
	char PluginPath[268], *TypePlugin = "Langage";
	HINSTANCE hPluginDLL;
	typedef WORD (WINAPI *tGetLang)();
	tGetLang GetLang;

	strcpy(PluginPath,"Langage/");
	strcat(PluginPath,LangName);

	__try
	{
		LOG (1, Langue_GetString(ID_PLG_INFO), TypePlugin);
		hPluginDLL = LoadLibrary(PluginPath);
		if (hPluginDLL == NULL)
		{
			LOG (1, Langue_GetString(ID_PLG_ERROR_LOAD), LangName);
			return FALSE;
		}
		GetLang = (tGetLang)GetProcAddress((HMODULE)hPluginDLL, "GetLang");
		if (GetLang)
		Langages[TotalLangages].LangID = GetLang();
		Langages[TotalLangages].hInst = hPluginDLL;
		strcpy (Langages[TotalLangages].Lang, Langue_GetStringByInst(ID_DLL_LANG, hPluginDLL));
		strcpy (Langages[TotalLangages].Info, Langue_GetStringByInst(ID_DLL_AUTHOR, hPluginDLL));

		LOG (1, Langue_GetString(ID_PLG_SUCCES), Langages[TotalLangages].Lang);
	}

    __except (EXCEPTION_EXECUTE_HANDLER  ) 
	{  
		char mod[256];

		FreeLibrary((HMODULE)hPluginDLL);
		sprintf(mod, Langue_GetString(ID_PLG_ERROR_VER), TypePlugin, LangName);
		RealErrorBox(mod);
		return FALSE;
	}
	return TRUE;
}


void Langue_DechargerPlugin()
{
	// i = 0: Ressources dans l'EXE; il n'est pas dcharg
	for (int i = 1; i < TotalLangages; i++)
		FreeLibrary(Langages[i].hInst);
}


void Langue_ChangeLang(int lang, BOOL bForce)
{
	if ((lang == AppLang) && !bForce)
		// Langue identique, ne fait rien
		return;

	AppLang = lang;
	strcpy( LangName, Langages[AppLang].Lang);
	// Initialise la langue des plugins.
	if (PluginVideoPresent)	PluginVideo->LangID = Langages[AppLang].LangID;
	if (PluginAudioPresent)	PluginAudio->LangID = Langages[AppLang].LangID;
	//Mise a jour des menus
	SendMessage(hWnd,WM_COMMAND,IDM_UPDATEMENU,0);
	
	setlocale( LC_ALL, Langue_GetString(ID_LOCALE_LANG));
}


static HGLOBAL Langue_FindLoadResource(HINSTANCE &hinst, LPCTSTR lpType, LPCTSTR lpName)
{
	HRSRC res;
	WORD wLanguage;
	HINSTANCE hModule;
	HGLOBAL hLoad;

	wLanguage = MAKELANGID(LANG_NEUTRAL, SUBLANG_NEUTRAL);
	if (AppLang >0 && AppLang < TotalLangages)
		hModule = Langages[AppLang].hInst;
	else
		// K!TV default language (French)
		hModule = Langages[0].hInst;

	res = FindResourceEx(hModule, lpType, lpName, wLanguage);
	if (!res) 
		return NULL;
	hLoad = LoadResource(hModule, res);
	hinst = hModule;
	return hLoad;
}


int Langue_DialogBox(HINSTANCE hinst, LPCTSTR templtname, HWND hwnd, DLGPROC dlgproc)
{
	HGLOBAL hLoad;

	hLoad = Langue_FindLoadResource(hinst, RT_DIALOG, templtname);
	if (hLoad) 
	{
		LPDLGTEMPLATE ResTemplate;

		ResTemplate = (LPDLGTEMPLATE)LockResource(hLoad);
		return DialogBoxIndirect(hinst, ResTemplate, hwnd, dlgproc);
	}
	return FALSE;
}


HWND Langue_CreateDialog(HINSTANCE hinst, LPCTSTR templtname, HWND hwnd, DLGPROC dlgproc)
{
	HGLOBAL hLoad;
	HWND rethwnd;
	
	rethwnd = NULL;
	hLoad = Langue_FindLoadResource(hinst, RT_DIALOG, templtname);
	if (hLoad) 
	{
		LPDLGTEMPLATE ResTemplate;

		ResTemplate = (LPDLGTEMPLATE)LockResource(hLoad);
		rethwnd = CreateDialogIndirect(hinst, ResTemplate, hwnd, dlgproc);
		FreeResource(hLoad);	// est-ce la bonne fonction ?
		return rethwnd;
	}
	else	return rethwnd;
}


HMENU Langue_LoadMenu(HINSTANCE hinst, LPCTSTR templtname)
{
	HGLOBAL hLoad;
	HMENU retmenu;
	
	retmenu = NULL;
	hLoad = Langue_FindLoadResource(hinst, RT_MENU, templtname);
	if (hLoad) 
	{
		MENUTEMPLATE  *MenuTemplate;

		MenuTemplate = LockResource(hLoad);
		retmenu = LoadMenuIndirect(MenuTemplate);
		return retmenu;
	}
	else	return retmenu;
}


char * Langue_GetString(int resourceID)
{
	HGLOBAL hLoad;
	HINSTANCE hinst;

	hLoad = Langue_FindLoadResource(hinst, RT_STRING, MAKEINTRESOURCE(resourceID>>4)+1);
	return Langue_ConvertString(resourceID, (WCHAR *)LockResource(hLoad) );
}


// Permet de chercher des infos d'un plugin qui n'est par encore charg
static char * Langue_GetStringByInst(int resourceID, HINSTANCE hinst)
{
	HGLOBAL hLoad;
	HRSRC res;
	WORD wLanguage;

	wLanguage = MAKELANGID(LANG_NEUTRAL, SUBLANG_NEUTRAL);
	res = FindResourceEx(hinst, RT_STRING, MAKEINTRESOURCE(resourceID>>4)+1, wLanguage);
	hLoad = LoadResource(hinst, res);
	return Langue_ConvertString(resourceID, (WCHAR *)LockResource(hLoad) );
}


// Lecture & recherche d'une chaine dans une string table
// Code inspir de :
// http://groups.google.fr/groups?hl=fr&selm=39917b32.792729906%40news.demon.co.uk&rnum=5
// Written By Marcus / June 1998
static char *Langue_ConvertString(int resourceID, WCHAR *pStr)
{
	static char	stringLoad[512];
	int nLenStr, len;
	
	stringLoad[0] = 0;

	if (!pStr)
		return stringLoad;

	// loop through finding the string in the source blockloaded res
	for(int i = 0; i < resourceID % 16; i++ )
	{
		nLenStr = (int)( *pStr++ );
		pStr += nLenStr;
	}
	// get the length of the string
	nLenStr = ( int )( *pStr++ );
	if ((UINT)nLenStr > 512)
		// Buffer is too small
		return stringLoad;

	#ifdef UNICODE
		_tcsncpy(stringLoad, pStr, nLenStr);
		len = nLenStr;
	#else
		// Convert from unicode to ANSI
		len = WideCharToMultiByte(CP_ACP, 0, pStr, nLenStr, stringLoad, 512, NULL, NULL);
	#endif
	
	stringLoad[len] = '\0';
	return stringLoad;
}
