/////////////////////////////////////////////////////////////////////////////
// lowlevel_io.c 
// Common BT and I2C lowlevel functions	
/////////////////////////////////////////////////////////////////////////////
// Copyright (c) 2000-2002 Quenotte  All rights reserved.
/////////////////////////////////////////////////////////////////////////////
//
//	This file is subject to the terms of the GNU General Public License as
//	published by the Free Software Foundation.  A copy of this license is
//	included with this software distribution in the file COPYING.  If you
//	do not have a copy, you may obtain a copy by writing to the Free
//	Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
//
//	This software is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details
/////////////////////////////////////////////////////////////////////////////
//
// This software was based on MultiDec 5.9 Those portions are
// Copyright (C) 1999/2000 Espresso (echter_espresso@hotmail.com)
//
/////////////////////////////////////////////////////////////////////////////
#include "stdafx.h"
#include "resource.h"
#include "..\Plugins\ResCommun.h"

#include "bt848.h"
#include "bt_io.h"
#include "externe/dsdrv.h"

#define I2C_DELAY 0
CRITICAL_SECTION m_cCrit;


void BT8X8_MaskDataByte(int Offset, BYTE d, BYTE m)
{
	BYTE a;

	a = BT8X8_ReadByte(Offset);
	a = (a & ~(m)) | ((d) & (m));
	BT8X8_WriteByte(Offset, a);
}

void BT8X8_MaskDataWord(int Offset, WORD d, WORD m)
{
	WORD a;

	a = BT8X8_ReadWord(Offset);
	a = (a & ~(m)) | ((d) & (m));
	BT8X8_WriteWord(Offset, a);
}

void BT8X8_AndOrDataDword(int Offset, DWORD d, DWORD m)
{
	DWORD a;

	a = BT8X8_ReadDword(Offset);
	a = (a & m) | d;
	BT8X8_WriteDword(Offset, a);
}

void BT8X8_AndDataByte(int Offset, BYTE d)
{
	BYTE a;

	a = BT8X8_ReadByte(Offset);
	a &= d;
	BT8X8_WriteByte(Offset, a);
}

void BT8X8_AndDataWord(int Offset, short d)
{
	WORD a;

	a = BT8X8_ReadWord(Offset);
	a &= d;
	BT8X8_WriteWord(Offset, a);
}

void BT8X8_OrDataByte(int Offset, BYTE d)
{
	BYTE a;

	a = BT8X8_ReadByte(Offset);
	a |= d;
	BT8X8_WriteByte(Offset, a);
}

void BT8X8_OrDataWord(int Offset, unsigned short d)
{
	WORD a;

	a = BT8X8_ReadWord(Offset);
	a |= d;
	BT8X8_WriteWord(Offset, a);
}

////////////////////////////////////////////////////////////////////////////////////
// I2C level	
////////////////////////////////////////////////////////////////////////////////////

static void I2CBus_wait(int us)
{
	if ( us > 0 )
	{ 
		 Sleep(us);
		 return;
	};
	Sleep(0);
	Sleep(0);
	Sleep(0);
	Sleep(0);
	Sleep(0);
}


static void I2C_SetLine(BOOL bCtrl, BOOL bData)
{
	BT8X8_WriteDword(BT848_I2C,(bCtrl<<1)|bData);
    I2CBus_wait(I2C_DELAY);
}


static BOOL I2C_GetLine()
{
    return BT8X8_ReadDword(BT848_I2C)&1;
}


BOOL I2CBus_AddDevice(BYTE I2C_Port)
{
    BOOL bAck;

    // Test whether device exists
	I2CBus_Lock();
	I2CBus_Start();
    bAck=I2CBus_SendByte(I2C_Port,0);
	I2CBus_Stop();
	I2CBus_Unlock();
    if(bAck) return TRUE;
      else return FALSE;
}

BOOL I2CBus_Lock()
{
	EnterCriticalSection(&m_cCrit);
	return TRUE;
}

BOOL I2CBus_Unlock()
{
	LeaveCriticalSection(&m_cCrit);
	return TRUE;
}

void I2CBus_Start()
{
    I2C_SetLine(0,1);
    I2C_SetLine(1,1);
    I2C_SetLine(1,0);
    I2C_SetLine(0,0);
}

void I2CBus_Stop()
{
    I2C_SetLine(0,0);
    I2C_SetLine(1,0);
    I2C_SetLine(1,1);
}

void I2CBus_One()
{
    I2C_SetLine(0,1);
    I2C_SetLine(1,1);
    I2C_SetLine(0,1);
}

void I2CBus_Zero()
{
    I2C_SetLine(0,0);
    I2C_SetLine(1,0);
    I2C_SetLine(0,0);
}

static BOOL I2CBus_Ack()
{
    BOOL bAck;
    
    I2C_SetLine(0,1);
    I2C_SetLine(1,1);
    bAck = !I2C_GetLine();
    I2C_SetLine(0,1);
    return bAck;
}

BOOL I2CBus_SendByte(BYTE nData, int nWaitForAck)
{
    I2C_SetLine(0,0);
    nData&0x80?I2CBus_One():I2CBus_Zero();
    nData&0x40?I2CBus_One():I2CBus_Zero();
    nData&0x20?I2CBus_One():I2CBus_Zero();
    nData&0x10?I2CBus_One():I2CBus_Zero();
    nData&0x08?I2CBus_One():I2CBus_Zero();
    nData&0x04?I2CBus_One():I2CBus_Zero();
    nData&0x02?I2CBus_One():I2CBus_Zero();
    nData&0x01?I2CBus_One():I2CBus_Zero();
    if (nWaitForAck) I2CBus_wait(nWaitForAck);
    return I2CBus_Ack();
}

BYTE I2CBus_ReadByte(BOOL bLast)
{
    int i;
    BYTE bData=0;
    
    I2C_SetLine(0,1);
    for (i=7; i>=0; i--)
    {
        I2C_SetLine(1,1);
        if (I2C_GetLine())
            bData |= (1<<i);
        I2C_SetLine(0,1);
    }

    bLast ? I2CBus_One() : I2CBus_Zero();
    return bData;
}


BYTE I2CBus_Read(BYTE nAddr)
{
    BYTE bData;
    
    I2CBus_Start();
    I2CBus_SendByte(nAddr,0);
    bData = I2CBus_ReadByte(TRUE);
    I2CBus_Stop();
    return bData;
}

BOOL I2CBus_Write(BYTE nAddr, BYTE nData1, BYTE nData2, BOOL bSendBoth)
{
    BOOL bAck;

    I2CBus_Start();
    I2CBus_SendByte(nAddr,0);
    bAck=I2CBus_SendByte(nData1,0);
    if(bSendBoth) bAck=I2CBus_SendByte(nData2,0);
    I2CBus_Stop();
    return bAck;
}
