/////////////////////////////////////////////////////////////////////////////
// Plugin.c
/////////////////////////////////////////////////////////////////////////////
// Copyright (c) 2000-2002 Quenotte  All rights reserved.
/////////////////////////////////////////////////////////////////////////////
//
//	This file is subject to the terms of the GNU General Public License as
//	published by the Free Software Foundation.  A copy of this license is
//	included with this software distribution in the file COPYING.  If you
//	do not have a copy, you may obtain a copy by writing to the Free
//	Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
//
//	This software is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details
/////////////////////////////////////////////////////////////////////////////
//
// This software was based on MultiDec 5.9 Those portions are
// Copyright (C) 1999/2000 Espresso (echter_espresso@hotmail.com)
//
/////////////////////////////////////////////////////////////////////////////
#include "stdafx.h"

#include "K!TV-XP.h"
#include "plugin.h"
#include "misc.h"
#include "resource.h"
#include "..\Plugins\ResCommun.h"
#include "thread.h"
#include "hardwaremisc.h"

// Pour le plugin audio
#include "mixerdev.h"
#include "audiocapture.h"

#include "DScaler/DS_deinterlace.h"
#include "DScaler/DS_ApiCommon.h"
#include "cpu.h"

int TotalPluginVideo, TotalPluginAudio;
struct ListePlugins ListePluginsVideo[MAX_PLUGINS];
struct ListePlugins ListePluginsAudio[MAX_PLUGINS];
extern HACCEL hAccel;
CRITICAL_SECTION PluginCrit;


// Pluging Vido ////////////////////////////
QuenotteTV_VideoPlugin *PluginVideo;
static tGetVPlugin		GetVPlugin;
BOOL PluginVideoPresent = FALSE;		// Plugin prsent dans le rpertoire courant (les fonctions sont accessibles)
BOOL IsPluginVideoRunning=FALSE;		// Plugin a t charg et tourne
char PlugVideoName[255] = "";
BOOL AutoProcessAudio=FALSE;

// Plugin Audio ////////////////////////////
QuenotteTV_AudioPlugin *PluginAudio;
static tGetPlugin		GetPlugin;
BOOL PluginAudioPresent = FALSE;
BOOL IsPluginAudioRunning=FALSE;
char PlugAudioName[255] = "";
BOOL AutoProcessVideo=FALSE;
BOOL ForcePluginVideo=FALSE;

// Plugin DSCALER///////////////////////////
DEINTERLACE_INFO DIinfo;
int DeInterlace=3;
BOOL IsDScalerPluginRunning=FALSE;
BOOL PluginDScalerPresent=TRUE;
DEINTERLACE_METHOD *DScalerMethod;



// Chargement du plugin de dsentrelacement
void Plugin_DScaler_Charger() 
{
	char mod[256], *TypePlugin = "DScaler";
	HMODULE hPlugInMod;
    GETDEINTERLACEPLUGININFO* pfnGetDeinterlacePluginInfo;
	
	mod[0]=0x00;
	EnterCriticalSection(&PluginCrit);
	__try
	{
		LOG (1, Langue_GetString(ID_PLG_INFO), TypePlugin);
		PluginDScalerPresent=FALSE;
		if(DeInterlace==2) hPlugInMod = LoadLibrary("DI_ScalerBob.dll");
		else if(DeInterlace==3) hPlugInMod = LoadLibrary("DI_Greedy2Frame.dll");
		else if(DeInterlace==4) hPlugInMod = LoadLibrary("DI_TwoFrame.dll");
		else if(DeInterlace==5) hPlugInMod = LoadLibrary("DI_Bob.dll");
		else if(DeInterlace==6) hPlugInMod = LoadLibrary("DI_Weave.dll");
		else {
			LeaveCriticalSection(&PluginCrit);
			return;
		}
		if (hPlugInMod == NULL)
		{
			LOG (1, Langue_GetString(ID_PLG_ERROR_LOAD), TypePlugin);
			DeInterlace=0;
			LeaveCriticalSection(&PluginCrit);
			return;
		}
	    pfnGetDeinterlacePluginInfo = (GETDEINTERLACEPLUGININFO*)GetProcAddress(hPlugInMod, "GetDeinterlacePluginInfo");
		if ( pfnGetDeinterlacePluginInfo == NULL)
		{
			PluginDScalerPresent=FALSE;
			FreeLibrary((HMODULE)hPlugInMod);
			sprintf(mod, Langue_GetString(ID_PLG_ERROR_INIT), TypePlugin);
			RealErrorBox(mod);
			LeaveCriticalSection(&PluginCrit);
			return;
		}

		LOG (1, Langue_GetString(ID_PLG_SUCCES), TypePlugin);
		PluginDScalerPresent=TRUE;
	
		DScalerMethod = pfnGetDeinterlacePluginInfo(CpuFeatureFlags);
	    if(DScalerMethod != NULL)
		{
			DScalerMethod->hModule = hPlugInMod;
			if(DScalerMethod->pfnPluginInit != NULL)
			{
		        DScalerMethod->pfnPluginInit();
			}
		}

	}	// try

    __except (EXCEPTION_EXECUTE_HANDLER  ) 
	{  
		PluginDScalerPresent=FALSE;
		FreeLibrary((HMODULE)hPlugInMod);
		sprintf(mod, Langue_GetString(ID_PLG_ERROR_VER), TypePlugin, "");
		RealErrorBox(mod);
		LeaveCriticalSection(&PluginCrit);
		return;
	}
	LeaveCriticalSection(&PluginCrit);
}


// Dchargement du plugin de dsentrelacement
void Plugin_DScaler_Decharger() 
{
	EnterCriticalSection(&PluginCrit);
	__try
	{
		if (PluginDScalerPresent)
		{
			if(DScalerMethod->pfnPluginExit!=NULL)
			{
				DScalerMethod->pfnPluginExit();
			}
			Sleep(20);
			FreeLibrary((HMODULE)DScalerMethod->hModule);
			Sleep(20);
			PluginDScalerPresent=FALSE;
		}
	}
    __except (EXCEPTION_EXECUTE_HANDLER  ) 
	{  
		LeaveCriticalSection(&PluginCrit);
		return;
	}
	LeaveCriticalSection(&PluginCrit);

}

// Cherche les plugins K!TV prsents dans le rpertoire courant.
void Plugin_Chercher()
{
    WIN32_FIND_DATA FindFileData;
    HANDLE hFindFile;
	int i;

	for (i=0; i<MAX_PLUGINS; i++)
	{
		ListePluginsVideo[i].Description[0] = 0x00;
		ListePluginsVideo[i].Name[0] = 0x00;
		ListePluginsAudio[i].Description[0] = 0x00;
		ListePluginsAudio[i].Name[0] = 0x00;
	}
	TotalPluginVideo = TotalPluginAudio = 0;
	
	Stop_Capture(FALSE);
	
	// On libere les plugins prcdemment chargs en mmoire
	Plugin_Video_Decharger();
	Plugin_Audio_Decharger();

	hFindFile = FindFirstFile("Plugins/*.dll", &FindFileData);

    if (hFindFile != INVALID_HANDLE_VALUE)
    {
        BOOL RetVal = TRUE;
        while (RetVal != 0)
        {
			Plugin_Video_Charger(FindFileData.cFileName);
			if (PluginVideoPresent)
			{
				strcpy(ListePluginsVideo[TotalPluginVideo].Description, PluginVideo->description);
				strcpy(ListePluginsVideo[TotalPluginVideo].Name, FindFileData.cFileName);
				Plugin_Video_Decharger();
				TotalPluginVideo++;
			} 
			else 
			{
				Plugin_Audio_Charger(FindFileData.cFileName);
				if (PluginAudioPresent)
				{
					strcpy(ListePluginsAudio[TotalPluginAudio].Description, PluginAudio->description);
					strcpy(ListePluginsAudio[TotalPluginAudio].Name, FindFileData.cFileName);
					Plugin_Audio_Decharger();
					TotalPluginAudio++;
				}
			}
            RetVal = FindNextFile(hFindFile, &FindFileData);
        }
    }
	FindClose(hFindFile);
	if(strlen(PlugVideoName))
		Plugin_Video_Charger(PlugVideoName);

	if(strlen(PlugAudioName))
		Plugin_Audio_Charger(PlugAudioName);

	Start_Capture(FALSE);
	Affichage_Overlay_Workout(FALSE);
	return;
}

// Chargement du plugin video
void Plugin_Video_Charger(char *PlugVideo) 
{
	char PluginPath[268], mod[256], *TypePlugin = "Vido";
	HINSTANCE hPluginDLL;
	mod[0]=0x00;

	strcpy(PluginPath,"Plugins/");
	strcat(PluginPath,PlugVideo);

	EnterCriticalSection(&PluginCrit);

	__try
	{
		LOG (1, Langue_GetString(ID_PLG_INFO), TypePlugin);
		PluginVideoPresent=FALSE;
		hPluginDLL = LoadLibrary(PluginPath);
		if (hPluginDLL == NULL)
		{
			LOG (1, Langue_GetString(ID_PLG_ERROR_LOAD), PlugVideo);
			LeaveCriticalSection(&PluginCrit);
			return;
		}

		GetVPlugin = (tGetVPlugin)GetProcAddress((HMODULE)hPluginDLL, "GetVPlugin");
		if ( GetVPlugin == NULL)
		{
			FreeLibrary((HMODULE)hPluginDLL);
			sprintf(mod, Langue_GetString(ID_PLG_ERROR_INIT), TypePlugin);
//			RealErrorBox(mod);
			LOG (1, mod);
			LeaveCriticalSection(&PluginCrit);
			return;
		}

		LOG (1, Langue_GetString(ID_PLG_SUCCES), TypePlugin);
		PluginVideoPresent=TRUE;
		PluginVideo= GetVPlugin();
		if((PluginVideo->version) != VIDEO_PLUGIN_VERSION)
		{
			Sleep(50);
			PluginVideoPresent=FALSE;
			IsPluginVideoRunning=FALSE;
			FreeLibrary((HMODULE)hPluginDLL);
			sprintf(mod, Langue_GetString(ID_PLG_ERROR_VER), TypePlugin, PlugVideo);
			RealErrorBox(mod);
			LeaveCriticalSection(&PluginCrit);
			return;
		}
		PluginVideo->hDllInstance = hPluginDLL;
		AutoProcessVideo=PluginVideo->AutoProcessVideo;
	}	// try

    __except (EXCEPTION_EXECUTE_HANDLER  ) 
	{  
		PluginVideoPresent=FALSE;
		FreeLibrary((HMODULE)hPluginDLL);
		sprintf(mod, Langue_GetString(ID_PLG_ERROR_VER), TypePlugin, PlugVideo);
		RealErrorBox(mod);
		LeaveCriticalSection(&PluginCrit);
		return;
	}
	LeaveCriticalSection(&PluginCrit);
}


// Dchargement du plugin video
void Plugin_Video_Decharger() 
{
	EnterCriticalSection(&PluginCrit);
	__try
	{
		if (PluginVideoPresent)
		{
			if (IsPluginVideoRunning) 
				PluginVideo->Stop();
			Sleep(20);
			FreeLibrary((HMODULE)PluginVideo->hDllInstance);
			Sleep(20);
			PluginVideoPresent=FALSE;
			IsPluginVideoRunning=FALSE;
		}
	}
    __except (EXCEPTION_EXECUTE_HANDLER  ) 
	{  
		LeaveCriticalSection(&PluginCrit);
		return;
	}
	LeaveCriticalSection(&PluginCrit);
}


// Chargement du plugin audio
void Plugin_Audio_Charger(char *PlugAudio)
{
	char PluginPath[268], mod[256], *TypePlugin = "Audio";
	HINSTANCE hPluginAudioDLL;
	mod[0]=0x00;

	strcpy(PluginPath,"Plugins/");
	strcat(PluginPath,PlugAudio);

	EnterCriticalSection(&PluginCrit);
	__try
	{
		LOG (1, Langue_GetString(ID_PLG_INFO), TypePlugin);
		PluginAudioPresent=FALSE;
	
		hPluginAudioDLL = LoadLibrary(PluginPath);
		if (hPluginAudioDLL == NULL)
		{
			LOG (1, Langue_GetString(ID_PLG_ERROR_LOAD), PlugAudio);
			LeaveCriticalSection(&PluginCrit);
			return;
		}

		GetPlugin =(tGetPlugin)GetProcAddress((HMODULE)hPluginAudioDLL, "GetPlugin");
		if (GetPlugin==NULL)
		{
			FreeLibrary((HMODULE)hPluginAudioDLL);
			sprintf(mod, Langue_GetString(ID_PLG_ERROR_INIT), TypePlugin);
//			RealErrorBox(mod);
			LOG (1, mod);
			LeaveCriticalSection(&PluginCrit);
			return;
		}

		LOG (1, Langue_GetString(ID_PLG_SUCCES), TypePlugin);
		PluginAudioPresent=TRUE;
		PluginAudio = GetPlugin();
		if((PluginAudio->version) != AUDIO_PLUGIN_VERSION)
		{
			Sleep(50);
			PluginAudioPresent=FALSE;
			FreeLibrary((HMODULE)hPluginAudioDLL);
			sprintf(mod, Langue_GetString(ID_PLG_ERROR_VER), TypePlugin, PlugAudio);
			RealErrorBox(mod);
			LeaveCriticalSection(&PluginCrit);
			return;
		}
		PluginAudio->Init();
		PluginAudio->hDllInstance = hPluginAudioDLL;
	}	// try

	__except (EXCEPTION_EXECUTE_HANDLER  ) 
	{  
		FreeLibrary((HMODULE)hPluginAudioDLL);
		sprintf(mod, Langue_GetString(ID_PLG_ERROR_VER), TypePlugin, PlugAudio);
		RealErrorBox(mod);
		LeaveCriticalSection(&PluginCrit);
		return;
	}
	LeaveCriticalSection(&PluginCrit);
}


// Dchargement du plugin audio
void Plugin_Audio_Decharger() 
{
	EnterCriticalSection(&PluginCrit);
	__try
	{
		if (PluginAudioPresent)
		{
			if (IsPluginAudioRunning)
				SendMessage(hWnd,WM_COMMAND,IDM_PLUGIN_AUDIO,0);				
			Sleep(20);
			FreeLibrary((HMODULE)PluginAudio->hDllInstance);
			Sleep(20);
			PluginAudioPresent=FALSE;
			IsPluginAudioRunning=FALSE;
		}
	}
    __except (EXCEPTION_EXECUTE_HANDLER  ) 
	{  
		LeaveCriticalSection(&PluginCrit);
		return;
	}
	LeaveCriticalSection(&PluginCrit);
}


// Charge les raccourcis claviers dfinis par K!TV dans son fichier RC
//
void InitKTVAccel(HINSTANCE hInstanceNew)
{
	static HINSTANCE hInstance = 0;
	
	if (hInstanceNew) 
		hInstance = hInstanceNew;

	if (!(hAccel = LoadAccelerators (hInstance, "K!TV_ACCEL")))
		RealErrorBox(Langue_GetString(ID_ERROR_NOACCEL));
}


// Aux raccourcis claviers dfinis par K!TV on ajoute ceux du plugin
// 
void AddPluginAccel()
{
	ACCEL* pAccelData = NULL;
	HANDLE hAccelData = NULL;
	int    NbAccel = 1, NbAccelKtv, i;
	BOOL   bPluginAudio, bPluginVideo;

	// Les raccourcis des plugins sont-ils valides ?
	bPluginVideo = PluginVideoPresent && IsPluginVideoRunning && (PluginVideo->accel != NULL) && (PluginVideo->NbAccel > 0);
	bPluginAudio = PluginAudioPresent && IsPluginAudioRunning && (PluginAudio->accel != NULL) && (PluginAudio->NbAccel > 0);

	if (!hAccel || !(bPluginVideo || bPluginAudio))
		return;

	// Nombre de raccourcis dfinis :
	NbAccelKtv = CopyAcceleratorTable( hAccel, NULL, 0 );
	NbAccel    = NbAccelKtv;
	if (bPluginVideo) NbAccel += PluginVideo->NbAccel;
	if (bPluginAudio) NbAccel += PluginAudio->NbAccel;
	
	// allocation d'assez de mmoire pour tout
	hAccelData = GlobalAlloc(GHND, sizeof(ACCEL) * NbAccel);
	if ( hAccelData )
		pAccelData = (ACCEL*)GlobalLock( hAccelData );
	
	if ( !hAccelData || !pAccelData )
		return;

	CopyAcceleratorTable( hAccel, pAccelData, NbAccelKtv );
	DestroyAcceleratorTable( hAccel );
	hAccel = NULL;
	
	// ajoutons les raccourcis du plugin dans les raccourcis de l'application
	for (i = 0; i < NbAccel - NbAccelKtv; i++)
	{
		ACCEL *pCurAccel = NULL, *pPluginAccel = NULL;

		pCurAccel = (ACCEL*)(pAccelData+NbAccelKtv+i);		
		if (bPluginVideo && bPluginAudio) 
		{
			// Plugin vido et audio tournent
			if (i >= PluginVideo->NbAccel)
				pPluginAccel = &PluginAudio->accel[i - PluginVideo->NbAccel];
			else
				pPluginAccel = &PluginVideo->accel[i];
		}
		else if (bPluginVideo)
			// Seul le plugin vido tourne
			pPluginAccel = &PluginVideo->accel[i];
		else if (bPluginAudio)
			// Seul le plugin audio tourne
			pPluginAccel = &PluginAudio->accel[i];

		pCurAccel->fVirt = pPluginAccel->fVirt;
		pCurAccel->key   = pPluginAccel->key;
		pCurAccel->cmd   = pPluginAccel->cmd;
//		LOG (1, "Accel fvirt=%d key=%3d:%c", pCurAccel->fVirt, pCurAccel->key, pCurAccel->key);
	}
	hAccel = CreateAcceleratorTable( pAccelData, NbAccel );		
	GlobalUnlock( hAccelData );
	GlobalFree( hAccelData );
}


void FreePluginAccel()
{
	// Tout supprimer et recharger depuis le fichier RC.
	DestroyAcceleratorTable(hAccel);
	InitKTVAccel(0);

	// Ajouter si l'un des plugin tourne
	if (IsPluginVideoRunning || IsPluginAudioRunning)
		AddPluginAccel();
}


// Le plugin veut-il se lancer automatiquement ?
// 
void ReconnaissanceSignal()
{	
	BOOL AutoID;
	BYTE *VBIdata;
	static int PreviousFrame = -1;
	
	// Si le VBI a dj t dcod nous sortons
	if (PreviousFrame == CurrentFrame)
		return;
	else
		PreviousFrame = CurrentFrame;

	if ( Capture_VBI ) 
		VBIdata = (LPBYTE) pVBILines[CurrentFrame];
	else
		VBIdata = NULL;

	// C'est le plugin qui dcide s'il veut analyser le signal ou non
	if (PluginVideoPresent)
		AutoProcessVideo = PluginVideo->AutoProcessVideo;

	// Synchro plugin audio/plugin video...
	if (PluginAudioPresent)
		AutoProcessAudio = PluginAudio->AutoProcessAudio;

	if ( PluginVideoPresent && AutoProcessVideo && !ChangeCH)
	{
		AutoID = PluginVideo->DetecteSignal(VBIdata);

		if ( (AutoID != IsPluginVideoRunning) && (!ForcePluginVideo) )
		{
			if ( AutoID )
				PostMessage(hWnd, WM_COMMAND, IDM_PLUGIN_VIDEO_ON, 0);
			else 
				PostMessage(hWnd, WM_COMMAND, IDM_PLUGIN_VIDEO_OFF, 0);	
		}
	}
}


BOOL Plugin_Video_Demarrer()
{
	// Si le plugin n'est pas trouv, ne rien faire !
	if (!PluginVideoPresent) return FALSE;
	
	EnterCriticalSection(&PluginCrit);
	Stop_Capture(OutDib);
	if ( !IsPluginVideoRunning )
	{
		if ( AutoProcessAudio && !IsPluginAudioRunning )	
			SendMessage(hWnd,WM_COMMAND,IDM_PLUGIN_AUDIO,0);

		VIA_FIX   = PluginVideo->VIA_FIX;
		VIA_FIX_X = PluginVideo->VIA_FIX_X;
		if (!PluginVideo->Start()) 
			IsPluginVideoRunning=FALSE;
		else 
			IsPluginVideoRunning=TRUE;
		AddPluginAccel();
		UpdateMenuBar();
		Affichage_Init_Screen_Struct();
 	}
	Start_Capture(OutDib);
	LeaveCriticalSection(&PluginCrit);
	return TRUE;
}


BOOL Plugin_Video_Stopper()
{
	// Si le plugin n'est pas trouv, ne rien faire !
	if (!PluginVideoPresent) return FALSE;
	
	EnterCriticalSection(&PluginCrit);
	Stop_Capture(OutDib);
	ForcePluginVideo=FALSE;
	CheckMenuItem( GetMenu(hWnd), IDM_PLUGIN_VIDEO_FORCE, MF_UNCHECKED );
	if (IsPluginVideoRunning)
	{
		PluginVideo->Stop();
		Sleep(50);
	}

	IsPluginVideoRunning=FALSE;
	FreePluginAccel();

	Affichage_Init_Screen_Struct();
	Start_Capture(OutDib);
	UpdateMenuBar();
	if(IsPluginAudioRunning)	SendMessage(hWnd,WM_COMMAND,IDM_PLUGIN_AUDIO,0);
	LeaveCriticalSection(&PluginCrit);
	return TRUE;
}

void Plugin_Audio_StartStop()
{
	if (!PluginAudioPresent) 
		return;

	if(!System_In_Mute)
	{
		if ( !IsPluginAudioRunning )
		{
			IsPluginAudioRunning=TRUE;
			AddPluginAccel();
			Mixer_Mute();
			SoundCapture_Open();
			SoundCapture_Play() ;
		}
		else
		{
			IsPluginAudioRunning=FALSE;
			FreePluginAccel();
			Mixer_UnMute();
			SoundCapture_Close();
			SoundCapture_Stop();
		}
		UpdateMenuBar();
	}
	else
	{
		IsPluginAudioRunning=FALSE;
		SoundCapture_Close();
		SoundCapture_Stop();
	}
}