/////////////////////////////////////////////////////////////////////////////
// Timer.cpp
/////////////////////////////////////////////////////////////////////////////
// Copyright (c) 2000-2002 Quenotte  All rights reserved.
/////////////////////////////////////////////////////////////////////////////
//
//	This file is subject to the terms of the GNU General Public License as
//	published by the Free Software Foundation.  A copy of this license is
//	included with this software distribution in the file COPYING.  If you
//	do not have a copy, you may obtain a copy by writing to the Free
//	Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
//
//	This software is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details
/////////////////////////////////////////////////////////////////////////////
//
// This software was based on MultiDec 5.9 Those portions are
// Copyright (C) 1999/2000 Espresso (echter_espresso@hotmail.com)
//
/////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "resource.h"
#include "..\Plugins\ResCommun.h"

#include "K!TV-XP.h"
#include "misc.h"
#include "mixerdev.h"
#include "procs.h"
#include "osd.h"
#include "plugin.h"
#include "bt848.h"
#include "btaudio.h"
#include "vbi-vt.h"
#include "hardwaremisc.h"
#include "IR_haup.h"
#include "VBI_VideoText.h"
#include "settings.h"
#include "nextview.h"

extern int mousewheel;
extern int MoveXDist;
extern int MoveYDist;
extern BOOL bShowCursor;


void Cursor_SetVisibility(BOOL bVisible);

// Gestion des timers de K!TV
void OnTimer(UINT wParam)
{
	char Text1[255], Text2[255];
	static HWND sNXThWnd=NULL;

	switch(wParam) 
	{

	case TIMER_CHAR: // Gestion du pave numerique
		int i;

		KillTimer(hWnd,TIMER_CHAR);
        i=atoi(ChannelString);
		ChannelString[0]=0x00;

		if (VTState!=VT_OFF) // Affichage de la page teletexte dont le numero a ete rentre via le pave numerique
        {
			SetVTPage(i, 0, FALSE, FALSE);
        }
		else  // Changement de canal/preselection dont le numero a ete rentre via le pave numerique
		{
			i-=1;
			if (( i >= 0 ) && ( i < MAXPROGS ))
			{
				if ( Program[i+1].Active )
					ChangementCanal(i+1);
			}
			else	
				OSD_ShowText_Top(hWnd,"");
		}
		break;

	case TIMER_FOREGROUND:	// Mise en Avant plan et recuperation du focus...
		KillTimer(hWnd, TIMER_FOREGROUND);
		if(bNXTVactive)	// Si prise en charge NexTView activee
		{
			EnumWindows(NexTView_Chercher,0);	//On recherche la fenetre de NexTView...
			if(NXThWnd==NULL)	/// On ne l'a pas trouvee...
			{
				sNXThWnd=NULL;	// On reinitialise la valeur sauvegardee HWND de NexTView...
				SetTimer(hWnd, TIMER_FOREGROUND, 50, NULL);	// On recommence alors dans 50ms...
			}
			else	//On a trouve la fenetre NexTView...
			{
				if(sNXThWnd==NULL)	SetTimer(hWnd, TIMER_FOREGROUND, 50, NULL);	// Si c'est la premiere fois qu'on la trouve on recommence l'operation au cas ou cela n'aurait pas ete fait correctement...
				else SetForegroundApp(FALSE);	// Sinon on remet K!TV XP en avant plan
				sNXThWnd=NXThWnd;	// On sauve la valeur de HWND de NexTView...
			}
		}
		break;

	case TIMER_CHANNEL: // Commutation automatique format MSP 34XX
		KillTimer(hWnd, TIMER_CHANNEL);

		// Si carte avec MSP34XX et format son Nicam Secam L avec commutation Nicam/AM
		// On attend que le MSP34XX ait commute pour afficher le format (Nicam stereo ou AM mono)
		// On utilise des Sleep courts et plus frequents pour ne pas provoquer un gel de l'image
		// et/ou un blocage du soft pendant l'attente (en multithread entre chaque sleep on rend la main)
		if(Has_MSP && MSPMode==7)
		{
			Audio_MSP_Watch_Mode();
			OSD_ShowText_TopMini(hWnd,Text,0,FALSE);
		}
		OSD_ShowText_Top( hWnd,Program[CurrentProgram].Name );
        break;
		
	case TIMER_CHANNELCHANGE: // Changement de canal... On attend pour que les donnees teletexte arrivent
		ChangeCH=FALSE;
		KillTimer(hWnd,TIMER_CHANNELCHANGE);
        break;

	case TIMER_IR: // On regarde si on a recu des infos par le port IR des cartes hauppauge
		IR_Hauppauge_Read_Key(FALSE);
        break;

	case TIMER_LBUTTONUP: // Bouton gauche de la souris relache
		if(!LButton.dbclk)
		{
			if (VTState!=VT_OFF) // Si teletexte on se deplace sur la page cliquee si elle est valide
			{
				int NumPage;

				NumPage = VT_GetPageNumberAt(VTPage - 100, VTSubPage, LButton.x, LButton.y);
				if ((NumPage >= 100) && (NumPage <= MAX_VT_PAGES)) 
				{
					SetVTPage(NumPage, 0, FALSE, FALSE);
					MoveXDist=0;
				}
			}
			if ( MoveXDist == -1 ) // Sinon et si on ne deplace pas la fenetre:
			{
				if (!bIsFullScreen) // Si on n'est en plein ecran, commutation avec/sans menus
					PostMessage(hWnd,WM_COMMAND,IDM_TOGGLE_MENU,0);
			}
		}
		else
		{
			LButton.dbclk=FALSE;
			PostMessage(hWnd,WM_COMMAND,IDM_FULLSCREEN,0);
		}
		MoveXDist=-1;
		MoveYDist=-1;
		KillTimer(hWnd,TIMER_LBUTTONUP);
		break;

	case TIMER_MOUSEWHEEL: // Roulette de la souris
		if(VTState!=VT_OFF) // Si teletexte on change la sous page
		{
			if(mousewheel==1)	SendMessage(hWnd,WM_COMMAND,IDM_VT_PAGE_UP,0);
			else if(mousewheel==-1)	SendMessage(hWnd,WM_COMMAND,IDM_VT_PAGE_DOWN,0);
		}
		else // Sinon on change de canal/preselection
		{
			if(mousewheel==1)	SendMessage(hWnd,WM_COMMAND,IDM_CHANNELPLUS,0);
			else if(mousewheel==-1)	SendMessage(hWnd,WM_COMMAND,IDM_CHANNELMINUS,0);
		}
		mousewheel=0;
		KillTimer(hWnd,TIMER_MOUSEWHEEL);
        break;
		  
	case TIMER_MOVIEBOOST: // Gestion des bandes noires
		if ( !IsPluginVideoRunning )
		{
			if (( AutoMovieBoostEnable ) && ( MovieBoost != 0 ))
			{
				Stop_Capture(FALSE);
				MovieBoost = 0;
				AutoMovieBoostEnable=FALSE;
				Affichage_Overlay_Workout(FALSE);
				Start_Capture(FALSE);
			}
		}
		else
		{
			if ( bAutoMovieBoost )
			{
				PluginVideo->MovieBoost = MovieBoost;
				if ( PluginVideo->AutoMovieBoostFound() && ( MovieBoost != PluginVideo->MovieBoost ) )
				{
					Stop_Capture(FALSE);
					MovieBoost = PluginVideo->MovieBoost;
					AutoMovieBoostEnable=TRUE;
					Affichage_Overlay_Workout(FALSE);
					Start_Capture(FALSE);
				}
			}
		}
        break;

	case TIMER_MSP: // Comutation automatique format MSP 34XX
		if ( AutoStereoSelect ) Audio_MSP_Watch_Mode();
        break;

	case TIMER_SHUTDOWN: //  gestion de l'arret de l'application
		GetLocalTime(&LocalTime);
			  
        if (( LocalTime.wHour == Shutdown.H ) && ( LocalTime.wMinute == Shutdown.M )) // Heure d'arret atteinte
		{
			bShowMenu = TRUE;
			if (bIsFullScreen)
				SendMessage(hWnd,WM_COMMAND,IDM_FULLSCREEN,0);

			if(bMinimized) ShowWindow(hWnd,SW_RESTORE); // Si fenetre iconifiee on la restaure a sa taille precedente...
			if(bMaximized) ShowWindow(hWnd,SW_RESTORE); // Si fenetre maximisee on la restaure a sa taille nominale...
			Shutdown.M = 0xff;
			Shutdown.H = 0xff;
			strcpy(Shutdown.Text, Langue_GetString(ID_SHTDWN));
            sprintf(Text, Langue_GetString(ID_SHTDWN_TIME), LocalTime.wHour, LocalTime.wMinute);
			strcat(Shutdown.Text, Text);
			Langue_DialogBox(hInst,"EXTINCTION",NULL,Stop_SystemProc);
		}
        break;

	case TIMER_STATUS: // Gestion de l'affichage de certaines infos dans la barre de status
		if ( !VideoPresent() )
		{
			StatusBar_Display(Langue_GetString(ID_NOSIGNAL));
			return;
		}
		Text1[0]=0x00;

		if(VideoSource!=0) // Si la source video n'est pas le tuner on affiche la source video
			strcpy(Text1,ChaineVideoSource[VideoSource]);
		else
		{
			if ( VT_GetStation() != 0x00 ) // On affiche les informations PDC/TDSP 
			{
				sprintf(Text1,"%s ", VT_GetStation());
			} 
			else if ( VPSLastName[0] != 0x00 ) // On affiche les informations VPS
			{
				sprintf(Text1,"%s ",VPSLastName);
				VPSLastName[0] = 0x00; 
			}
				  
			if ( IsPluginVideoRunning ) // On affiche des infos du plugin video
			{	
				PluginVideo->GetStatus(Text2,FALSE);
				strcat(Text1,Text2);
			}
		}
  		StatusBar_Display(Text1);
			  
		if ( System_In_Mute ) 
			StatusBar_Display(Langue_GetString(ID_MUTE));
		Text1[0]=0x00;
		Text2[0]=0x00;

		if(VTState!=VT_OFF) // Affichage pour savoir si l'affichage du Teletexte est effectue
		{
			strcat(Text1, "TT");
		}

		if(Sous_Titres.bActif) // Affichage pour savoir si l'affichage des sous-titrages est effectue
		{
			if(VTState!=VT_OFF)	strcat(Text1, "+");
			strcat(Text1, "ST");
			if(Sous_Titres.STPages[Sous_Titres.PageActive]!=0)
			{
				sprintf(Text2,"(%d)",Sous_Titres.STPages[Sous_Titres.PageActive]);
				strcat(Text1,Text2);
			}
		}
		if ( IsPluginVideoRunning ) // Affichage d'infos sur le plugin
		{
			if(Sous_Titres.bActif ||(VTState!=VT_OFF))	strcat(Text1, "+");
			PluginVideo->GetStatus(Text2,TRUE);
			strcat(Text1,Text2);
		}

		if(DeInterlace!=0 && ModeOverlay) // Affichage d'infos sur le plugin de desentrelacement
		{
			if ( IsPluginVideoRunning )
				strcat(Text1, "+");
			else if( (Sous_Titres.bActif) || (VTState!=VT_OFF) )	strcat(Text1, "+");
			if(DeInterlace==1) // Dsentrelacement Hardware 1
				strcat(Text1,"DI-HW1");
			else if(DeInterlace==2) // Dsentrelacement Hardware 2
				strcat(Text1,"DI-HW2");
			else	strcat(Text1,DScalerMethod->szName); // Dsentrelacement DScaler
		}
		StatusBar_Update(2, Text1);
        break;

	case TIMER_ST: // Rafraichissement des infos des pages de sous-titres
		{
		static int STCount=0;
		static int PageActive=0;

		if(Sous_Titres.bRun && Sous_Titres.STCount==STCount && Sous_Titres.PageActive==PageActive)
		return;

		if(!Sous_Titres.bTeleTexte)
		{
			if(Sous_Titres.STCount==0)
			{
				Sous_Titres.bRun=FALSE;
				KillTimer(hWnd,TIMER_VTUPDATE);
			}
			else
			{
				if(Sous_Titres.PageActive>Sous_Titres.STCount-1)
					Sous_Titres.PageActive=Sous_Titres.STCount-1;
				Sous_Titres.bRun=TRUE;
				VTPage=Sous_Titres.STPages[Sous_Titres.PageActive];
				VTSubPage=0;
	
				SetVTPage(VTPage, VTSubPage, TRUE, FALSE);
				KillTimer(hWnd,TIMER_VTUPDATE);
				SetTimer(hWnd, TIMER_VTUPDATE, TIMER_VTUPDATE_MS, NULL);
			}
			if(Sous_Titres.STCount!=STCount)
			{
				InvalidateRect(hWnd,NULL,FALSE);
				Affichage_Overlay_Workout(FALSE);
			}
			PageActive=Sous_Titres.PageActive;
			STCount=Sous_Titres.STCount;
		}
		}
		break;

	case TIMER_VISIBILITY:	 // Visibilite de la souris
		KillTimer(hWnd, TIMER_VISIBILITY);
        if (!bInMenuOrDialogBox)
        Cursor_SetVisibility(FALSE);
        break;
     
	case TIMER_VTFLASHER: // Flashing du teletexte
		VT_Refresh(TRUE);
		break;

	case TIMER_VTUPDATE: // Rafraichissement du teletexte
		VT_Refresh(FALSE);
		break;

	case OSD_TIMER_ID:
	case OSD_TIMER_ID_TOP:
	case OSD_TIMER_ID_TOP_MINI:
	case OSD_TIMER_ID_CENTER:
		// Nettoyage de l'OSD si necessaire
		OSD_Clear(hWnd,wParam-OSD_TIMER_ID);
		break;

	default :
		if ( wParam >= 200 )
			KillTimer(hWnd, wParam);
		break;

	}
	return;
}


// Reglage de la visibilite du curseur de la souris
void Cursor_SetVisibility(BOOL bVisible)
{
    static int nCursorIndex = 1;
    if(bVisible)
    {
        while(nCursorIndex < 1)
        {
            ShowCursor(TRUE);
            nCursorIndex++;
        }
    }
    else
    {
        while(nCursorIndex > 0)
        {
            ShowCursor(FALSE);
            nCursorIndex--;
        }
    }
}


// Gestion de la visibilite du curseur de la souris
void Cursor_UpdateVisibility()
{
    KillTimer(hWnd, TIMER_VISIBILITY);
    if (bInMenuOrDialogBox)
    {
        Cursor_SetVisibility(TRUE);
    }
    else if (!bAutoHideCursor)
    {
		if (bIsFullScreen && VTState==VT_OFF)
            Cursor_SetVisibility(FALSE);
        else
            Cursor_SetVisibility(bShowCursor);
    }
    else
     {
        Cursor_SetVisibility(TRUE);
        SetTimer(hWnd, TIMER_VISIBILITY, 5000, NULL);
    }
}